
	/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
	//                                             	   
    // Trade, Quality Upgrading and Input Linkages: A Theory with Evidence from Colombia (Cecilia Fieler, Marcela Eslava, Daniel Xu), August 2017*/

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
	//                                             	   
    // Reading raw plant-level database. 
    // Input file: d53.a8801 (raw database) [SAS FILE]
    // Output file: EAM88_EFX               [STATA FILE]
    //
	//////////////////////////////////////////////////////////*/


/* Modify routes to following databases to the relevant subfolder where DANE saved it into your DANE user. Files eam82.dat, eam83.dat, etc, are the plant-level EAM files that you should have obtained from DANE*/

Filename  eam82  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\eam82.dat');
Filename  eam83  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\eam83.dat');
Filename  eam84  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\eam84.dat');
Filename  eam85  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\eam85.dat');
Filename  eam86  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\eam86.dat');
Filename  eam87  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\eam87.dat');
Filename  eam88  ('\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\Primary Databases\Eam\PLANOS EAM 1982_1991\d53.a8801');
options symbolgen; options macrogen;

/*input: d53.a8801'

output :Basefinal8891.dta*/

 /*Reading the raw database (EAM-1988)*/

%macro year(firstyear,lastyear); 

%do i=&firstyear %to &lastyear;
 
DATA eam&i (COMPRESS=ON);
 INFILE eam&i lrecl=1250;
  INPUT 
/*Variables generales*/
  DPTO 1226-1227 CIIU5 1-5  NORDEN 13-18
 
/*Personal Ocupado*/
 /*Total*/
  PEROC 19-23  TOTSAL 164-173 TOTPREST 234-243 
 /*Dueos*/
  PERDUENOH 24-28  PERDUENOM 64-68 
 /*Directivos*/
  PERDIRH 29-33 PERDIRM 69-73 DIRSAL 104-113 DIRPREST 174-183 
/*Empleados(no incluye tecnicos)*/
  PEREMPLH 34-38   PEREMPLM 74-78 EMPLSAL 114-123  EMPLPREST 184-193
/*Tcnicos nacionales*/
  PERTECNH 39-43 PERTECNM 79-83 TECNSAL 124-133 TECNPREST 194-203
/*Tecnicos extrangeros */
  PERTECEH 44-48 PERTECEM 84-88 TECESAL 134-143 TECEPREST 204-213
/*Obreros*/
  PEROBRH 49-53 PEROBRM 89-93 OBRSAL 144-153 OBRPREST 214-223  
/*Aprendices*/
  PERAPRENTH 54-58  PERAPRENTM 94-98 APRENTSAL 154-163 APRENTPREST 224-233


/*Otras variables*/
  PROBRUTA 1184-1193  TIMPUIND 1054-1063
  CONINTER 1194-1203  CENERCON 294-303  GASTGENE 474-483
  TINVACFN 604-613    TINVACFU 664-673  TINVACUP 714-723
  TDEPRECI 884-893    TACTFIJO 944-953  TVTAACFI 834-843
  VGASINDU 374-383    MATPRNEX 954-963
  PROTOTAL 974-983
  VALORCONSEXT 964-973
  VALORVEN 1134-1143    VALORCOM 1164-1173
  VALORVENNAL 1114-1123 VALORCOMNAL 1144-1153
  VALORVENEXT 1124-1133 VALORCOMEXT 1154-1163

/*ACTIVOS*/
/*Compras: nuevos(cn) y usados(cu) y producidos para uso propio(prod)*/
                      terrenos_cu   614-623                        terrenos_r   724-733
edificios_cn  564-573 edificios_cu  624-633  edificios_cup 674-683 edificios_r  734-743
maquinaria_cn 574-583 maquinaria_cu 634-643 maquinaria_cup 684-693 maquinaria_r 744-753
eq_transp_cn  584-593 eq_transp_cu  644-653 eq_transp_cup  694-703 eq_transp_r  754-763
equipo_cn     594-603 equipo_cu     654-663 equipo_cup     704-713 equipo_r     764-773
			
/*Ventas*/
terrenos_vrt   784-793  
edificios_vrt  794-803 
maquinaria_vrt 804-813 
eq_transp_vrt 814-823
equipo_vrt     824-833 

/*Saldo final*/
terrenos_sf   894-903  
edificios_sf  904-913 
maquinaria_sf 914-923 
eq_transp_sf 924-933 
equipo_sf     934-943 
/*Compras (generate the pruchases variables)*/
terrenos_c=terrenos_cu;    
edificios_c=sum(edificios_cn,edificios_cu,edificios_cup);  
maquinaria_c=sum(maquinaria_cn, maquinaria_cu, maquinaria_cup); 
eq_transp_c=sum(eq_transp_cn, eq_transp_cu, eq_transp_cup); 
equipo_c=sum(equipo_cn,equipo_cu,equipo_cup);     
/*Otras*/
PBRUTA=PROBRUTA-TIMPUIND;
CONSUMO=CONINTER-TIMPUIND;
VALORAGR=PBRUTA-CONSUMO;
GASTOSGE=GASTGENE+TDEPRECI;
IBRUTA=TINVACFN+TINVACFU+TINVACUP-TVTAACFI;
INETA=IBRUTA-TDEPRECI;
ACTI3=INT(CIIU5/100);
ACTIV=INT(CIIU5/10);
year=19&i;

%end;

%mend;
%year(82,88)

DATA eam8288;
set eam82 eam83 eam84 eam85 eam86 eam87 eam88;
run;

PROC EXPORT DATA= eam8288
            OUTFILE= "\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA\FEX_AER_programs\Results\db\EAM8288_EFX.dta" 
            DBMS=STATA REPLACE;
/*Change the above route so the file is saved to your DANE user rather than Marcela Eslava's*/

RUN;
