
*=============================================================================================================================================
* Trade, Quality Upgrading and Input Linkages: A Theory with Evidence from Colombia (Cecilia Fieler, Marcela Eslava, Daniel Xu), August 2017
* Reading database, adding 1994,  and generating tables
*=============================================================================================================================================

	
    *>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  Program's Parameters <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	pause on
    set more off, perm
	clear
	clear matrix
	set mem 50m
	
*===========================================================================================================
*In the following globals, change \TC1_MARCELA ESLAVA to the user assigned to you by DANE
*=======================================================================================================	
	global prog_dir "Y:\TC1_MARCELA ESLAVA\FEX_AER_programs"
	
	global logfile_dir "Y:\TC1_MARCELA ESLAVA\FEX_AER_programs\logs"
	cap mkdir "$logfile_dir"
	
	global direct_results "Y:\TC1_MARCELA ESLAVA\FEX_AER_programs\Results"
   	 cap mkdir "$direct_results" 
    
	global other "Y:\TC1_MARCELA ESLAVA\FEX_AER_programs\external databases"

	global aran "Y:\TC1_MARCELA ESLAVA\FEX_AER_programs\external databases"
	
*===========================================================================================================
*In the following global, change route to that where DANE saved original databases for individual materials and outputs at plant level
* (Plant-product or Plant-input level databases)
*=======================================================================================================	
	
global pym "Y:\TC1_MARCELA ESLAVA\Primary Databases\Eam\Productos y materiales"

*===========================================================================================================
*In the following cd statement, change route to that where DANE saved original databases for the plant-level information
*=======================================================================================================	

cd "\\bdatos_server\SALA DE PROCESAMIENTO\TC1_MARCELA ESLAVA/Primary Databases\"	
	

*===========================================================================================================
*From this point on, you do not need to make more changes
*=======================================================================================================	
	
	global cd $direct_results
	
	
	*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   
   
	*=========================
	* I. Read 1994 AMS data and create plant-level database merging with 1982-1988
	*========================	
	do "$prog_dir\2.0. Eam_reducida_GetYear1994.do"
	do "$prog_dir\2.1. Setting AMS_EFX.do"

	*=================================
	* II.  Create additional variables from raw AMS variables
	*=================================
	
	do "$prog_dir\2.2. Additional variable creation.do"

	*===========================================================
	* III. Create product level database (Input and output prices at input and output level)
	*===========================================================
	
	do "$prog_dir\2.3. Product level database.do"
	
	*==========================================
	* IV.  Create Tariffs at plant level from tariffs at the product level
	*==========================================
	
	do "$prog_dir\2.4 Tariffs.do"
	
	*==========================================
	* V.  Create quality measures: q
	*==========================================

	do "$prog_dir\2.5 q Measures.do"
	
	*==========================================
	* VI.  Generate Moments Tables: 7, 8, 10, 13
	*==========================================
	
	do "$prog_dir\2.6 Tables 7 8 10 13.do"
	* Notice: because of changes in the numbering of tables, internally the program refers to tables 4, 5, 7 and 10. This correspond, respectively, to table 7, 8, 10, and 13 in the published paper.	
		
	*==============================================================================================================================
	* VII.  TABLE 9:  Input and output prices vs. skill and quality
	*==============================================================================================================================
	
	do "$prog_dir\2.7 Table 9.do"
		
	*==============================================================================================================================
	* VIII.  TABLE 11:  Import behavior and skill intensity
	*==============================================================================================================================

	do "$prog_dir\2.8 Table 11.do"	
	
	*==============================================================================================================================
	* IX. TABLE 12:  Within-firm changes and input tariffs, panel regressions
	*==============================================================================================================================
    
	do "$prog_dir\2.9 Table 12.do"
	
	*==============================================================================================================================
	* X. TABLES A3 & A4:  Moments by sectors
	*==============================================================================================================================
    
	do "$prog_dir\2.10 Tables A3 & A4.do"
	
	*==============================================================================================================================
	* XI. TABLES A1 & A2:  Tariffs and skills
	*==============================================================================================================================
    
	do "$prog_dir\2.11 Tables A1 & A2.do"
	
	*==============================================================================================================================
	* XII. TABLES 7, 8, 10, 13 & C3 BOOTSTRAPPED
	*==============================================================================================================================
    
	do "$prog_dir\2.12 Tables 7 8 10 13 boot.do"	
	
	*==============================================================================================================================
	* XIII.  Erase Project Databases
	*==============================================================================================================================
	
	cap erase "$cd\db\EAM8288_EFX.dta"
	cap erase "$cd\db\EAM8288_EFX_Final.dta"
	cap erase "$cd\db\norden_nordest.dta"
	cap erase "$cd\db\EAM8288&94_EFX_Final.dta"
	cap erase "$cd\db\mat_price_prodlev8288_94.dta"
	cap erase "$cd\db\out_price_prodlev8288_94.dta"
	cap erase "$cd\db\aran_ciiu.dta"
	cap erase "$cd\db\input_tariff8288_94.dta"
	cap erase "$cd\db\input_tariff8288.dta"
	cap erase "$cd\db\output_tariff8288_94.dta"
	cap erase "$cd\db\dshift.dta"
	cap erase "$cd\db\dshift_94.dta"
	
	*===========================
 
