	
    clear all                                           		
	
	pause on
	
	cap log close
	log using "$logfile_dir\2.1 Setting AMS_EFX(`c(current_date)').log", replace

	*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	
    *=============================================================
    *  I. RENAME VARIABLES FOR 1988 & 1994 AND APPEND DATABASES
	*=============================================================
    
	*--------------------
	* For Year 1982-1988
	*--------------------
	use "$cd/db/EAM8288_EFX.dta", clear
	
	rename 	prototal		vq
	rename 	valorven		tsales
	rename 	valorvenext		Sales_ext
	egen    L_adm=rowtotal(perdirh perdirm)
	rename 	peroc	   		wrk
	rename 	coninter		vm
	rename 	cenercon econ_q
	egen 	payroll=rowtotal(totsal totprest)
	gen ciiu3_r2=floor(ciiu5/100)
	gen ciiu_r2=floor(ciiu5/10)
	replace ciiu3_r2=311 if ciiu3_r2==312 //sectors 311 y 312, both related to food processing , are combined into a unified sector code: 311	
	ren ciiu5 ciiu5_r2
	rename  totsal wage
	rename  totprest welfare
	
	label var ciiu3_r2 "Ciiu rev2 3digits"
    label var econ_q "energy consumed quantity"	
	label var vm "raw material consumption"	
	label var vq "gross value of production"	
	
	saveold "$cd/db/EAM8288_EFX_Final.dta", replace
	*------------------------------------
	*	For Year 1994
    *------------------------------------

	use "$cd/db\1994_reducida_EFX.dta", clear 

	rename 	valpro	 			vq
    label var vq "gross value of production"	
	rename 	valorven 			tsales
	rename 	per_tot				wrk
	rename 	materiales			vm
	label var vm "raw material consumption"	

    save "$cd/db\1994_reducida_EFX.dta", replace
	
    *-----------------------------------------------------------
	*	 3. Append the 1982-1988 and the 1994 databases
	*-----------------------------------------------------------
    use "$cd/db/EAM8288_EFX_Final.dta", clear
	append using "${cd}\db\1994_reducida_EFX.dta"
	erase "$cd/db\1994_reducida_EFX.dta"
	
   *	Fix nordest
   *	------------	
   *	This is done because nordest will be used as a key for merging other databases and havving missings doesn't allow it.

	replace nordest=norden*(-1) if nordest==. & year<1992	
	
	*=============================================================
    *  II. DESCRIBE ORIGINAL SECTORS IN DATABASE & FIX 1994 VARIABLES
	*=============================================================
	
	*-------------------------------
	*	Fix the 1992-1994 variables 
    *-----------------------------
	/*turn missings into zeros for individual wage and or worker items (specific categories are usually 
	left as missings if the plant reports no workers of that specific type) we do this conditional 
	on the respective total (wage bill or number of employees) not being missing or zero*/
	
	foreach var of varlist valci7*{
	*qui replace `var'=0 if `var'==. & valci74_7_40!=0 & valci74_7_40!=. & year>1991 & year<1994
	qui replace `var'=0 if `var'==. & valci74_7_37!=0 & valci74_7_37!=. & year==1994
	}

	foreach var of varlist  valci8*{
	qui replace `var'=0 if `var'==. & wrk!=0 & wrk!=. & year>1991 & year<1995
	}
	
	
	*=============================================================
    *  III. READING PPI
	*=============================================================
	
		cap n drop _merge
		merge m:1 year using "$other\ippm_09_8209.dta"

		sum ippm_09 if year==1988
		gen ippm_88=ippm_09/r(mean)
		drop if _merge==2
		drop _merge

    *=============================================================
    *  IV. REMOVE SECTORS WHERE A SINGLE PUBLIC OR SEMI-PUBLIC COMPANY DOMINATED FOR DECADES AND MERGE LONGITUDINAL IDENTIFIERS
	*=============================================================

*	Eliminating the observations that belong to coffee treshing sector
*	-------------------------------------------------------------------

	drop if year<1992 & (ciiu5==31212 | ciiu5==31168)
	drop if year==1994 & (act94==31212 | act94==31168)


*	Eliminating the observations that belong to oil sector
*	-------------------------------------------------------
	drop if (ciiu_r2==3530 | ciiu_r2==3540) & year<1997	 
	
		
*	Nordest - Norden correlatives (Longitudinal identifiers)
*	-------------------------------------------------------	
	
	preserve
	use "$cd/db/norden_nordest.dta", clear
	drop nordest
	keep if year<1989
	save "$cd/db/norden_nordest_r.dta", replace
	restore
	
	merge m:1 norden year using "$cd/db/norden_nordest_r.dta"
	drop if _merge==2
	replace nordest=nordest2 if nordest2!=.
	replace nordest= -norden if nordest==.
	drop if year==0
	
    *=============================================================
    *  V. MERGE WITH PRICE INDEX
	*=============================================================
	
	mmerge year nordest using "$prog_dir\prices\temp_database\Final_Input_deflo_me.dta", type(1:1) uif(inrange(year,1982,1988) | year==1994)
	rename (lnp P_ind) (lnp_in P_ind_in)
	drop if _merge==2
	mmerge year nordest using "$prog_dir\prices\temp_database_outp\Final_Output_deflo_me.dta", type(1:1) uif(inrange(year,1982,1988) | year==1994)
	rename (lnp P_ind) (lnp_out P_ind_out)
	drop if _merge==2

	
    *=============================================================
    *  VI. SAVE
	*=============================================================
	keep if inrange(year,1982,1988) | year==1994
	sum 
	saveold "$cd/db\EAM8288&94_EFX_Final.dta", replace 

	cap log close

	
