	cap n log close
	log using "$logfile_dir\2.2 Additional variable creation (`c(current_date)').log", replace

*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  Program's Parameters <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

clear all
set mem 50m
	
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   

    *=============================================================
    *  I. CALL DATABASE AND PERFORM SIMPLE DESCRIPTION
	*=============================================================
	use "$cd/db\EAM8288&94_EFX_Final.dta", clear
	ren  ciiu_r2 sic
	tab sic
		

	*========================
	*II. CREATE VARIABLES
	*========================
	

		*---------------------------------	
		*	A. Exports variable creation
		*---------------------------------
		
			*i. Eam only
			gen     esales=Sales_ext if year<1992
			replace esales=0 if esales==. & year<1992  //  if the exports is "." we input 0
			label var esales "Exports in thousand of pesos(AMS)" 
			table year, c(count esales mean esales sum esales min esales max esales) 
			
			*	Relative measure creation - firm level.
			
			*iii. Export share (EAM only)	
			gen shexp = esales/tsales
			label var shexp "Exports share of total sales (EAM)"
			sum shexp, det
			replace shexp=. if shexp>=15
			
		*---------------------------------	
		*	B. Imports variable creation
		*---------------------------------

		*	- Purchases
			gen vfm_purch=valorcomext if year<1992
			replace vfm_purch=0 if vfm_purch==. & year<1992
			label var vfm_purch "Imports (purchases) in thousand of pesos (AMS)"

		*	- Consumption	
			gen vfm=valorconsext if year<1992
			replace vfm=0 if vfm==.  & year<1992
			label var vfm "Imports (consumption) in thousand of pesos (AMS)"	
			
		*	Relative measure creation - firm level.

		*	- Consumption 	
			gen shimp=vfm/vm
			label var shimp "Imports share of total c (AMS)"

		*	- Purchases 	
			gen shimp_purch=vfm_purch/vm
			label var shimp_purch "Imports share of total sales (AMS)"
			

	*------------------------	
	* Payroll
	*------------------------

	gen pay_swide=(emplsal+emplprest+dirsal+dirprest+tecesal+tecnsal+teceprest+tecnprest) if year<1992
	replace pay_swide=valci73_7_10+valci73_7_20+valci73_7_29+valci73_7_36+valci72_7_10+valci72_7_20+valci72_7_29+valci72_7_36 if year==1994
	
	gen wage_u=obrsal
	gen welfare_u=obrprest
	gen pay_u=(wage_u+welfare_u) if year<1992
	replace pay_u=valci71_7_10+valci71_7_20+valci71_7_29+valci71_7_36 if year==1994
	
			
	*----------------------------------------------------
	* Skilled and Unskilled Employment  & Skill Intensity 
	*----------------------------------------------------
	* skill Employment : total workers in "administracin y ventas" minus owners in "administracin y ventas" //
	*                     + "empleados" within "produccion" (equivalent to technicians for pre-92) minus owners in "production employeed"
	gen wrk_swide=peremplh+peremplm+perdirh+perdirm+pertecnh+perteceh+pertecnm+pertecem if year<1992
	replace wrk_swide=valci85_8_5 + valci86_8_5-valci85_8_1-valci86_8_1+valci83_8_5 + valci84_8_5-valci83_8_1-valci84_8_1  if  year==1994
	gen wrk_u=perobrh+perobrm if year<1992
	replace wrk_u=valci81_8_5 + valci82_8_5-valci81_8_1-valci82_8_1	if  year==1994
	gen s_sharewide=(wrk_swide)/(wrk_swide+wrk_u)	
		
		
		
	*---------------
	*Wage per worker
	*----------------
	gen wpw= (pay_swide+pay_u)/(wrk)
	gen lnwpw=ln(wpw)
	gen wpw_skill=(pay_swide)/(wrk_swide)
	gen wpw_unskill=pay_u/wrk_u
	
	label var wpw "Wage per worker"
	label var lnwpw "Ln (wage per worker)"
	label var wpw_skill "Wage per worker for the skilled"
	label var wpw_unskill "Wage per worker for the unskilled"
		
	*---------------------------
	*Wage per worker - Deflated
	*---------------------------
	gen wpw_real= ((pay_swide+pay_u)/(wrk))/ippm_88
	gen lnwpw_real=ln(wpw_real)
	gen wpw_skill_real=((pay_swide)/(wrk_swide))/ippm_88
	gen wpw_unskill_real=(pay_u/wrk_u)/ippm_88
	
	label var wpw_real "Wage per worker - Deflated"
	label var lnwpw_real "Ln (wage per worker) - Deflated"
	label var wpw_skill_real "Wage per worker for the skilled - Deflated"
	label var wpw_unskill_real "Wage per worker for the unskilled - Deflated"
	
	*--------------
	*Skill premia
	*--------------
	gen skillpremwide=ln(wpw_skill/wpw_unskill)
	sum skillpremwide, d
	replace skillpremwide=. if skillpremwide>r(p99)|skillpremwide<r(p1)
	label var skillpremwide "skill premium"
	
	*----------------------
	* Total sales
	*----------------------
	gen ln_sales=ln(tsales)
	egen ts=sum(tsales), by (year)
	gen tsales_real = tsales/ippm_88
	gen ln_sales_real = ln(tsales_real)
	egen ts_real=sum(tsales_real), by(year)
	
	*-----------------------------
	* Plant- trade status
	*-----------------------------

	gen D_domestic=1 if esales==0 & vfm==0
	replace D_domestic=0 if (esales>0 & esales!=.)
	replace D_domestic=0 if (vfm>0 & vfm!=.)
	gen D_e=1 if (esales>0 & esales!=.)
	replace D_e=0 if esales==0 
	gen D_i=1 if (vfm>0 & vfm!=.)
	replace D_i=0 if vfm==0 
	gen D_ei=1 if (esales>0 & esales!=.)
	replace D_ei=1 if (vfm>0 & vfm!=.)
	replace D_ei=0 if esales==0 & vfm==0
	gen D_domestic2=1-D_ei
	sum D_domestic2 D_domestic
	drop D_domestic2
	
	*-----------------------------
	* Relative price index
	*-----------------------------

	gen lnp_in_rel = lnp_in - ln(ippm_88)
	gen lnp_out_rel = lnp_out - ln(ippm_88)	
	
	*=============================================================
	*   III.  SAVE DATABASE
	*=============================================================
   
	save "$cd/db\EAM8288&94_EFX_Final.dta", replace
	
	cap n log close
	
