 
	cap n log close
	
	log using "$logfile_dir\2.3. Product level database (`c(current_date)').log", replace
	
    *>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  Program's Parameters <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	 
	clear all
	set mem 50m
    
	*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  
  
    *=============================================================
    *  I. VARIABLE DEFINITION
	*=============================================================

  *  Variable definition in the original Raw database
  
  * Variable definition changes depending on the variable "tipo", this variable describes if the observation is an input (tipo=8) or
  * if the observation is an output (tipo=3)
  
    
		  * c1= Quantity produced if tipo==3 (output) | Quantity of input consumed of both national and foreign if tipo==8 (input)
         
		  * v1= output value if tipo==3 (output) | input value consumed of both national and foreign if tipo==8 (input)
        
		  * c2= Inventory at end of year if tipo==3 (output) | Quantity of input , just foreign if tipo==8 (input)
		
		  * v2= Inventory value at end of year if tipo==3 (output) | Input value consumed , just foreign if tipo==8 (input)
		  
foreach year in 82 83 84 85 86 87 88 94 {

if `year'<85 {
local folder y82_84
dis "82-84"
}
else if `year'!=94 {
local folder y85_91
dis "85-88"
}
else {
local folder y1994
dis "1994"
}
		  
    *=============================================================
    *  II. CALL AND CLEAN DATABASE
    *=============================================================
    *  II.i. Materials
    *=============================================================
    
	if `year'!=94 {
	
	use "$pym\`folder'\PROD`year'.dta" , clear
         
		rename pro codigo
		sum codigo
		di      "Drop codes corresponding to total values"
		tab  codigo tipo if codigo==999 | codigo==99999999 | codigo==38888888 | codigo==39999307 | codigo==39999404  | codigo==. | codigo==0, m  
		drop if codigo==99999999      // code corresponding to total values
		drop if codigo==999           // this code stands for goods transed  ""ad-valorem"",
		drop if codigo==38888888      // this code stands for inflation adjustments
		drop if codigo==39999307      // this code corresponds to non-transformed inputs sold by the firm
		drop if codigo==39999404      // this code corresponds to products made for other firms (outsourcing) **
		drop if codigo==. | codigo==0 // Dropping missings

		keep if tipo==8 //keeping inputs

		*	The following replaces are done because there are two plant which reports two differents sector in the same year
		*	I assign the sector that contributes more to total sales of plant (looking in product level output database)
		if `year'==83 {
		replace ciiu = 3115 if orden==220029
		replace ciiu5 = 31153 if orden==220029
		}
		if `year'==88 {
		replace ciiu = 3560 if orden==440394
		replace ciiu5 = 35603 if orden==440394
		}	
		
		}
		
		else {
		
		use "$pym\y1994\materia.dta" , clear

		sum codigo
		di      "Drop codes corresponding to total values"
		tab  codigo if codigo==999 | codigo==99999999 | codigo==38888888 | codigo==39999307 | codigo==39999404  | codigo==. | codigo==0, m  
		drop if codigo==99999999      // code corresponding to total values
		drop if codigo==999           // this code stands for goods transed  ""ad-valorem"",
		drop if codigo==38888888      // this code stands for inflation adjustments
		drop if codigo==39999307      // this code corresponds to non-transformed inputs sold by the firm
		drop if codigo==39999404      // this code corresponds to products made for other firms (outsourcing) **
		drop if codigo==. | codigo==0 // Dropping missings

		collapse (sum) v1=valqcons c1=cantmate, by(nordest codigo)
	
		}
	
	if `year' != 94 rename (orden ciu4) (norden ciiu4_r2) 					  				   					  
	save "$cd/db/mat_price_prodlev`year'.dta", replace
	
	  
    *  II.ii. Products
    *=============================================================
    
	if `year'!=94 {
	
	use "$pym\`folder'\PROD`year'.dta" , clear
         
		rename pro codigo
		sum codigo
		di      "Drop codes corresponding to total values"
		tab  codigo if codigo==999 | codigo==99999999 | codigo==38888888 | codigo==39999307 | codigo==39999404  | codigo==. | codigo==0, m  
		drop if codigo==99999999      // code corresponding to total values
		drop if codigo==999           // this code stands for goods transed  ""ad-valorem"",
		drop if codigo==38888888      // this code stands for inflation adjustments
		drop if codigo==39999307      // this code corresponds to non-transformed inputs sold by the firm
		drop if codigo==39999404      // this code corresponds to products made for other firms (outsourcing) **
		drop if codigo==. | codigo==0 // Dropping missings
		    
		keep if tipo==3 //keeping outputs

		*	The following replaces are done because there are two plant which reports two differents sector in the same year
		*	I assign the sector that contributes more to total sales of plant (looking in product level output database)
		if `year'==83 {
			replace ciiu = 3115 if orden==220029
			replace ciiu5 = 31153 if orden==220029
		}
		if `year'==88 {
			replace ciiu = 3560 if orden==440394
			replace ciiu5 = 35603 if orden==440394
		}
		
	collapse (sum) v1 c1, by(orden codigo ciu3 ciu4 ciu5 agrup ciiu ciiu5)
  
	}
	
	else {
	
	use "$pym\y1994\producto.dta" , clear

	sum codigo
	di      "Drop codes corresponding to total values"
	tab  codigo if codigo==999 | codigo==99999999 | codigo==38888888 | codigo==39999307 | codigo==39999404  | codigo==. | codigo==0, m  
	drop if codigo==99999999      // code corresponding to total values
	drop if codigo==999           // this code stands for goods transed  ""ad-valorem"",
	drop if codigo==38888888      // this code stands for inflation adjustments
	drop if codigo==39999307      // this code corresponds to non-transformed inputs sold by the firm
	drop if codigo==39999404      // this code corresponds to products made for other firms (outsourcing) **
	drop if codigo==. | codigo==0 // Dropping missings

	collapse (sum) v1=valvfab c1=cantpro, by(nordest codigo)
	
	}
  						  
	if `year' != 94 rename orden norden 					  
	save "$cd/db/out_price_prodlev`year'.dta", replace

 }
 
	*================================
	*	III. APPEND
	*================================
 
		use "$cd/db/mat_price_prodlev82.dta", clear
	gen year = 1982
	foreach year in 83 84 85 86 87 88 94 {
		append using "$cd/db/mat_price_prodlev`year'.dta"
		erase "$cd/db/mat_price_prodlev`year'.dta"
		replace year=19`year' if year==.
	}
	save "$cd/db/mat_price_prodlev8288_94.dta", replace
	erase "$cd/db/mat_price_prodlev82.dta"


	use "$cd/db/out_price_prodlev82.dta", clear
	gen year = 1982
	foreach year in 83 84 85 86 87 88 94 {
		append using "$cd/db/out_price_prodlev`year'.dta"
		erase "$cd/db/out_price_prodlev`year'.dta"
		replace year=19`year' if year==.
	}
	save "$cd/db/out_price_prodlev8288_94.dta", replace
	erase "$cd/db/out_price_prodlev82.dta"	
	
	*================================
	*	IV. PRICES
	*================================
	
	*Inputs
	
	use "$cd/db/mat_price_prodlev8288_94.dta", clear
	
	*--------------
	*	IPP
	*--------------
	
	merge m:1 year using "$other\ippm_09_8209.dta"
	
	sum ippm_09 if year==1988
	gen ippm_88=ippm_09/r(mean)
	drop if _merge==2
	drop _merge
	
	*--------------

	gen price_real = (v1/ippm_88)/c1
	gen ln_price_real = ln(price_real)

	save "$cd/db/mat_price_prodlev8288_94.dta", replace

	*Outputs
	
	use "$cd/db/out_price_prodlev8288_94.dta", clear
	
	*--------------
	*	IPP
	*--------------
	
	merge m:1 year using "$other\ippm_09_8209.dta"
	
	sum ippm_09 if year==1988
	gen ippm_88=ippm_09/r(mean)
	drop if _merge==2
	drop _merge
	
	*--------------

	gen price_real = (v1/ippm_88)/c1
	gen ln_price_real = ln(price_real)

	gen r = v1/ippm_88
	gen lnr = ln(r)

	save "$cd/db/out_price_prodlev8288_94.dta", replace
	
	*=============================================
	*	v. MERGE WITH LONGITUDINAL IDENTIFIERS & SAVE
	*=============================================

	use "$cd/db/mat_price_prodlev8288_94.dta", clear
	mmerge norden year using "$cd/db/norden_nordest.dta", type(n:1) missing(nom)
	tab year _merge
	replace nordest = nordest2 if nordest2!=.
	replace nordest = -norden if nordest==.
	drop if _merge==2
	drop _merge
	save "$cd/db/mat_price_prodlev8288_94.dta", replace

	use "$cd/db/out_price_prodlev8288_94.dta", clear
	mmerge norden year using "$cd/db/norden_nordest.dta", type(n:1) missing(nom)
	tab year _merge
	replace nordest = nordest2 if nordest2!=.
	replace nordest = -norden if nordest==.
	drop if _merge==2
	drop _merge
	save "$cd/db/out_price_prodlev8288_94.dta", replace

	log close
