	cap n log close
	log using "$logfile_dir\2.6 Tables 7 8 10 13 (`c(current_date)').log", replace


	pause on
	global sector 0 // this sector is used to indicate that the exercise is being performed for the whole set of sectors of Manuf Industry in the database.
	

	*==============================================================================================================================
	* I. Variable Creation
	*==============================================================================================================================
	

	use "$other\trade_sic3_DNP.dta", clear
	rename fobpeso_ciiu_r2 EXP_
	rename cif_pesos IMP_
	
	tab sic3

	reshape wide EXP_ IMP_, i(year) j(sic3)
	gen IMP_$sector=.
	gen EXP_$sector=.
		
	format %10.0f EXP_$sector IMP_$sector
	recast double EXP_$sector IMP_$sector
	
	replace IMP_${sector}=IMP_311+IMP_312+IMP_313+IMP_314+IMP_321+IMP_322+IMP_323+IMP_324+IMP_331+IMP_332+IMP_341+IMP_342+IMP_351+IMP_352+IMP_353+IMP_354+IMP_355+IMP_356+IMP_361+IMP_362+IMP_369+IMP_371+IMP_372+IMP_381+IMP_382+IMP_383+IMP_384+IMP_385+IMP_390
	replace EXP_${sector}=EXP_311+EXP_312+EXP_313+EXP_314+EXP_321+EXP_322+EXP_323+EXP_324+EXP_331+EXP_332+EXP_341+EXP_342+EXP_351+EXP_352+EXP_353+EXP_354+EXP_355+EXP_356+EXP_361+EXP_362+EXP_369+EXP_371+EXP_372+EXP_381+EXP_382+EXP_383+EXP_384+EXP_385+EXP_390
	
	keep year IMP_$sector EXP_$sector
	
	save "$cd\db\exp_imp_DNP_$sector.dta", replace
	
	use "$cd\db\EAM8288&94_EFX_Final.dta", clear
		
	cap drop _merge 
	merge m:1 year using "$cd\db\exp_imp_DNP_$sector.dta" 
	drop if _merge==2
	drop _merge
	erase "$cd\db\exp_imp_DNP_$sector.dta"
	replace IMP_${sector}=0 if IMP_${sector}==.
	replace EXP_${sector}=0 if EXP_${sector}==.
	
	gen nlnr_mi_exp=ln(tsales)-ln(ts-EXP_${sector}+IMP_${sector}) 
	
	*---------------------------------------------------------------------------
	* Merge with shift database
	
	mmerge year nordest using "$cd/db/dshift.dta", type(n:1)
	tab _merge
	tab year _merge
	rename _merge q_merge
	
	mmerge year nordest using "$cd/db/dshift_94.dta", type(n:1)
	tab _merge
	tab year _merge
	replace q_merge= _merge if year==1994
	drop _merge
	
	*---------------------------------------------------------------------------
	* qglobals
	
	cap drop k_imp_ait_hdfe2s_8894
	gen k_imp_ait_hdfe2s_8894 = k_imp_ait_hdfe2s
	replace k_imp_ait_hdfe2s_8894 = k_imp_ait_hdfe2s_94_2 if year==1994
		
	global dshift k_imp_ait_hdfe2s
	global dshift_94 k_imp_ait_hdfe2s_8894 
	
	*---------------------------------------------------------------------------
	
	global ifall "inrange(year,1982,1988)"
	global if1994 "year==1994"
			
	*--------------------------------------------------------------------------
	*exports variable creation
			
	        gen  dumexp=1 if esales>0 & esales!=.
			replace dumexp=0 if esales==0
			
			gen  dumimp=1 if vfm>0 & vfm!=.
			replace dumimp=0 if vfm==0
	
	*--------------------------------------------------------------------------
	* Table C3 - Creating categories of size [Ycat]  ( by ln (domestic revenue)) and categories of Wage per worker [Wcat]  (by ln (wage per worker))
		
		gen lndr=ln(tsales-esales)
		label var lndr "log domestic sales (AMS)"
		gen lndr_real=ln((tsales-esales)/ippm_88)
		label var lndr_real "log domestic sales (AMS), deflated"		
		gen lndr_sort=lndr if year<1992 // Variable defined for all the period.
		*replace lndr_sort=lndrD if year>91

		foreach s in 1982 1983 1984 1985 1986 1987 1988 {
		
		xtile q_y_`s'=lndr_sort if  year==`s' , n(4)	
	
		xtile q_w_`s'=lnwpw if  year==`s', n(4)
				
		foreach var in $dshift $dshift_94 {
		xtile q2_`var'_`s'=`var' if year==`s' & lndr_sort!=., n(4)
		
		xtile q2_y_`var'_`s'=lndr_sort if  year==`s' & `var'!=. , n(4)	

		}
		}
	
		gen ycat=.
		gen wcat=.
		foreach var in $dshift $dshift_94 {
		gen `var'_cat2=.
		gen y2`var'cat=.
		}
		
			
		foreach s in 1982 1983 1984 1985 1986 1987 1988 {
	
		replace ycat=q_y_`s' if year==`s'
		
		replace wcat=q_w_`s' if year==`s'

		foreach var in $dshift $dshift_94 {
		
			replace `var'_cat2=q2_`var'_`s' if year==`s'
			drop q2_`var'_`s'
			replace y2`var'cat=q2_y_`var'_`s' if year==`s'
			drop q2_y_`var'_`s'
			
			}
			
		drop q_y_`s' q_w_`s' 
		
		}
	
		
	 *--------------------------------------------------------------------------
		* employment variable creation
		
		    gen employment=wrk_swide+wrk_u			
			gen skill_prem=(wpw_skill/wpw_unskill)
			gen ln_skill_prem = ln(skill_prem)
		
	                               *************************  Generating Tables  ********************************

	*==============================================================================================================================
	*II.  TABLE AGGREGATES: Employment, sales, imports and exports aggregates.
	*==============================================================================================================================
								   
	preserve
	
	cap n drop ipc_09
	cap n drop ipc_08
	cap n drop ipc_88
	merge m:1 year using "Other\ipc_yearly_54_14", keep(3)
	
	gen total_wrk = wrk_swide + wrk_u
	gen n_r_mi_exp = exp(nlnr_mi_exp)
	gen tsales_ipc09 = tsales / ipc_09
	gen IMP_0_ipc09 = IMP_0/ipc_09
	gen EXP_0_ipc09 = EXP_0/ipc_09
	
	collapse (count) N_plants = tsales (sum) wrk_swide wrk_u total_wrk tsales tsales_ipc09 (mean) ipc_09 IMP_0 IMP_0_ipc09 EXP_0 EXP_0_ipc09 (sum) n_r_mi_exp, by(year)
	
	xmlsave "$cd/Paper_EFX_aggregates_table.xls", replace doctype(excel)
	
	restore
	
	*==============================================================================================================================
								   
			cap erase "$cd/Paper_EFX_tables_4_5_7_10.txt"
			cap erase "$cd/Paper_EFX_tables_4_5_7_10.xls"						   
				   
	*==============================================================================================================================
	*III.  Below instructions refer to tables with following names (numbering in previous version)
	*	TABLE 4: Unconditional distribution of sales and measured skill intensity
	*	  TABLE 10: Changes in the distributions of sales and skill intensity
	*	  TABLE C.5: Changes in the distributions of white-collar shares
	*==============================================================================================================================

		foreach year in all 1994 {
	
		di " Unconditional Dist of Normalized domestic sales"	
		tabstat nlnr_mi_exp if ${if`year'} , stat(p5 p10 p25 p50 p75 p90 p95) save
		matrix dists_mshare`year'=r(StatTotal)'

		tabstat nlnr_mi_exp if ${if`year'} , stat(mean) save
		matrix dists_mshare`year'_m=r(StatTotal)'

		di " Unconditional Dist of skill intensity (i.e share of High skill labor)"	
		tabstat s_sharewide if ${if`year'} , stat(p5 p10 p25 p50 p75 p90 p95) save
		matrix dists_sharewide`year'=r(StatTotal)'

		tabstat s_sharewide if ${if`year'} , stat(mean) save
		matrix dists_sharewide`year'_m=r(StatTotal)'

		tabstat s_sharewide if ${if`year'} [w=employment], stat(mean) save
		matrix dists_sharewide`year'_wm=r(StatTotal)'

		global table410c5_row
		global matrixk`year'
		foreach var in ${dshift_94} {
		
			matrix `var'`year'=[0,0,0,0,0,0,0]
			matrix `var'`year'_m=[0]
				
			di " Unconditional Dist of `var'"
						
			tabstat `var'  if ${if`year'},  stat(p5 p10 p25 p50 p75 p90 p95) save					
			matrix `var'`year'=r(StatTotal)'
			
			tabstat `var'  if ${if`year'},  stat(mean) save					
			matrix `var'`year'_m=r(StatTotal)'
				
			global matrixk`year' ${matrixk`year'}  \ `var'`year'' \ `var'`year'_m'		
			global table410c5_row ${table410c5_row} "`var'_Percentile_5" "`var'_Percentile_10" "`var'_Percentile_25" "`var'_Percentile_50" "`var'_Percentile_75" "`var'_Percentile_90" "`var'_Percentile_95" "`var'_Percentile_mean"

			}

	}


	* ------------------------------------------------
	* Export Table 4_10_C5
	*-------------------------------------------------

	foreach year in all 1994 {
	dis "`year'"
	matrix Table8_`year'=[dists_mshare`year'' \ dists_mshare`year'_m' \ dists_sharewide`year'' \ dists_sharewide`year'_m' \ dists_sharewide`year'_wm' ${matrixk`year'}]
	}
	
	matrix Table8=[Table8_all,Table8_1994]
	
	local r = rowsof("Table8")
	matrix J = J(`r',1,.)
	matrix Table8 =  [Table8,J]
	
	forvalues row = 1(1)`r' {
		matrix Table8[`row',3]=Table8[`row',2]-Table8[`row',1]
	}
	
	#d ;

	xml_tab Table8 using
	"$cd/Paper_EFX_tables_4_5_7_10.xls", append sheet (Table4_10_C5) cnames("1982-1988" "1994" "Final - Initial")
	rnames("Normalized domestic sales -Percentile 5"
	"Normalized domestic sales -Percentile 10"
	"Normalized domestic sales -Percentile 25"
	"Normalized domestic sales -Percentile 50"
	"Normalized domestic sales -Percentile 75"
	"Normalized domestic sales -Percentile 90"
	"Normalized domestic sales -Percentile 95"
	" Average Normalized domestic sales"
	"Unconditional distribution of Skill Intensity -Percentile 5"
	"Unconditional distribution of Skill Intensity -Percentile 10"
	"Unconditional distribution of Skill Intensity -Percentile 25"
	"Unconditional distribution of Skill Intensity -Percentile 50"
	"Unconditional distribution of Skill Intensity -Percentile 75"
	"Unconditional distribution of Skill Intensity -Percentile 90"
	"Unconditional distribution of Skill Intensity -Percentile 95"
	" Average Skill Intensity (Unweighted)"
	" Average Skill Intensity (Weighted by L) Lwhite/L"
	${table410c5_row})

	;

	# d cr;

	
	*==============================================================================================================================
	* IV.  TABLE 5: Joint distributions of sales with other characteristics (in %)
	*==============================================================================================================================
		
		* Skill intensity by Ycat *
		***************************
		
		di " Non weighted mean of Skill intensity by quartile of Ycat (domestic revenue)"
		tabstat s_sharewide if inrange(year,1982,1988), by (ycat) stat (mean) save

		matrix dists_sharewide=[r(Stat1),r(Stat2),r(Stat3),r(Stat4)]
				
		local export_sales esales
		local imports_inputs vfm
								
		* % of exporters and % importers by quartile of domestic sales *
		****************************************************************
		
		preserve
		
		collapse (sum) dumexp dumimp (count) nplant=tsales, by (year ycat)

			* % exporters
		gen eplant=dumexp/nplant
		tabstat eplant if inrange(year,1982,1988), by (ycat) stat (mean) save
					
		matrix eshare=[r(Stat1),r(Stat2),r(Stat3),r(Stat4)]
					
			* % importers
		gen iplant=dumimp/nplant
		
		tabstat iplant if inrange(year,1982,1988), by (ycat) stat (mean) save
					
		matrix ishare=[r(Stat1),r(Stat2),r(Stat3),r(Stat4)]
							
		restore
		
		* Import intensity *
		********************
		
		di "Unconditional Non weighted  Import intensity" 	
		cap drop iint
		gen iint=`imports_inputs'/vm if inrange(year,1982,1988)
		
					tabstat iint if inrange(year,1982,1988), by (ycat) stat (mean) save
					
					matrix miintnw=[r(Stat1),r(Stat2),r(Stat3),r(Stat4)]
					
		* Import intensity *
		********************

		di " Non weighted unconditional export intensity "
		cap drop eint
		gen eint=`export_sales'/tsales if inrange(year,1982,1988)
		
						tabstat eint if  inrange(year,1982,1988) , by (ycat) stat (mean) save
						
						matrix meintnw=[r(Stat1),r(Stat2),r(Stat3),r(Stat4)]
			
		* K-quality measures *
		**********************
	
		foreach var in $dshift {
		
		di "Demand shifter, `var'"
		
					tabstat `var' if inrange(year,1982,1988), by (ycat) stat (mean) save
					
					matrix `var' = [r(Stat1),r(Stat2),r(Stat3),r(Stat4)]
		   
		   }

	* ------------------------------------------------
	* Export Table 5
	*-------------------------------------------------
	
		global k
		
		global table5_row

		foreach var in $dshift {

			global k ${k} \ `var''
			
			global table5_row  $table5_row 	 "`var'_Ycat_Quartile_q1" "`var'_Ycat_Quartile_q2" "`var'_Ycat_Quartile_q3" "`var'_Ycat_Quartile_q4" 

		}

	matrix Table5=[dists_sharewide' \ ishare' \ miintnw' \ eshare' \ meintnw' ${k}]
	matrix Table5=Table5[1....,1....]


	#d ;

	xml_tab Table5 using
	"$cd/Paper_EFX_tables_4_5_7_10.xls", append sheet (Table5) cnames("1982-1988")
	rnames(	"Average Skill intensity - Ycat Quartile q1"
	"Average Skill intensity - Ycat Quartile q2"
	"Average Skill intensity - Ycat Quartile q3"
	"Average Skill intensity - Ycat Quartile q4"
	"% Importers by - Ycat Quartile q1"
	"% Importers by - Ycat Quartile q2"
	"% Importers by - Ycat Quartile q3"
	"% Importers by - Ycat Quartile q4"
	"Non weighted unconditional import intensity  - Ycat Quartile q1" 
	"Non weighted unconditional import intensity  - Ycat Quartile q2" 
	"Non weighted unconditional import intensity  - Ycat Quartile q3" 
	"Non weighted unconditional import intensity  - Ycat Quartile q4" 
	"% Exporters by - Ycat Quartile q1"
	"% Exporters by - Ycat Quartile q2"
	"% Exporters by - Ycat Quartile q3"
	"% Exporters by - Ycat Quartile q4"
	"Non weighted unconditional export intensity  - Ycat Quartile q1" 
	"Non weighted unconditional export intensity  - Ycat Quartile q2" 
	"Non weighted unconditional export intensity  - Ycat Quartile q3" 
	"Non weighted unconditional export intensity  - Ycat Quartile q4" 
	${table5_row}
	)
	;

	# d cr;	

	*==============================================================================================================================
	* V.  TABLE 7:  Aggregate skill intensity and premium
	*==============================================================================================================================
					
			*-----------------------------------
			* Aggregate Skill Premium estimation
			*-----------------------------------
			
			* Non- Weighted Average  Skill Premium (skillpu88)
			tabstat skill_prem if inrange(year,1982,1988), stat(mean) save
            matrix skillpu=r(StatTotal)'
								
			tabstat skill_prem if inrange(year,1982,1988) [w=employment], stat(mean) save
            matrix skillpw=r(StatTotal)'

		    *-------------------------------------
			* Aggregate Skill Intensity estimation
			*-------------------------------------
			
			tabstat s_sharewide [w=employment] if inrange(year,1982,1988), stat(mean) save
						
			matrix Agg_skill_int=r(StatTotal)'
			
		    *-------------------------------------
			* Quality measure
			*-------------------------------------
			
			tabstat k_imp_ait_hdfe2s if inrange(year,1982,1988) & q_merge!=2, stat(mean) save
						
			matrix Quality=r(StatTotal)'			
		
		   * ------------------------------------------------
		   * Export Table 7
		   *-------------------------------------------------
			     
		   matrix Table9=[skillpu'\skillpw'\Agg_skill_int'\Quality']
	       matrix Table9=Table9[1...,1...]
		   
		   	#d ;
			
			xml_tab Table9 using
			"$cd/Paper_EFX_tables_4_5_7_10.xls", append sheet (Table7) cnames("1982-1988")
			rnames( "skill premium wwhite/wblue"
					"emplyoment weighted skill premium wwhite/wblue"
			        "skill intensity Lwhite/L" "Quality measure")
			        
			;
			
		    # d cr;	

			
	*==============================================================================================================================
	* VI.a . TABLE C.3 wages Joint Distribution Wages and sales and correlation between quartiles
	*==============================================================================================================================
		
		tab ycat wcat if inrange(year,1982,1988),  matcell(ywfreq)
		scalar total_N = r(N)
		matrix temp=(ywfreq/total_N)
	
		matrix ywshare=[temp[1..4,1]',temp[1..4,2]',temp[1..4,3]',temp[1..4,4]']
						
	*==============================================================================================================================
	* VI.b . TABLE C.3 q Joint Distribution Demand shifter and sales and correlation between quartiles
	*==============================================================================================================================
				
		foreach var in $dshift {
		
		tab y2`var'cat `var'_cat2 if inrange(year,1982,1988),  matcell(y`var'f)
		tab year if inrange(year,1982,1988) & `var'_cat2!=. & y2`var'cat!=.
		scalar total_N = r(N)
		matrix temp=(y`var'f/total_N)
	
		matrix list temp
	
		matrix y`var's = [temp[1..4,1]',temp[1..4,2]',temp[1..4,3]',temp[1..4,4]']
		
		}

	* ---------------------------------------------------------------------------
	* Export Joint Distribution Wages and sales and correlation between quartiles
	*----------------------------------------------------------------------------

	matrix JD_wage_size=[ywshare']

	# d ;
	xml_tab JD_wage_size using "$cd/Paper_EFX_tables_4_5_7_10.xls",  append sheet (Table C3 w) cnames("1982-1988")

	rnames( "Joint size  , Wage distribution (y1,w1)" 
	"Joint size  , Wage distribution (y2,w1)"
	"Joint size  , Wage distribution (y3,w1)"
	"Joint size  , Wage distribution (y4,w1)"
	"Joint size  , Wage distribution (y1,w2)"
	"Joint size  , Wage distribution (y2,w2)"
	"Joint size  , Wage distribution (y3,w2)"
	"Joint size  , Wage distribution (y4,w2)"
	"Joint size  , Wage distribution (y1,w3)"
	"Joint size  , Wage distribution (y2,w3)"
	"Joint size  , Wage distribution (y3,w3)"
	"Joint size  , Wage distribution (y4,w3)"
	"Joint size  , Wage distribution (y1,w4)"
	"Joint size  , Wage distribution (y2,w4)"
	"Joint size  , Wage distribution (y3,w4)"
	"Joint size  , Wage distribution (y4,w4)") 
	;

	# d cr;

	* ---------------------------------------------------------------------------
	* Export Joint Distribution demand shifter and sales and correlation between quartiles for year 1988
	* ---------------------------------------------------------------------------
	
	matrix one = [1\1\1\1\1\1\1\1\1\1\1\1\1\1\1\1]
	
		global k	
		
		global table8_col
								
				foreach var in $dshift {

					global k ${k} , y`var's'
					global table8_col "${table8_col}" "`year' `var'"
				}


		matrix JD_dshift_size=[one ${k}]
		matrix JD_dshift_size=JD_dshift_size[1...,2...]
		
		dis `"$table8_col"'
		
		# d ;
		xml_tab JD_dshift_size using "$cd/Paper_EFX_tables_4_5_7_10.xls",  append sheet (Table C3 q) 
		
		cnames(	"1982-1988 k_imp_ait_hdfe2s")

		rnames( "Joint size  , Demand shifter distribution (y1,dsh1)" 
		"Joint size  , Demand shifter distribution (y2,dsh1)"
		"Joint size  , Demand shifter distribution (y3,dsh1)"
		"Joint size  , Demand shifter distribution (y4,dsh1)"
		"Joint size  , Demand shifter distribution (y1,dsh2)"
		"Joint size  , Demand shifter distribution (y2,dsh2)"
		"Joint size  , Demand shifter distribution (y3,dsh2)"
		"Joint size  , Demand shifter distribution (y4,dsh2)"
		"Joint size  , Demand shifter distribution (y1,dsh3)"
		"Joint size  , Demand shifter distribution (y2,dsh3)"
		"Joint size  , Demand shifter distribution (y3,dsh3)"
		"Joint size  , Demand shifter distribution (y4,dsh3)"
		"Joint size  , Demand shifter distribution (y1,dsh4)"
		"Joint size  , Demand shifter distribution (y2,dsh4)"
		"Joint size  , Demand shifter distribution (y3,dsh4)"
		"Joint size  , Demand shifter distribution (y4,dsh4)") 
		;

		# d cr;		
			

	cap n log close
		   
		   
		   
	*==============================================================================================================================
	*	.	Correlation between quality variables
	*==============================================================================================================================

	matrix corr = J(11,11,0)
	matrix corr_yearly_av = J(11,11,0)

	local h = 0
	foreach var1 in k_imp_ait_hdfe2s s_sharewide lnwpw_real lnp_in_rel lnp_out_rel D_i shimp D_e shexp ln_sales_real lndr_real {
		
		local h = `h' + 1
		
		local j = 0
		foreach var2 in k_imp_ait_hdfe2s s_sharewide lnwpw_real lnp_in_rel lnp_out_rel D_i shimp D_e shexp ln_sales_real lndr_real {
		
		local j = `j' + 1

			qui corr  `var1' `var2'
			
			matrix corr[`h',`j'] = r(rho)
			
			forvalues year = 1982(1)1988 { 
			
				qui corr  `var1' `var2' if year==`year'
				
				matrix corr_yearly_av[`h',`j'] = corr_yearly_av[`h',`j'] + r(rho)/7

			}
		}
	}


		# d ;
		xml_tab corr using "$cd/Paper_EFX_tables_4_5_7_10.xls",  append sheet (q correl) 
		
		cnames(	"k_imp_ait_hdfe2s" "s_sharewide" "lnwpw_real" "lnp_in_rel" 
		"lnp_out_rel" "D_i" "shimp" "D_e" "shexp" "ln_sales_real" "lndr_real")

		rnames(	"k_imp_ait_hdfe2s" "s_sharewide" "lnwpw_real" "lnp_in_rel" 
		"lnp_out_rel" "D_i" "shimp" "D_e" "shexp" "ln_sales_real" "lndr_real");

		# d cr;		
	
		# d ;
		xml_tab corr_yearly_av using "$cd/Paper_EFX_tables_4_5_7_10.xls",  append sheet (q correl yearly average) 
		
		cnames(	"k_imp_ait_hdfe2s" "s_sharewide" "lnwpw_real" "lnp_in_rel" 
		"lnp_out_rel" "D_i" "shimp" "D_e" "shexp" "ln_sales_real" "lndr_real")

		rnames(	"k_imp_ait_hdfe2s" "s_sharewide" "lnwpw_real" "lnp_in_rel" 
		"lnp_out_rel" "D_i" "shimp" "D_e" "shexp" "ln_sales_real" "lndr_real");

		# d cr;	
		
	matrix corr = J(11,11,.)
	matrix p_value = J(11,11,.)

	local h = 0
	foreach var1 in k_imp_ait_hdfe2s s_sharewide lnwpw_real lnp_in_rel lnp_out_rel D_i shimp D_e shexp ln_sales_real lndr_real {
		
		local h = `h' + 1
		
		local j = 0
		foreach var2 in k_imp_ait_hdfe2s s_sharewide lnwpw_real lnp_in_rel lnp_out_rel D_i shimp D_e shexp ln_sales_real lndr_real {
		
		local j = `j' + 1

		if "`var1'" != "`var2'" {
		
			qui canon  (`var1') (`var2')
			matrix ccorr = e(ccorr)
			matrix stat = e(stat_1)
			
			matrix corr[`h',`j'] = ccorr[1,1]
			matrix p_value[`h',`j'] = stat[1,5]
			
		}
			
		else {
		
			matrix corr[`h',`j'] = 1
			matrix p_value[`h',`j'] = 0
		
		}
		
		}
	}
		
	# d ;
	xml_tab p_value using "$cd/Paper_EFX_tables_4_5_7_10.xls",  append sheet (p-values q correl) 
	
	cnames(	"k_imp_ait_hdfe2s" "s_sharewide" "lnwpw_real" "lnp_in_rel" 
	"lnp_out_rel" "D_i" "shimp" "D_e" "shexp" "ln_sales_real" "lndr_real")

	rnames(	"k_imp_ait_hdfe2s" "s_sharewide" "lnwpw_real" "lnp_in_rel" 
	"lnp_out_rel" "D_i" "shimp" "D_e" "shexp" "ln_sales_real" "lndr_real");

	# d cr;	
	
