	cap n log close
	log using "$logfile_dir\2.9 Table 12 (`c(current_date)').log", replace
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


    *==============================================================
    *  I. CALL DATABASE, SUMMARIZE VARIABLE AND MERGE WITH TARIFFS
	*==============================================================
	
	use "$cd\db\EAM8288&94_EFX_Final.dta", clear
		
	xtset nordest year
	
	*---------------------------------------------------------------------------
	* Merge with Khandelwal measure dataset
	
	mmerge year nordest using "$cd/db/dshift.dta", type(1:1) missing(nom)
	drop if _merge==2

	*---------------------------------------------------------------------------
	* Merge with Output and Input tariffs
	
	mmerge year nordest using "$cd/db/input_tariff8288_94.dta", type(1:1)
	tab year _merge
	drop if _merge==2
	
	mmerge year nordest using "$cd/db/output_tariff8288_94.dta", type(1:1)
	tab year _merge
	drop if _merge==2	
		
	*---------------------------------------------------------------------------
	* Globals
	
	global dependent s_sharewide lnwpw_real lnp_in_rel lnp_out_rel k_imp_ait_hdfe2s D_i shimp D_e shexp
	global independent tariff_in tariff_out
	
	foreach var of global independent {
	
	replace `var' = `var'/100
	replace l_`var' = l_`var'/100
	
	}
	
	*---------------------------------------------------------------------------
	* Summarize
		
	sum $dependent tariff_in tariff_out 	
	
    *==============================================================
    *  II. REGRESSIONS AT PLANT LEVEL
	*==============================================================
	
	cap n erase "$cd/Paper_EFX_Table12.xls"
	cap n erase "$cd/Paper_EFX_Table12.txt"
	
	xtset nordest year
				
	foreach dep of global dependent {

		* OLS absorbing

		reghdfe `dep' tariff_in if inrange(year,1982,1988), abs(nordest year)
		outreg2  tariff_in using "$cd/Paper_EFX_Table12.xls", append addtext(Plant FE, Yes, Year FE, Yes, Sector, No, Period, 1982-1988, Model, OLS)

	}

	foreach dep of global dependent {
		
		* 2SLS absorbing, using lagged tarrif as instrument

		reghdfe `dep' (tariff_in = l_tariff_in) if inrange(year,1982,1988), abs(nordest year) est(2sls) stage(first) ivsuite(ivregress)
		outreg2  tariff_in using "$cd/Paper_EFX_Table12.xls", append addtext(Plant FE, Yes, Year FE, Yes, Sector, No, Period, 1982-1988, Model, "IV (lag of tariffs as instrument)")


	}
	
*********** Output tariffs ******************
	
	foreach dep of global dependent {

		* OLS absorbing

		reghdfe `dep' tariff_out if inrange(year,1982,1988), abs(nordest year)
		outreg2  tariff_out using "$cd/Paper_EFX_Table12.xls", append addtext(Plant FE, Yes, Year FE, Yes, Sector, No, Period, 1982-1988, Model, OLS)

	}

	foreach dep of global dependent {
		
		* 2SLS absorbing, using lagged tarrif as instrument

		reghdfe `dep' (tariff_out = l_tariff_out) if inrange(year,1982,1988), abs(nordest year) est(2sls) stage(first) ivsuite(ivregress)
		outreg2  tariff_out using "$cd/Paper_EFX_Table12.xls", append addtext(Plant FE, Yes, Year FE, Yes, Sector, No, Period, 1982-1988, Model, "IV (lag of tariffs as instrument)")


	}	
	
*********** Input & Output tariffs ******************
	
	
	foreach dep of global dependent {

		* OLS absorbing

		reghdfe `dep' tariff_in tariff_out if inrange(year,1982,1988), abs(nordest year)
		outreg2  tariff_in tariff_out using "$cd/Paper_EFX_Table12.xls", append addtext(Plant FE, Yes, Year FE, Yes, Sector, No, Period, 1982-1988, Model, OLS)

	}

	foreach dep of global dependent {
		
		* 2SLS absorbing, using lagged tarrif as instrument

		reghdfe `dep' (tariff_in tariff_out = l_tariff_in l_tariff_out) if inrange(year,1982,1988), abs(nordest year) est(2sls) stage(first) ivsuite(ivregress)
		outreg2  tariff_in tariff_out using "$cd/Paper_EFX_Table12.xls", append addtext(Plant FE, Yes, Year FE, Yes, Sector, No, Period, 1982-1988, Model, "IV (lag of tariffs as instrument)")


	}	
	
cap n erase "$cd/Paper_EFX_Table12.txt"

		
	*	III. DESCRIPTIVE STATISTICS
	*=========================================================================
	
	cap n erase "$cd/Paper_EFX_descriptives.xls"
	
		matrix des = [1,2,3,4,5,6]
		matrix colnames des = Obs Mean Median SD Min Max
		
	foreach var in $dependent $independent {
	
	sum `var' if inrange(year,1982,1988), det
	
	matrix des = [des\r(N),r(mean),r(p50),r(sd),r(min),r(max)]
	
	}
	
	matrix rownames des = one $dependent $independent
	
	matrix des = des[2...,1...]
	
	xml_tab des using "$cd/Paper_EFX_Table12_descrip.xls",  replace sheet (Plant level) cnames(Obs Mean Median SD Min Max) rnames($dependent $independent)
	
	cap n log close
