	cap log close

	log using "building external databases\logs\EXP_DNP_(`c(current_date)').log", replace

	*============================*
	* Import from Excel to STATA *
	*============================*
	
	* Imports
	
	import excel "building external databases\MCIIU.xlsx", cellrange(A3:G4428) first clear
	rename (Año FOBdoláres CIFdoláres) (Ao FOBdolres CIFdolres)
	rename 	(Ao CiiuRev2 DescripciónCIIURev2 PesoNetoKilos FOBdolres CIFdolres CIFpesos) ///
			(year sic sec_description_m pesoneki_x m_fobdolar_ciiu_r2 cif_dolar cif_pesos)
			
	destring year sic, replace
	keep if year<2011
	
	foreach var in m_fobdolar_ciiu_r2 cif_dolar cif_pesos {
	replace `var' = round(`var')
	}
	
	save "building external databases\MCIIU_temp", replace

	* Exports
	
	import excel "building external databases\XCIIU.xlsx", cellrange(A3:F4348) first clear
	rename (Año Fobdoláres) (Ao Fobdolres)
	rename 	(Ao CiiuRev2 Descripción Fobdolres FobPesos PesoNetoKilos) ///
			(year sic sec_description fobdolar_ciiu_r2 fobpeso_ciiu_r2 pesoneki_x)
			
	destring year sic, replace
	keep if year<2011
	
	foreach var in fobdolar_ciiu_r2 fobpeso_ciiu_r2 {
	replace `var' = round(`var')
	}
	
	save "building external databases\XCIIU_temp", replace
	
	
	*============================================================*
	* Aggregate international trade by year for relevant sectors *
	*============================================================*
	
	global trade X M
	
	local nameX EXP
	local nameM IMP
	local varX fobpeso_ciiu_r2
	local varM cif_pesos
	
	foreach x of global trade {
	
	use "building external databases/`x'CIIU_temp.dta", clear
	keep if year>1981 & year<2008
	replace year=year-1900
	replace `var`x''=`var`x''/1000
	
	if "`x'"=="M" {
		replace `var`x''=cif_dolar*194.26/1000 if year==86
		* As for the year 1986 we don't have imports in Colombian Pesos CIF, we need to convert the US dolars CIF into Pesos
		* 194.26 is the exchange rate for 1986.
		}
	
	gen sic3=floor(sic/10)
	gen sic2=floor(sic/100)
	saveold "building external databases/`name`x''_82_07_DNP.dta", replace
		
	}
	

	use "building external databases/IMP_82_07_DNP.dta", clear
	keep if sic3>300 & sic3<400
	collapse (sum) cif_pesos, by(year sic3)
	save "building external databases/IMP_temp.dta", replace
	
	use "building external databases/EXP_82_07_DNP.dta", clear
	keep if sic3>300 & sic3<400
	collapse (sum) fobpeso_ciiu_r2, by(year sic3)
	
	merge 1:1 year sic3 using "building external databases/IMP_temp.dta"
	drop _merge
	replace year=year+1900
	saveold "external databases/trade_sic3_DNP.dta", replace
	
	erase "building external databases/IMP_temp.dta"
	erase "building external databases/IMP_82_07_DNP.dta"
	erase "building external databases/EXP_82_07_DNP.dta"
	erase "building external databases/XCIIU_temp.dta"
	erase "building external databases/MCIIU_temp.dta"
	
	cap log close
	
	
	