
	cap log close

	log using "$cd\logs\8.log_const_outp_price.log" , replace
	use "$cd\temp_database\Out_82to12_3_me.dta", replace
	
	* ===================================================================
	* I. Create required variables for input prices: shares, prices
	* ===================================================================
		  
	* create price variable
	
	replace valvfab=. if valvfab<0
	replace cantpro=. if cantpro<0
	gen price=valvfab/cantpro
	 
	 count
	 sum valvfab cantpro price, d
	 bys year: sum valvfab cantpro price
	 
	 gen lnprice=ln(price)
	 sum lnprice, d
	 
	 * Now I'll delete those products that never reported valvfab

	bys nordest codigo: egen max_v=max(valvfab)
	keep if max_v>0 & max_v!=.
	drop max_v
		

	* generate the share of product h in plant's  total value of materials 
	bys nordest year: egen total_output=total(valvfab)
	gen codigo_val=valvfab
	replace codigo_val=0 if codigo_val==.
	gen share_h_j_t = codigo_val/total_output
	bys nordest codigo: egen share_h_j=mean(share_h_j_t)
	drop total_output codigo_val
	
		* Those products (h) that have a share of 0 or missing within plant j : 
	
	count if share_h_j==. 
	count if share_h_j==0
	sum share_h_j, d

	/****DIVISIA SHARES ******/
	
	egen id_code = group(nordest codigo)
	
	xtset id_code year
	
	* generate the share of product h in plant's  total value of materials in t-1
	gen l_share_h_j_t = l.share_h_j_t
	replace l_share_h_j_t = 0 if l_share_h_j_t==.
	gen mean_share_h_j_t_t1 = (share_h_j_t + l_share_h_j_t)/2
	
	drop share_h_j_t l_share_h_j_t
	
		* Those products (h) that have a share of 0 or missing within plant j : 
	
	count if share_h_j==. 
	count if share_h_j==0
	sum share_h_j, d
	
	* verifying that shares add up to 1 within each firm
	
	bys nordest year : egen verifying=total(share_h_j)
	
	di "in the next table we verify that sum of shares of inputs within each plant add up to 1"
	tab verifying
	
	* ===================================================================
	* I. Create tornquist index of price changes
	* ===================================================================
	
	
	////****** Log price changes at the product level ****////
	
	
	* generate change in lnprice within product-firm
	 sort nordest codigo year
	 gen d_lnprice=lnprice-lnprice[_n-1] if nordest==nordest[_n-1] & codigo==codigo[_n-1] & year[_n-1]==year-1
	 bys year: sum d_lnprice
	 
	 * now we'll truncate the distribution to eliminate outliers this could be done within codigo & year
	 
	 sum d_lnprice, d
	 
	 egen p1= pctile(d_lnprice) , p(1)
	 sum p1
 	 egen p99= pctile(d_lnprice) , p(99)
	 sum p99
	 
	 replace d_lnprice=p1 if d_lnprice<p1 &  d_lnprice!=.
	 replace d_lnprice=p99 if d_lnprice>p99 &  d_lnprice!=.
	 
	 count
	 sum d_lnprice, d
	 	  
	 save "$cd\temp_database\Out_82to12_4_me.dta", replace
	  
	 /**filling missing values in log price change to be able to aggregate over all products ever produced by plant****/
	 
	count if d_lnprice==.
	bys year: count if d_lnprice==.
	 
 	bys codigo ciiu3_r2 year : egen m_dlnprice=mean(d_lnprice) /* h=product, k =ciiu3_r2 t=year*/
	replace d_lnprice=m_dlnprice if d_lnprice==. 
	count if d_lnprice==. & year!=1982
	drop m_dlnprice
	
	bys codigo year : egen m_dlnprice=mean(d_lnprice) /* h=product, k =ciiu3_r2 t=year*/ 
	replace d_lnprice=m_dlnprice if d_lnprice==. 
	count if d_lnprice==. & year!=1982
	drop m_dlnprice
	
	bys ciiu3_r2 year : egen m_dlnprice=mean(d_lnprice) /* h=product, k =ciiu3_r2 t=year*/
	replace d_lnprice=m_dlnprice if d_lnprice==. 
	count if d_lnprice==. & year!=1982
	drop m_dlnprice
	
	bys year : egen m_dlnprice=mean(d_lnprice) /* h=product, k =ciiu3_r2 t=year*/ 
	replace d_lnprice=m_dlnprice if d_lnprice==. 
	count if d_lnprice==. & year!=1982
	drop m_dlnprice

	 
	/****** tornquist index at plant level ***/
	
	gen auxiliar=share_h_j*d_lnprice
	tabstat year if auxiliar==. , by (year)
	sum auxiliar, d
	
	collapse (mean) ciiu3_r2 (sum) tornquist=auxiliar , by(nordest year)
	
	sum tornquist, d
	bys year: sum tornquist, d
	
		 save "$cd\temp_database\tornquist.dta", replace
	

	
	* ===================================================================
	* I. Create base year index of prices for each plant 
	*  
	* ===================================================================

	use "$cd\temp_database\Out_82to12_4_me.dta", replace
	
	 * Now generate within year price deviations plant vs. rest
	 
	 bys codigo year: egen m_price_ht=mean(price)
	 gen desv_lnprice_h=lnprice-ln(m_price_ht)
	 
	 sum desv_lnprice_h, d
	 
	 replace desv_lnprice_h=0 if desv_lnprice_h==. 
	 sum desv_lnprice_h, d
	 
	 /*missing values will not contribute to deviating plant's base year price from average, but shares will still add to 1*/
	 gen aux2=desv_lnprice_h*share_h_j 
		 
	collapse (sum) desv_price_base=aux2 , by(nordest year)
	sum desv_price_base, d
	
	 * Define base year
 
	 sort nordest year
	 gen base_year=1 if nordest[_n]!=nordest[_n-1]
	 replace base_year=1 if nordest[_n]==nordest[_n-1] & year[_n]!=year[_n-1]+1
	 replace base_year=0 if base_year!=1
	 drop if base_year==0
	 
	count
	sum desv_price_base, d
	
	save "$cd\temp_database\base year desvs.dta", replace 
	
	
	* Now constructing index prices
	 use   "$cd\temp_database\tornquist.dta", clear
	 merge 1:1 nordest year using "$cd\temp_database\base year desvs.dta"
	 replace base_year=0 if base_year==.
	 
	 foreach year in 1982 1983 1984 1985 1986 1987 1988 1989 1990 1991 1992 1993 1994 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009 2010 2011 2012 {
	 
	 cap drop first_round
	 gen first_round=1 if base_year==1 & year==`year'
	 replace first_round=0 if first_round==.
	 
	 if `year'==1982{
	 gen lnp=ln(1)+ desv_price_base if  first_round==1
	 gen P_ind=exp(lnp)
	 }
	
	if `year'>1982{
	 sort nordest year
	 replace lnp=lnp[_n-1]+tornquist if first_round==0 & year==`year' & nordest==nordest[_n-1]
	 replace P_ind=exp(lnp)
	 
	
	
	bys ciiu3_r2: egen m_Pindex=mean(P_ind) if year==`year'
	replace   lnp=ln(m_Pindex)+ desv_price_base if first_round==1 & year==`year'
	replace P_ind=exp(lnp)
	
 	  di "missings in P_ind for year: `year' after desviating from ciiu & year"
	 count if P_ind==. & year==`year'
	 
	cap drop m_Pindex
	 
	}
		}
	
	keep year nordest P_ind lnp
	
	bys year: sum P_ind, d
	
	
	sort nordest year
	 save "$cd\temp_database\Final_Output_deflo_me.dta", replace
	 
	 log close
