!****************************************************************************
!
!  PROGRAM: MC_identify
!
!  PURPOSE: Monte Carlo simulations to estimate model on generated data
!
!****************************************************************************

module parameters_model
		real(8), parameter      :: mysigma = 5.0d0                              ! elasticity of substitution across sectors
        real(8), parameter      :: markup = mysigma/(mysigma-1.d0)
        real(8), parameter      :: myalpha = 0.7d0			            	    ! Cobb-Douglas weight on labor
        real(8), parameter      :: tariff0 = .32d0	
        real(8), parameter      :: income0 = 1.d0
        real(8), parameter      :: pf0 = 1.d0
        real(8), parameter      :: sigmal = 1.6d0                               ! elasticity of substitution between skilled and unskilled workers
		integer(4), parameter   :: sizef = 100000
        real(8), parameter      :: womega = 1.d0/sizef                          ! firm weight (1/sizef)
        integer(4), parameter   :: sizeq = 201					                ! size of q grid and number of firm draws
        real(8), parameter      :: qlow = 0., qup = 10.d0                       ! lower and upper bound of existing varieties: nq varieties exist for any q\in [qlow,qbar]; low = (1.d0/(sizeq-1.))
        integer(4), parameter   :: n = 18                                       ! number of parameters to be estimated. One extra is l3 which is calibrated (no standard errors)
        integer(4), parameter   :: n1 = 13, n2 = 4, n3 = 1                      ! number of parameters to be estimated in each stage
        integer(4), parameter   :: nc = 2                                       ! number of parameters counterfactual
        real(8)                 :: xminall(n), xmin1(n1), xmin2(n2), xmin3(n3)
        real(8)                 :: xall0(n), x10(n1), x20(n2), x30(n3)          ! current data        
        real(8)                 :: omin1, omin2, omin3
        logical                 :: toprint, pflag, saflag
        real(8), parameter      :: nfirms_data0 = 6710.d0                        ! number of plants
        integer(4)              :: nexit, nimport, nexport                      ! number of firms exiting, importing and exporting
        real(8) :: xest(n), xest1(n1), xest2(n2), xest3(n3)
        real(8), dimension(n) :: separ
        real(8), dimension(n,2) :: xinterval                    ! interval for parameter guesses
    
        real(8) :: delta123(n), delta1(n1), delta2(n2), delta3(n3)
end module parameters_model
  
	! ----------------------------------------------------------------------------
	 module parameters_quality
     use parameters_model
        real(8) :: pca, pch, pcf                                        ! consumer's price indices---total, domestic and foreign only
	    real(8), dimension(sizeq) :: qgrid
	    real(8), dimension(sizeq) :: chiq, chiqf                        ! demand shifter in Home and Foreign
        real(8), dimension(sizeq) :: priceqa, priceqh, priceqf          ! price index all goods, home and foreign goods only
		real(8), dimension(sizeq) :: consumptionchi, intermediatechi
		real(8), dimension(sizeq, 4) :: rfirmp                          ! price component for each firm type; representative firm p
		real(8), dimension(sizeq, 4) :: nq, zq                  		! zq=1/aq is the productivity of a representative firm (in the Melitz sense) of quality q.
		real(8), dimension(sizeq) :: fxq	    	                    ! fixed cost of production for each firm-quality
        real(8), dimension(sizeq), parameter :: wq = 1.d0               ! labor cost index for firm of quality q
     end module parameters_quality
     
	! ----------------------------------------------------------------------------
	module parameters_firm
        use parameters_model
		integer(4), dimension(sizef) :: posqomega			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	    integer(4), dimension(sizef) :: eposqomega			        	! positition of quality choice
		integer(4), dimension(sizef) :: ediscreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	end module parameters_firm

	! ----------------------------------------------------------------------------
	module parameters_qf
		use parameters_model
		real(8), dimension(sizef) :: qomega						        ! choice of quality (level)
		real(8), dimension(sizef, sizeq) :: zqomega		        		! productivity of each firm 
        real(8), dimension(sizef) :: fxomega, fmomega                   ! firm-specific import and export cost
        real(8), dimension(sizef,2) :: skomega                          ! firm-specific measurement error of skilled workers
    end module parameters_qf
    
    ! ----------------------------------------------------------------------------
	module random_draw
	    use parameters_model
        real(8), dimension(sizef,2) :: random_firms
        real(8), dimension(sizef,2) :: random_fxcost
        real(8), dimension(sizef) :: random_skill
    end module random_draw  
    
    ! ----------------------------------------------------------------------------
	module crosssec
    use parameters_quality
    use parameters_firm
        real(8), dimension(sizef) :: imp_share, exp_share              ! share of materials imported
        real(8), dimension(sizef) :: inprice, outprice
        real(8), dimension(sizef) :: nhsales
        real(8), dimension(sizef) :: skilla, skillm                     ! actual and measured skill intensity
        real(8), dimension(sizeq) :: zbarq                              ! quantity equivalence
        real(8), dimension(sizeq) :: skillq                             ! skill intensity by quality level
        real(8), dimension(sizeq) :: wageq                              ! average wage of a firm with quality q
        real(8) :: aggskm, aggskpm                                      ! aggregate measured skill intensity and premium
        real(8) :: timport, absorption
        real(8), dimension(sizef) :: tsales
    end module crosssec
    
	! ----------------------------------------------------------------------------
    module parameters_polytope1
        use parameters_model
		integer(4), parameter :: np1 = n1 + 1	
        integer(4), parameter ::  maxfun = 200*n1, maxiter = 200*n1       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), dimension(n1) :: usual_delta
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytope1
    ! ----------------------------------------------------------------------------
    module parameters_polytope2
        use parameters_model
		integer(4), parameter :: np1 = n2 + 1	
        integer(4), parameter ::  maxfun = 200*n2, maxiter = 200*n2       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), dimension(n2) :: usual_delta
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytope2
	! ----------------------------------------------------------------------------
    module parameters_polytope3
        use parameters_model
		integer(4), parameter :: np1 = n3 + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8) :: usual_delta(n3)
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytope3
    ! ----------------------------------------------------------------------------
    module parameters_polytopeall
        use parameters_model
		integer(4), parameter :: np1 = n + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), dimension(n) :: usual_delta 
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytopeall
    ! ----------------------------------------------------------------------------
    module parameters_sanneal
		use parameters_model
		real(8), parameter :: inittemp = 1.d0
        real(8), parameter :: stoptemp = 1.e-8
        integer(4), parameter :: maxconsec = 3000 !1000                      ! maximum number of rejections before program stops
        integer(4), parameter :: maxtry = 1000    !300                       ! maximum evaluations at any temperature
        integer(4), parameter :: maxsucc = 30     !20                        ! maximum number of successes within one temperature
		integer(4), parameter :: maxsanneal = 5                              ! runs at most 5 simulated annealing algorithms for any guess
        integer(4), parameter :: maxfail = 30     !20                        ! maximum number of attempts to find an equilibrium in a neighborhood of current
        real(8), dimension(n1) :: mydelta1
        real(8), dimension(n2) :: mydelta2
        real(8), dimension(n3) :: mydelta3
    end module parameters_sanneal
    
    ! ----------------------------------------------------------------------------
    module parameters_polytopec
        use parameters_model
        integer(4), parameter :: np1 = nc + 1	
        integer(4), parameter ::  maxfun = 200*nc, maxiter = 200*nc       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), parameter :: usual_delta = 0.1d0
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 1, max_xiguess = 1               !runs at most 5 polytopes per initial guess
    end module parameters_polytopec
    ! ----------------------------------------------------------------------------
    module parameters_estimate
		real(8) :: mu1, sigma1                                          ! z1(omega) log normal mean and variance
        real(8) :: sigma2                                               ! z2(omega) normal, variance. Mean is zero.
        real(8) :: z3                                                   ! parameter of zbar
        real(8) :: f1, f2                                               ! fixed cost parameters
        real(8) :: fm, fx                                               ! fixed costs to import and export
        real(8) :: fmvar, fxvar                                         ! variance of fixed import and export cost
        real(8) :: l1, l2, l3
        real(8) :: qf                                                   ! quality of representative foreign firm 
        real(8) :: qcf                                                  ! quality of representative foreign consumer
        real(8) :: skpremium                                            ! skill premium wageh/wagel, where wagel = 1, normalized above
        real(8) :: yf 
        real(8) :: varpi                                                ! variance of firm-specific classification error
        real(8) :: nu                                                   ! complementarity parameter in production
        real(8) :: pf, yf0, income, tariff, nfirms_data
               
        
        ! at optimal
        real(8) :: emu1, esigma1                                          ! z1(omega) log normal mean and variance
        real(8) :: esigma2                                               ! z2(omega) normal, variance. Mean is zero.
        real(8) :: ez3                                                   ! parameter of zbar
        real(8) :: ef1, ef2                                               ! fixed cost parameters
        real(8) :: efm, efx                                               ! fixed costs to import and export
        real(8) :: efmvar, efxvar                                         ! variance of fixed import and export cost
        real(8) :: el1, el2, el3
        real(8) :: eqf                                                   ! quality of representative foreign firm 
        real(8) :: eqcf                                                  ! quality of representative foreign consumer
        real(8) :: eskpremium                                            ! skill premium wageh/wagel, where wagel = 1, normalized above
        real(8) :: eyf 
        real(8) :: evarpi                                                ! variance of firm-specific classification error
        real(8) :: enu                                                   ! complementarity parameter in production
        
    end module parameters_estimate

	! ----------------------------------------------------------------------------
	! fortran_codes/data/february 2016/new baseline Jan 27.xlsx
    module data_moments
		integer(4), parameter :: sizecoef = 4
		real(8), dimension(4,4) :: qsalesw_data
        real(8), dimension(5)   :: psize_data                           ! ln(domestic sales) distribution of plants 10, 25, 50, 75, 90 percentiles, with feenstra adjustment        
        real(8), dimension(5)   :: pskill_data                          ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(4)   :: sksize_data                          ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(4)   :: exint_data                           ! average export intensity by quartile of domestic sales 
        real(8), dimension(4)   :: imint_data                           ! average import intensity by quartile of domestic sales
        real(8), dimension(4)   :: exshr_data                           ! % exporters by quartile of domestic sales
        real(8), dimension(4)   :: imshr_data                           ! % importers by quartile of domestic sales
        real(8), dimension(4)   :: kimp_ai                              ! Khandelwal's measure of quality; out of sample; median set to 0 or average to 1?
        real(8)                 :: skshare_data                         ! aggregate measured skill share
        real(8)                 :: skpremium_data                       ! skill premium = (average wage of white/average wage of blue-collar workers) 
        real(8)                 :: exitrate                             ! historical exit rate
        real(8), dimension(2)   :: skill_pdata, sales_pdata
        real(8) :: meanq_data, seq_data
        
        ! data on USA skill premium and skill intensity for estimating bounds (Autor, Katz, Krueger, QJE1998) See calculation in myaggregates_colombiagov.xlsx
        ! ----------------------------------------------------------------------------
        real(8), parameter :: USAskint =  .229d0                          ! table 1 appendix page 1209, average between 1990 and 1980 census, college equivalents
        
        ! price regressions, standard errors
        ! ----------------------------------------------------------------------------
        real(8), dimension(2), parameter :: skill_sepdata = (/ 0.037d0, 0.019d0 /)
        
        ! ----------------------------------------------------------------------------
        integer(4), parameter :: nweight = 53
        real(8), dimension(nweight) :: mweight   ! weighting matrix is a diagonal matrix, and it should be the inverse of variance
        integer(4), parameter :: nboot = 47
        
    end module data_moments    

    
    ! ----------------------------------------------------------------------------
	module counterfactual
        use parameters_model
		integer(4), dimension(sizef) :: posqomega0, posqomega1			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted0, discreted1			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	    real(8) :: aggska0, aggska1, dq01                                        ! aggregate skill intensity
        real(8) :: aggimps0, aggimps1, aggexps0, aggexps1
        real(8), dimension(sizeq) :: chiq0
        
        ! parameters to estimate factors augmenting foreign prices, foreign demand shifter and domestic consumer income; wfactor changes weight of firms for free entry counterfactual
        real(8) :: pfactor, yfactor, ycfactor, wfactor
        
        ! data>aggregates (Feenstra numbers)
        real(8), parameter :: impp_data01 = .119d0             ! change in import/total sales in domestic market (aggreg_all)
        real(8), parameter :: expe_data01 = .03d0              ! change in exports/total sales in domestic market (aggreg_all)
               
        real(8), parameter:: tariff1 = 0.12                    ! tariffs in 1994 are the same as submission, not in new data tables     
        real(8), parameter :: nfirms_data1 = 7304.d0                                                          ! number of plants (in March 8 moments)
        
    end module counterfactual
    
    ! =============================================================================================
    program MC_indentify
    ! =============================================================================================
    INCLUDE 'link_fnl_static.h'
    use parameters_model
    use numerical_libraries
    use parameters_estimate
    use parameters_quality
    use parameters_firm
    use counterfactual
    implicit none
    
    ! Variables
    ! =============================================================================================
   	real(8), dimension(n) :: xinit
    real(8) :: xinit1(n1), xinit2(n2), xinit3(n3)
    real(8) :: obj1, obj2, obj3, rd
    logical :: succ
    integer(4), parameter :: maxguess = 100          ! repeat optimization
    integer(4) :: nguess
    integer(4) :: j
    logical :: myflag
    integer :: startime, endtime, hz
	
    ! results from optimization algorithms
    real(8), dimension(n) :: xmin
    real(8), dimension(n, n+1) :: v
	real(8), dimension(n+1) :: fv
    real(8), dimension(n1, n1+1) :: v1
	real(8), dimension(n1+1) :: fv1
    real(8), dimension(n2, n2+1) :: v2 
	real(8), dimension(n2+1) :: fv2
    real(8), dimension(n3, n3+1) :: v3
	real(8), dimension(n3+1) :: fv3
    real(8) :: fmin, obj                                    ! function evaluated at minimizer
    logical :: smin                                         ! success

    real(8), dimension(n) :: xi
    real(8), dimension(n) :: interval, xtup, xtdown, intertol
    logical :: intol, isspread
    
    ! to calculate max quality at home and report difference from foreign
    real(8) :: qhmax
    integer(4) :: posqm(1)
    
    real(8) :: distl, distp         ! distance to estimated parameters (not in x)
    
    ! counterfactual
    real(8), dimension(nc) :: xcounter, xcounter0
	real(8), dimension(nc, nc+1) :: vc
	real(8), dimension(nc+1) :: fvc
    
    ! =============================================================================================
    ! preliminaries: get weights, get random variables, construct grid
    call read_weights
    call read_random
    call gen_qgrid
    call read_counter(xcounter0)
    pf = pf0; income = income0; tariff = tariff0; nfirms_data = nfirms_data0
    
    ! ---------------------------------------------------------------------------------------------
    ! get parameter estimate and standard errors
    call read_initial(xest, separ)
    call xallto123(xest, xest1, xest2, xest3)
    call xtoguessall(xest)
    intertol = 2.*separ
    interval = 5.*separ
    call gettol(xest, intertol, xtup, xtdown)
    call setdelta(xest, xtup, xtdown)
    call moments0(xest, succ)
    call saveest(xest)
   
open(7, FILE='MCcnotes_100T.txt', STATUS='REPLACE')

open(22, FILE='MCparameter100T.csv', STATUS='REPLACE')
write(22,*), 'nguess, parameter, value'
	
open(23, FILE='MCx100T.csv', STATUS='REPLACE')
write(23,*), 'nguess, xest, xnewe'
		
open(25, FILE='MCparameter_all100T.csv', STATUS='REPLACE')
write(25,*), 'nguess, opt, parameter, value'
	
! print initial parameters
nguess = 0
write(22,*), nguess, ', mu1, ', mu1
write(22,*), nguess, ', sigma1, ', sigma1
write(22,*), nguess, ', sigma2, ', sigma2
write(22,*), nguess, ', z3, ', z3
write(22,*), nguess, ', f1, ', f1
write(22,*), nguess, ', f2, ', f2
write(22,*), nguess, ', fm, ', fm
write(22,*), nguess, ', fmvar, ', fmvar
write(22,*), nguess, ', fx, ', fx
write(22,*), nguess, ', fxvar, ', fxvar
write(22,*), nguess, ', l1, ', l1
write(22,*), nguess, ', l2, ', l2
write(22,*), nguess, ', skpremium, ', skpremium
write(22,*), nguess, ', qf, ', qf
write(22,*), nguess, ', yf, ', yf
write(22,*), nguess, ', qcf, ', qcf
write(22,*), nguess, ', varpi, ', varpi
write(22,*), nguess, ', nu, ', nu

! print initial parameters
nguess = 0
write(25,*), nguess, ', ', 0, ', mu1, ', mu1
write(25,*), nguess, ', ', 0, ', sigma1, ', sigma1
write(25,*), nguess, ', ', 0, ', sigma2, ', sigma2
write(25,*), nguess, ', ', 0, ', z3, ', z3
write(25,*), nguess, ', ', 0, ', f1, ', f1
write(25,*), nguess, ', ', 0, ', f2, ', f2
write(25,*), nguess, ', ', 0, ', fm, ', fm
write(25,*), nguess, ', ', 0, ', fmvar, ', fmvar
write(25,*), nguess, ', ', 0, ', fx, ', fx
write(25,*), nguess, ', ', 0, ', fxvar, ', fxvar
write(25,*), nguess, ', ', 0, ', l1, ', l1
write(25,*), nguess, ', ', 0, ', l2, ', l2
write(25,*), nguess, ', ', 0, ', skpremium, ', skpremium
write(25,*), nguess, ', ', 0, ', qf, ', qf
write(25,*), nguess, ', ', 0, ', yf, ', yf
write(25,*), nguess, ', ', 0, ', qcf, ', qcf
write(25,*), nguess, ', ', 0, ', varpi, ', varpi
write(25,*), nguess, ', ', 0, ', nu, ', nu

 
! ====================================================
toprint = .false.
nguess = 1
CALL random_seed()

do  while (nguess <= maxguess)
    print *, '========================================================================='
    print *, 'MC simulation', nguess
    write(7,*), '========================================================================='
    write(7,*), 'MC simulation', nguess
     
    ! set new random draws
    call get_random
    
    ! set firm choices to optimal
    posqomega = eposqomega
    discreted = ediscreted
    
    ! get new guess to run algorithm on
    succ = .false.
    do while (.not. succ)
        succ = .true.
        call selectx0(interval, xinit, succ)
    end do
    !xinit = xest
    
    ! objective in initial guess
    call xallto123(xinit, xinit1, xinit2, xinit3)
    xmin1 = xinit1; xmin2 = xinit2; xmin3 = xinit3; xmin = xinit
    call calc_obj1(xinit1, obj1, succ); call calc_obj2(xinit2, obj2); call calc_obj3(xinit3, obj3)
    call distpar(xinit, distl, distp)
    print *, ' '
    print *, 'initial obj T123', obj1+obj2+obj3, obj1, obj2, obj3
    print *, '# parameters out of tolerance, before', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    write(7,*), '# parameters out of tolerance, before', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    print *, 'distance parameters, before', distl, distp
    write(7,*), 'distance parameters, l, %, before', distl, distp
    
    xmin = xinit
    call polytope1p23(xmin1, v1, fv1); xmin1 = v1(:, 1); obj = fv(1); 
    call calc_obj1p23(xmin1, xmin, obj, succ)
    call distpar(xmin, distl, distp)
    print *, ' '     
    print *, '# parameters out of tolerance, after optimization 1', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    print *, 'distance parameters ', distl, distp
    write(7,*), 'distance parameters, l, %, after 1', distl, distp
   
    ! print results of first optimization
    write(25,*), nguess, ', ', 1, ', mu1, ', mu1
    write(25,*), nguess, ', ', 1, ', sigma1, ', sigma1
    write(25,*), nguess, ', ', 1, ', sigma2, ', sigma2
    write(25,*), nguess, ', ', 1, ', z3, ', z3
    write(25,*), nguess, ', ', 1, ', f1, ', f1
    write(25,*), nguess, ', ', 1, ', f2, ', f2
    write(25,*), nguess, ', ', 1, ', fm, ', fm
    write(25,*), nguess, ', ', 1, ', fmvar, ', fmvar
    write(25,*), nguess, ', ', 1, ', fx, ', fx
    write(25,*), nguess, ', ', 1, ', fxvar, ', fxvar
    write(25,*), nguess, ', ', 1, ', l1, ', l1
    write(25,*), nguess, ', ', 1, ', l2, ', l2
    write(25,*), nguess, ', ', 1, ', skpremium, ', skpremium
    write(25,*), nguess, ', ', 1, ', qf, ', qf
    write(25,*), nguess, ', ', 1, ', yf, ', yf
    write(25,*), nguess, ', ', 1, ', qcf, ', qcf
    write(25,*), nguess, ', ', 1, ', varpi, ', varpi
    write(25,*), nguess, ', ', 1, ', nu, ', nu
     
    
    call sanneal123(xmin1, xmin1, fmin, smin)
    call calc_obj1p23(xmin1, xmin, obj, succ)
    call distpar(xmin, distl, distp)
    print *, ' '     
    print *, '# parameters out of tolerance, after optimization 2 ', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    write(7,*), '# parameters out of tolerance, after optimization 2', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    print *, 'distance parameters ', distl, distp
     write(7,*), 'distance parameters, l, %, after 2', distl, distp
         
    ! print results of first optimization
    write(25,*), nguess, ', ', 2, ', mu1, ', mu1
    write(25,*), nguess, ', ', 2, ', sigma1, ', sigma1
    write(25,*), nguess, ', ', 2, ', sigma2, ', sigma2
    write(25,*), nguess, ', ', 2, ', z3, ', z3
    write(25,*), nguess, ', ', 2, ', f1, ', f1
    write(25,*), nguess, ', ', 2, ', f2, ', f2
    write(25,*), nguess, ', ', 2, ', fm, ', fm
    write(25,*), nguess, ', ', 2, ', fmvar, ', fmvar
    write(25,*), nguess, ', ', 2, ', fx, ', fx
    write(25,*), nguess, ', ', 2, ', fxvar, ', fxvar
    write(25,*), nguess, ', ', 2, ', l1, ', l1
    write(25,*), nguess, ', ', 2, ', l2, ', l2
    write(25,*), nguess, ', ', 2, ', skpremium, ', skpremium
    write(25,*), nguess, ', ', 2, ', qf, ', qf
    write(25,*), nguess, ', ', 2, ', yf, ', yf
    write(25,*), nguess, ', ', 2, ', qcf, ', qcf
    write(25,*), nguess, ', ', 2, ', varpi, ', varpi
    write(25,*), nguess, ', ', 2, ', nu, ', nu
     
    call polytope1p23(xmin1, v1, fv1); xmin1 = v1(:, 1); obj = fv(1); 
    call calc_obj1p23(xmin1, xmin, obj, succ)
    call distpar(xmin, distl, distp)
    print *, ' '     
    print *, '# parameters out of tolerance, after optimization 3 ', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    write(7,*), '# parameters out of tolerance, after optimization 3', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    print *, 'distance parameters ', distl, distp
    write(7,*), 'distance parameters, l, %, after', distl, distp
    write(7,*), '# parameters out of tolerance, after optimization', count( (xmin<xtdown) .or. (xmin>xtup) ), count( (xmin<xtdown) .or. (xmin>xtup) )/real(n)
    write(7,*), 'parameter, in tolerance = 0, out = 1'
    
    write(22,*), nguess, ', mu1, ', mu1
    write(22,*), nguess, ', sigma1, ', sigma1
    write(22,*), nguess, ', sigma2, ', sigma2
    write(22,*), nguess, ', z3, ', z3
    write(22,*), nguess, ', f1, ', f1
    write(22,*), nguess, ', f2, ', f2
    write(22,*), nguess, ', fm, ', fm
    write(22,*), nguess, ', fmvar, ', fmvar
    write(22,*), nguess, ', fx, ', fx
    write(22,*), nguess, ', fxvar, ', fxvar
    write(22,*), nguess, ', l1, ', l1
    write(22,*), nguess, ', l2, ', l2
    write(22,*), nguess, ', skpremium, ', skpremium
    write(22,*), nguess, ', qf, ', qf
    write(22,*), nguess, ', yf, ', yf
    write(22,*), nguess, ', qcf, ', qcf
    write(22,*), nguess, ', varpi, ', varpi
    write(22,*), nguess, ', nu, ', nu

         
    ! print results of first optimization
    write(25,*), nguess, ', ', 3, ', mu2, ', mu1
    write(25,*), nguess, ', ', 3, ', sigma2, ', sigma1
    write(25,*), nguess, ', ', 3, ', sigma2, ', sigma2
    write(25,*), nguess, ', ', 3, ', z3, ', z3
    write(25,*), nguess, ', ', 3, ', f1, ', f1
    write(25,*), nguess, ', ', 3, ', f2, ', f2
    write(25,*), nguess, ', ', 3, ', fm, ', fm
    write(25,*), nguess, ', ', 3, ', fmvar, ', fmvar
    write(25,*), nguess, ', ', 3, ', fx, ', fx
    write(25,*), nguess, ', ', 3, ', fxvar, ', fxvar
    write(25,*), nguess, ', ', 3, ', l2, ', l1
    write(25,*), nguess, ', ', 3, ', l2, ', l2
    write(25,*), nguess, ', ', 3, ', skpremium, ', skpremium
    write(25,*), nguess, ', ', 3, ', qf, ', qf
    write(25,*), nguess, ', ', 3, ', yf, ', yf
    write(25,*), nguess, ', ', 3, ', qcf, ', qcf
    write(25,*), nguess, ', ', 3, ', varpi, ', varpi
    write(25,*), nguess, ', ', 3, ', nu, ', nu
     
    do j = 1, n
        write(23,*), nguess, ', ', xest(j), ', ', xinit(j)
    end do
    
    nguess = nguess + 1
end do
close(22)
close(23)
close(25)

end program MC_indentify
    
    