! -------------------------------------------------------
! calculate objective function: Match import penetration and export expansion/GDP
! -------------------------------------------------------
    subroutine calc_objc(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use crosssec
	implicit none

	real(8), dimension(nc), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    real(8), dimension(sizef) :: employment, avgwage, kmeasureB01
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguessc(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    
    call firmout1
    
    call momentc(obj)
    
    ! calculate changes in aggregate skill intensity and qtilde
    if (toprint) then
        call firmout1
        call firmout2
        call firmout3
        
        ! skill intensity
        where (discreted == 0)
        employment = 0.d0
        elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment = (myalpha/markup)*tsales/avgwage
        end where
        aggska1 = sum(employment*skilla)/sum(employment)
        
        ! qtilde
        do j = 1, sizef
             if ((discreted0(j) == 0) .or. (discreted(j) == 0)) then
                kmeasureB01(j) = 0.d0
            else
                kmeasureB01(j) = log(chiq0(posqomega(j))) +  (1.-mysigma)*log(zbarq(posqomega(j)))  - log(chiq0(posqomega0(j))) -  (1.-mysigma)*log(zbarq(posqomega0(j)))  
            end if
        end do
        dq01 = sum(kmeasureB01)/real(sizef)
                
    end if
    
    end subroutine calc_objc

    ! ============================================================
    subroutine momentc(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8) :: firm_imp, cimp
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + (expe_data01 - (aggexps1 - aggexps0))**2.d0
    
    end subroutine momentc
    
    
! -------------------------------------------------------
! save initial conditions
! -------------------------------------------------------
subroutine save_eq0
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), dimension(sizef) :: employment, avgwage
    real(8) :: firm_imp, cimp, avgkm0
    ! -------------------------------------------------------
    posqomega0 = posqomega
    discreted0 = discreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
    
    ! aggregates
    yf0 = yf
    chiq0 = chiq
                
    ! labor outcomes
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment = (myalpha/markup)*tsales/avgwage
    end where
    aggska0 = sum(employment*skilla)/sum(employment)
        
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps0 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps0 = sum(exp_share*tsales)*womega/absorption
    
    nfirms_data = nfirms_data1
    
end subroutine save_eq0

   
! =========================================================
subroutine  guesstoxc(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nc), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    !x(3) = log(ycfactor)
    
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,nc
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine guesstoxc

! =========================================================
subroutine  xtoguessc(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nc), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    ycfactor = 1.d0
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, n
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine xtoguessc

