! ===============================================================
! Given the equilibrium of the economy, derives the characteristics of firms
! no price and skill moments except for rank
! ===============================================================

    subroutine firmout1
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    implicit none
    
    integer(4) :: j, firmj, qj
    real(8), dimension(sizef) :: fsales, hsales
    real(8) :: cimp, firm_imp                   ! total consumer and firm imports
    
    ! ===============================================
    ! outcomes by firm: sales, import and export intensity only
    ! ===============================================
    do j = 1, sizef
		select case(discreted(j))
        case(0)
            outprice(j) = 0.d0
            hsales(j) = 0.d0
            fsales(j) = 0.d0
            tsales(j) = 0.d0
            imp_share(j) = 0.d0
            exp_share(j) = 0.d0
        case(1)
            outprice(j) = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = 0.d0
            tsales(j) = hsales(j) + fsales(j)
            imp_share(j) = 0.d0
            exp_share(j) = 0.d0
        case(2)
            outprice(j) = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = 0.
            tsales(j) = hsales(j) + fsales(j)
            exp_share(j) = 0.
            imp_share(j) = ( priceqf(posqomega(j))/priceqa(posqomega(j)) )**(1.-mysigma)
        case(3)
            outprice(j) = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = outprice(j)**(1.-mysigma)*chiqf(posqomega(j))
            tsales(j) = hsales(j) + fsales(j)
            exp_share(j) = fsales(j)/tsales(j)
            imp_share(j) = 0.
        case(4)
            outprice(j) = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = outprice(j)**(1.-mysigma)*chiqf(posqomega(j))
            tsales(j) = hsales(j) + fsales(j)
            exp_share(j) = fsales(j)/tsales(j)
            imp_share(j) = ( priceqf(posqomega(j))/priceqa(posqomega(j)) )**(1.-mysigma)
        end select
        
    end do

    ! Normalized sales, domestic
    ! ----------------------------------------------------------------------------
    call count_firms
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    timport = cimp + firm_imp
    absorption = sum(hsales*womega) + timport/(1.+tariff)
    nhsales = (hsales*womega/absorption)*(sizef-nexit)/nfirms_data
    
    end subroutine firmout1
    
    ! =========================================================================
    ! decompose income into labor income, fixed costs, tariff transfers and deficit
    ! =========================================================================
    subroutine decomposey
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    implicit none
    
    integer(4) :: j
    real(8), dimension(sizef) :: fsales
    real(8) :: cimp, firm_imp                   ! total consumer and firm imports
    
    real(8) :: lumpt, wbill, fxc, profit, deficit
    
    ! -----------------------------------------------
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    timport = cimp + firm_imp
    lumpt = tariff*timport/(1.+tariff)          ! tariff lumpsum transfer
    
    wbill = myalpha/markup*sum(tsales*womega)      ! wage bill
        
    fxc = 0.d0
    do j = 1, sizef
        select case(discreted(j))
        case(1)
            fxc = fxc + fxq(posqomega(j))
        case(2)
            fxc = fxc + fxq(posqomega(j)) + fmomega(j)
        case(3) 
            fxc = fxc + fxq(posqomega(j)) + fxomega(j)
        case(4)
            fxc = fxc + fxq(posqomega(j)) + fmomega(j) + fxomega(j)
        end select
    end do
    fxc = fxc*womega
    
    profit = (markup-1.)/markup*sum(womega*tsales) - fxc
    
    deficit = income - wbill - lumpt - fxc - profit
    
    print *, ' ================================================== '
    print *, ' decomposition of income, numbers, shares (firmout1.f90)'
    print *, ' wage bill   ', wbill, wbill/income
    print *, ' fixed costs ', fxc, fxc/income
    print *, ' profit      ', profit, profit/income
    print *, ' transfer    ', lumpt, lumpt/income
    print *, ' deficit     ', deficit, deficit/income
    print *, ' '
    print *, ' decomposition of total sales'
    print *, ' wages          ', wbill/sum(tsales*womega)
    print *, ' material inputs', (1-myalpha)/markup
    print *, ' fixed cost     ', fxc/sum(tsales*womega)
    print *, ' profit         ', profit/sum(tsales*womega)
    print *, ' sum            ', (wbill + fxc + profit)/sum(tsales*womega) + (1-myalpha)/markup
    print *, ' '
    print *, 'absorption check', (sum(tsales*(1.-exp_share)*womega) + timport/(1.+tariff) - absorption)/absorption
    print *, 'absorption, closed econ', absorption, income/(1.-(1.-myalpha)/markup)
    print *, 'total expenditure - T  ', income + (1-myalpha)/markup*sum(tsales*womega) - lumpt
    !print *, 'deficit, imports-exports', deficit, timport/(1.+tariff) - sum(tsales*exp_share*womega)
    !print *, 'gross import, domest. M ', firm_imp/sum(tsales*womega)/(1-myalpha)*markup, sum((1.-imp_share)*tsales*womega)/sum(womega*tsales)
    !print *, ' '
    print *, 'tsales/absorption', sum(tsales*womega)/absorption
    !stop
    end subroutine decomposey