! ===============================================================
! Given the equilibrium of the economy, derives the characteristics of firms
! ===============================================================

    subroutine firmout3
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    implicit none
    
    integer(4) :: j, firmj, qj
    real(8), dimension(sizeq,4) :: outpq        ! auxiliary variables for prices
    real(8), dimension(sizeq) :: inpq, spendq, quantity, qdiff, phivo, outpq2       
    
    ! input prices by quality level
    ! ===============================================
    zbarq = exp(qgrid*z3)       ! common component for productivity
    outpq(:,1) = markup*priceqh**(1.-myalpha)/zq(:,1)
    outpq(:,3) = markup*priceqh**(1.-myalpha)/zq(:,3)
    outpq(:,2) = markup*priceqa**(1.-myalpha)/zq(:,2)
    outpq(:,4) = markup*priceqa**(1.-myalpha)/zq(:,4)
    
    do qj = 1, sizeq                        ! firm of output quality qgrid(qj)
        qdiff = qgrid - nu*qgrid(qj)
        call phiv0(sizeq, qdiff, phivo)
        quantity = sum(nq*outpq**(-mysigma),2)*priceqh**(mysigma-1.)*phivo*zbarq
        spendq = sum(nq*outpq**(1.-mysigma),2)*priceqh**(mysigma-1.)*phivo
        inpq(qj) = sum(spendq)/sum(quantity)
    end do
     
    ! outcomes by firm
    ! ===============================================
    do j = 1, sizef
		select case(discreted(j))
        case(0)
            outprice(j) = 0.d0
            inprice(j) = 0.d0
        case(1)
            outprice(j) = markup*priceqh(posqomega(j))**(1.-myalpha)/zqomega(j, posqomega(j))
        case(2)
            outprice(j) = markup*priceqa(posqomega(j))**(1.-myalpha)/zqomega(j, posqomega(j))
        case(3)
            outprice(j) = markup*priceqh(posqomega(j))**(1.-myalpha)/zqomega(j, posqomega(j))
        case(4)
            outprice(j) = markup*priceqa(posqomega(j))**(1.-myalpha)/zqomega(j, posqomega(j))
        end select
        
        if (discreted(j) > 0) then
            outprice(j) = outprice(j)/zbarq(posqomega(j))
            inprice(j) = inpq(posqomega(j))
        end if
    end do

    end subroutine firmout3