! ==================================================
! gets moments for a given x0 and stores them as data
! -------------------------------------------------

    subroutine moments0(x, succ)
    use parameters_model
    implicit none
    
    real(8), dimension(n), intent(in) :: x
    logical, intent(out) :: succ
    
    ! ----------------------------------------------
    call xtoguessall(x)
    call gen_xfirms          ! firm productivity, fixed import and export cost
    call gen_xquality        ! foreign price and demand, consumer demand shifter
    call equilibrium(succ)
    call firmout1
    call firmout2
    call firmout3
    
    call momentsall0
    
    
    end subroutine moments0
    
    ! ====================================================
    subroutine momentsall0
    use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
	implicit none
    
    integer(4), dimension(sizef) :: qsize, qwage
    real(8), dimension(sizef) :: var1, var2
    real(8), dimension(4,4) :: wquartiles
    integer(4) :: j, jj, pospi(1)
    real(8) :: aux1
    
    ! price regressions
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector, yvector2
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: incoeff, outcoeff       ! regression results: coefficients
    
    ! quartiles of size and wage
    ! ----------------------------------------------------------------------------------
    var1 = log(nhsales)
    var2 = real(posqomega)        ! wage rank coincides with actual skill intensity (and quality)
    wquartiles = reshape(mweight(1:16), shape(qsalesw_data))
    call quartiles(var1, qsize)
    call quartiles(var2, qwage)
    call match_quartile(qwage, qsize, qsalesw_data)
    
    if (toprint) then
        print *, ' '
        print *, '========================================================='
        print *, 'joint quartile distribution of wages and size. weight, obj'
        do j = 1, 4
            do jj = 1, 4
            print *, j, jj, qsalesw_data(j,jj)
            end do
        end do   
        print *, ' ' 
    end if
    
    ! percentiles of home sales & skill intensity (unconditional distributions)
    ! ----------------------------------------------------------------------------------
    var1 = log(nhsales)       ! normalized domestic sales
    call pdistribution(var1, psize_data)
    if (toprint) then
        print *, 'distribution of normalized domestic sales'
        print *, '10% ', psize_data(1)
        print *, '25% ', psize_data(2)
        print *, '50% ', psize_data(3)
        print *, '75% ', psize_data(4)
        print *, '90% ', psize_data(5)
        print *, 'ln(90%/10%) ', psize_data(5) - psize_data(1)
        print *, ' ' 
    end if
         
    ! for each size quartile... qsize
    ! -----------------------------------------------
    var1 = log(nhsales)
    call byqsize1(qsize, exint_data, imint_data, exshr_data, imshr_data)
    if (toprint) then    
        print *, 'export intensity by quartiles of size'
        print *, '1 ', exint_data(1)
        print *, '2 ', exint_data(2)
        print *, '3 ', exint_data(3)
        print *, '4 ', exint_data(4)
        print *, " "
    
        print *, 'import intensity by quartiles of size'
        print *, '1 ', imint_data(1)
        print *, '2 ', imint_data(2)
        print *, '3 ', imint_data(3)
        print *, '4 ', imint_data(4)
        print *, " "
    
        print *, '% exporters by quartiles of size'
        print *, '1 ', exshr_data(1)
        print *, '2 ', exshr_data(2)
        print *, '3 ', exshr_data(3)
        print *, '4 ', exshr_data(4)
        print *, " "
    
        print *, '% importers by quartiles of size'
        print *, '1 ', imshr_data(1)
        print *, '2 ', imshr_data(2)
        print *, '3 ', imshr_data(3)
        print *, '4 ', imshr_data(4)
        print *, " "
    end if
    
    ! exit
    ! -----------------------------------------------
    exitrate = real(nexit)/real(sizef)
    if (toprint) print*, 'exit', exitrate
    
    ! ==========================================
    ! skills
    ! ==========================================
    call pdistribution(skillm, pskill_data)
    
    if (toprint) then
        print *, ' '
        print *, 'distribution of skill intensity'
        print *, '10% ', pskill_data(1)
        print *, '25% ', pskill_data(2)
        print *, '50% ', pskill_data(3)
        print *, '75% ', pskill_data(4)
        print *, '90% ', pskill_data(5)
        print *, 'ln(90%/10%) ', pskill_data(5) - pskill_data(1)
        print *, ' ' 
    end if
         
    ! for each size quartile... qsize
    ! -----------------------------------------------
    var1 = log(nhsales)
    call quartiles(var1, qsize)
    do j = 1, 4
        var1 = 0.d0
        where (qsize == j) var1 = 1.d0
        sksize_data(j) = sum( var1*skillm )/sum(var1)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    end do
    
    if (toprint) then    
        print *, 'skill intensity by quartiles of size'
        print *, '1 ', sksize_data(1)
        print *, '2 ', sksize_data(2)
        print *, '3 ', sksize_data(3)
        print *, '4 ', sksize_data(4)
        print *, " "
    end if
    
    ! measured skill premium
    ! -----------------------------------------------
    skpremium_data = aggskpm 
    skshare_data = aggskm
    if (toprint) then
        print *, 'skill premium & intensity'
        print *, 'prem', skpremium_data
        print *, 'int. ', skshare_data
        print *, ' ' 
    end if
    
    ! ==================================================================================
    ! price regressions
    ! ----------------------------------------------------------------------------------
    tobs = sizef - nexit
    allocate(yvector(tobs), yvector2(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! first regression: output prices on measured skill intensity
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            yvector2(jj) = log(inprice(j))
            xmat(jj,2) = skillm(j)
        end if
    end do
    ! because I get an error that prices are too high, demean yvectors
    yvector = yvector - sum(yvector)/real(tobs)
    yvector2 = yvector2 - sum(yvector2)/real(tobs)
    
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols_novar(tobs, nvar, yvector, xmat, outcoeff)      ! output prices on skill intensity
    call ols_novar(tobs, nvar, yvector2, xmat, incoeff)      ! input  prices on skill intensity
    skill_pdata(1) = outcoeff(2)
    skill_pdata(2) = incoeff(2)
    if (toprint) then
        print*, 'coefficient, ln(unit price) on (skill intensity)'
        print*, 'output', skill_pdata(1)
        print*, 'input ', skill_pdata(2)
        print *, ' '

    end if
    
    ! ln sales
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            xmat(jj,2) = log(tsales(j))
        end if
    end do
    call ols_novar(tobs, nvar, yvector, xmat, outcoeff)      ! output prices on skill intensity
    call ols_novar(tobs, nvar, yvector2, xmat, incoeff) 
    sales_pdata(1) = outcoeff(2)
    sales_pdata(2) = incoeff(2) 
    if (toprint) then
        print*, 'coefficient, ln(unit price) on ln(sales)'
        print*, 'output coef', sales_pdata(1)
        print*, 'input coeff', sales_pdata(2)
        print *, ' '
    end if
    
    deallocate(yvector, yvector2, xmat)
    
    
    ! location of quality
    ! ---------------------------------------------------------------------
    where (discreted == 0)
        var1 = 0.d0
    elsewhere
        var1 = qgrid(posqomega)
    end where
    
    meanq_data = sum(var1)/real(sizef - nexit)
    where (discreted == 0) var1 = meanq_data
    seq_data = sum((var1 - meanq_data)**2.)/real(sizef - nexit)
    seq_data = seq_data**.5
    
    
   if (toprint) then
        print*, 'mean and standard error of quality'
        print*, 'mean   ', meanq_data
        print*, 'std err', seq_data
        print *, ' '
   end if
    
    end subroutine momentsall0