! ****************************************************************
! PURPOSE: Match moments and calculate objective function
! ****************************************************************

subroutine moments3(obj)
	use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
	implicit none
    
    real(8), intent(out) :: obj
    integer(4) :: j, jj, pospi(1)
    
    ! price regressions
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector, yvector2
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: incoeff, outcoeff       ! regression results: coefficients
    
    ! ==================================================================================
    ! price regressions
    ! ----------------------------------------------------------------------------------
    obj = 0.d0
    tobs = sizef - nexit
    allocate(yvector(tobs), yvector2(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! first regression: output prices on measured skill intensity
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            yvector2(jj) = log(inprice(j))
            xmat(jj,2) = skillm(j)
        end if
    end do
    ! because I get an error that prices are too high, demean yvectors
    yvector = yvector - sum(yvector)/real(tobs)
    yvector2 = yvector2 - sum(yvector2)/real(tobs)
    
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols_novar(tobs, nvar, yvector, xmat, outcoeff)      ! output prices on skill intensity
    call ols_novar(tobs, nvar, yvector2, xmat, incoeff)      ! input  prices on skill intensity
    obj = obj + mweight(48)*(outcoeff(2) - skill_pdata(1))**2. + mweight(49)*(incoeff(2) - skill_pdata(2))**2.
    if (toprint) then
        print*, 'coefficient, ln(unit price) on (skill intensity)'
        print*, 'output', skill_pdata(1), outcoeff(2),  mweight(48)*(outcoeff(2) - skill_pdata(1))**2.
        print*, 'input ', skill_pdata(2), incoeff(2) ,  mweight(49)*(incoeff(2)  - skill_pdata(2))**2.
        print *, ' '
        print *, ' objective prices = ', obj
        print *, ' ------------------------------------------------------------'
    end if
    deallocate(yvector, yvector2, xmat)
    
end subroutine moments3
