! read initial guesses, random variables, and standard errors from data

! ==========================================================================
! initial guesses
! ==========================================================================
subroutine  read_initial(x, mypar)
	use parameters_firm
    use parameters_quality
    implicit none

    real(8), dimension(n), intent(out) :: x
    real(8), dimension(n), intent(out) :: mypar
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
    !open(1, FILE='mar17I.dat', STATUS='OLD')       
    !open(1, FILE='mar17_long.dat', STATUS='OLD')       
    open(1, FILE='may17IB.txt', STATUS='OLD')       
    
    open(2, FILE='mar17Ise.dat', STATUS='OLD')      
    
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, n
        read(1,*), x(j) 
        read(2,*), mypar(j) 
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
    close(2)
      
end subroutine  read_initial 

! diagonal of weighting matrix 
! =====================================================================    
subroutine  read_weights
    use data_moments
	implicit none
	integer(4) :: j
        
    ! ------------------------------------------
    mweight = 0.001d0      ! some moments, like US data and exit do not have data
    mweight(51) = 0.001d0
    mweight(52) = 0.01d0
    
    ! standard errors from bootstrap
    open(1, FILE='mse_2016.dat', STATUS='OLD')
    do j = 1, nboot
        read(1,*), mweight(j)       ! standard error of moments
    end do
    if (nweight>=53) read(1,*), mweight(53)            ! aggregate measured skill intensity
    close(1)
    
    ! moments from regression results on prices
    mweight(j:j+1) = skill_sepdata
    mweight = 1./(mweight)**2.
    
    mweight = 1.d0; mweight(17:21) = 1./100.**2
end subroutine read_weights

subroutine read_random
    use numerical_libraries
    use random_draw
    implicit none    
    
    integer(4) :: j, j1
    real(8) :: rd
    
    ! -----------------------------------------------
    !open(1, FILE='runiform.dat', STATUS='OLD')
    open(1, FILE='runiform_long.dat', STATUS='OLD')
    
    ! firm productivity
    do j1 = 1, 2
        do j = 1, sizef
            read(1,*), rd
		    random_firms(j,j1) = DNORIN(rd)       ! transforms uniform into standard normal
        end do
	end do
        
    ! importing and exporting fixed costs
    do j1 = 1, 2
        do j = 1, sizef
            read(1,*), rd
		    random_fxcost(j,j1) = DNORIN(rd)	 ! transforms uniform into standard normal
        end do
	end do
    
    ! measurement error in skills
    do j = 1, sizef
        read(1,*), rd
		random_skill(j) = rd					  ! truncated logistic
	end do
	close(1)
     
end subroutine read_random

subroutine  readx01(x,xnum)
	use parameters_firm
    use parameters_quality
    implicit none

    real(8), dimension(n), intent(out) :: x
    integer(4), intent(in) :: xnum
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
    if (xnum == 0) then
        open(1, FILE='currentx0.txt', STATUS='OLD')      ! change in weight of skill intensity + bigger punishment for lack of spread in skill intensity
    else
        open(1, FILE='tresult01.txt', STATUS='OLD')      ! change in weight of skill intensity + bigger punishment for lack of spread in skill intensity
    end if
    read(1,*), aux
    read(1,*), auxl
	do j = 1, n
        read(1,*), x(j) 
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  readx01 


! =================================================================================================

subroutine get_random
    use numerical_libraries
    use random_draw
    implicit none    
    
    integer(4) :: j, j1
    real(8) :: rd
    
    ! -----------------------------------------------
    
    ! firm productivity
    do j1 = 1, 2
        do j = 1, sizef
            call random_number(rd)
		    random_firms(j,j1) = DNORIN(rd)       ! transforms uniform into standard normal
        end do
	end do
        
    ! importing and exporting fixed costs
    do j1 = 1, 2
        do j = 1, sizef
            call random_number(rd)
		    random_fxcost(j,j1) = DNORIN(rd)	 ! transforms uniform into standard normal
        end do
	end do
    
    ! measurement error in skills
    do j = 1, sizef
        call random_number(rd)
		random_skill(j) = rd					  ! truncated logistic
	end do
	close(1)
     
end subroutine get_random



! =================================================================================================
subroutine  read_counter(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(nc), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    	
    ! ELASTIC LABOR, mar 2017
    open(1, FILE='counterEL.txt', STATUS='OLD')            ! main benchmark 
    
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nc
        read(1,*), x(j)  
    end do
	!do j = 1, sizef
	!	read(1,*), posqomega(j)
	!end do
	!do j = 1, sizef 
	!	read(1,*), discreted(j)
	!end do
	!do j = 1, sizeq
	!	read(1,*), priceqh(j)
 !   end do
	!do j = 1, sizeq
	!	read(1,*), intermediatechi(j)
	!end do
	close(1)
      
end subroutine  read_counter 