! ------------------------------------------------------
! selects an initial guess with
!   - well spread quality choices
!   - skill intensity increasing in region where quality is chosen
! ------------------------------------------------------

    subroutine selectx0(xc, interval, x0, succ)
    use parameters_model
    implicit none
    
    real(8), dimension(n), intent(in) :: xc, interval
    real(8), dimension(n), intent(out) :: x0
    logical, intent(out) :: succ
    real(8) :: x1(n1), x2(n2), x3(n3), OBJ
    real(8), dimension(n1, n1+1) :: v01
	real(8), dimension(n1+1) :: fv01
    real(8), dimension(n2, n2+1) :: v02
	real(8), dimension(n2+1) :: fv02
    logical :: isspread
    real(8), dimension(n) :: myinterval
    integer(4) :: iter
    ! --------------------------------------------
    
    
    ! spread of qualities
    ! -----------------------------------------------
    isspread = .false.
    iter = 0
    !myinterval = 0.d0
    do while (.not. isspread) 
        call newguess(xc, interval, x0)
        call xallto123(x0, x1, x2, x3)
        call spread1(x1, isspread, succ)
        iter = iter + 1
        print *, 'search for spread in q', iter, isspread
    end do
    
        
    ! calculate equilibrium and firm1 for x1
    ! -----------------------------------------------
    call xtoguess1(x1)
    call gen_xfirms          ! firm productivity, fixed import and export cost
    call gen_xquality        ! foreign price and demand, consumer demand shifter
    call equilibrium(succ)
    call firmout1
    
    ! spread of skill choices
    ! -----------------------------------------------
    call spread2(x2, obj)
    if (obj>0.4) then
        call polytope02(x2, v02, fv02)
        x2 = v02(:, 1)
        print *, 'spread 2 of initial guess after polytope', fv02(1)
    else
        print *, 'spread 2 of initial guess no polytope', obj
    end if
    
    call x123toall(x0, x1, x2, x3)
    
    end subroutine selectx0
    
    ! ================================================
    subroutine spread1(x1, isspread, succ)
    use parameters_estimate
    use parameters_quality
    use parameters_firm
    implicit none
    
    real(8), dimension(n1), intent(in) :: x1
    logical, intent(out) :: isspread
    logical, intent(out) :: succ
    integer(4) :: j, jj, pospi(1), mgap, isnq
    real(8) :: tnq(sizeq), aux1
    ! -------------------------------------
    isspread = .true.
    call xtoguess1(x1)
    ! calculate equilibrium
    call gen_xfirms          ! firm productivity, fixed import and export cost
    call gen_xquality        ! foreign price and demand, consumer demand shifter
    call equilibrium(succ)
    
    ! no corners
    jj = count( (posqomega == 1) .or. (posqomega == sizeq) ) + .2*count( ((discreted>0) .and. (posqomega <= 5)) .or. (posqomega >= sizeq - 5) )  
    if (real(jj)/real(sizef-nexit) > 0.01) isspread = .false.
   ! print *, real(jj)/real(sizef-nexit), (real(jj)/real(sizef-nexit) > 0.01), isspread
    
    ! spread
    pospi = maxloc(sum(nq,2))
    jj = count( (posqomega >= pospi(1)-2) .and. (posqomega <= pospi(1)+2) ) 
    if (real(jj)/real(sizef-nexit) > 0.3) isspread = .false.
   ! print *, real(jj)/real(sizef-nexit), (real(jj)/real(sizef-nexit) > 0.3), isspread
    
       
    end subroutine spread1
    
    ! ================================================
    subroutine spread2(x2, obj)
    use parameters_estimate
    use parameters_quality
    use parameters_firm
    use crosssec
    implicit none
    
    real(8), dimension(n2), intent(in) :: x2
    real(8), intent(out) :: obj
    real(8) :: skap(5)
    ! -------------------------------------
    call xtoguess2(x2)
    call firmout2
    
    call pdistribution(skilla, skap)
    obj = 0.d0
    obj = obj + 1. - (skap(5) - skap(1))/(skillq(sizeq) - skillq(1))
    obj = obj + 100.*max(0.d0, 0.008d0 - (skap(2) - skap(1)))
    obj = obj + 50.*max(0.d0, 0.03d0 - (skap(5) - skap(4)))
    
    end subroutine spread2