!****************************************************************************
!
!  PROGRAM: MC_counter
!
!  PURPOSE:  Do Monte Carlo simulations to check whether functions P and chi 
!  are continuous in tariffs, y*, p*
!
!****************************************************************************
module parameters_model
		real(8), parameter :: mysigma = 5.0d0				                ! elasticity of substitution across sectors
        real(8), parameter :: markup = mysigma/(mysigma-1.d0)
        real(8), parameter :: myalpha = 0.7d0			            	    ! Cobb-Douglas weight on labor
        real(8), parameter :: tariff0 = .32d0	
        real(8), parameter :: income0 = 1.d0
        real(8), parameter :: sigmal = 1.6d0                                ! elasticity of substitution between skilled and unskilled workers
		integer(4), parameter :: sizef = 100000
        real(8) :: womega                                                   ! firm weight (1/sizef), changes only with free entry counterfactual
        integer(4), parameter :: sizeq = 201					            ! size of q grid and number of firm draws
        real(8), parameter :: qlow = 0., qup = 10.d0                        ! lower and upper bound of existing varieties: nq varieties exist for any q\in [qlow,qbar]; low = (1.d0/(sizeq-1.))
        integer(4), parameter :: n = 18                                     ! number of parameters to be estimated. One extra is l3 which is calibrated (no standard errors)
        integer(4), parameter :: n1 = 13, n2 = 4, n3 = 1                    ! number of parameters to be estimated in each stage
        logical :: toprint, pflag, toprint2
        real(8), parameter :: nfirms_data0 = 6710.d0                                                         ! number of plants (in March 8 moments)
        integer(4) :: nexit, nimport, nexport                               ! number of firms exiting, importing and exporting
        integer(4), parameter :: nc = 2                                     ! number of parameters of counterfactual
		integer(4), parameter :: nci = 4                                    ! ... with inelastic labor
        
end module parameters_model

	! ----------------------------------------------------------------------------
	 module parameters_quality
     use parameters_model
        real(8) :: pca, pch, pcf                                        ! consumer's price indices---total, domestic and foreign only
	    real(8), dimension(sizeq) :: qgrid
	    real(8), dimension(sizeq) :: chiq, chiqf                        ! demand shifter in Home and Foreign
        real(8), dimension(sizeq) :: priceqa, priceqh, priceqf          ! price index all goods, home and foreign goods only
		real(8), dimension(sizeq) :: consumptionchi, intermediatechi
		real(8), dimension(sizeq, 4) :: rfirmp                          ! price component for each firm type; representative firm p
		real(8), dimension(sizeq, 4) :: nq, zq                  		! zq=1/aq is the productivity of a representative firm (in the Melitz sense) of quality q.
		real(8), dimension(sizeq) :: fxq	    	                    ! fixed cost of production for each firm-quality
        real(8), dimension(sizeq) :: wq	    	                    ! labor cost index for firm of quality q
     end module parameters_quality
     
	! ----------------------------------------------------------------------------
	module parameters_firm
        use parameters_model
		integer(4), dimension(sizef) :: posqomega			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	end module parameters_firm

	! ----------------------------------------------------------------------------
	module parameters_qf
		use parameters_model
		real(8), dimension(sizef) :: qomega						        ! choice of quality (level)
		real(8), dimension(sizef, sizeq) :: zqomega		        		! productivity of each firm 
        real(8), dimension(sizef) :: fxomega, fmomega                   ! firm-specific import and export cost
        real(8), dimension(sizef,2) :: skomega                          ! firm-specific measurement error of skilled workers
    end module parameters_qf
    
    ! ----------------------------------------------------------------------------
	module random_draw
	    use parameters_model
        real(8), dimension(sizef,2) :: random_firms
        real(8), dimension(sizef,2) :: random_fxcost
        real(8), dimension(sizef) :: random_skill
    end module random_draw  
    
    ! ----------------------------------------------------------------------------
	module crosssec
    use parameters_quality
    use parameters_firm
        real(8), dimension(sizef) :: imp_share, exp_share              ! share of materials imported
        real(8), dimension(sizef) :: inprice, outprice
        real(8), dimension(sizef) :: nhsales
        real(8), dimension(sizef) :: skilla, skillm                     ! actual and measured skill intensity
        real(8), dimension(sizeq) :: zbarq                              ! quantity equivalence
        real(8), dimension(sizeq) :: inpq                               ! input price
        real(8), dimension(sizeq) :: skillq                             ! skill intensity by quality level
        real(8), dimension(sizeq) :: wageq                              ! average wage of a firm with quality q
        real(8) :: aggskm, aggskpm, aggska                                      ! aggregate measured skill intensity and premium
        real(8) :: timport, absorption
        real(8), dimension(sizef) :: tsales
    end module crosssec
    
    ! ----------------------------------------------------------------------------
    module parameters_polytopec
        use parameters_model
        integer(4), parameter :: np1 = nc + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), parameter :: usual_delta = 0.1d0
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytopec
    ! ----------------------------------------------------------------------------
    module parameters_polytopeci        ! inelastic labor
        use parameters_model
        integer(4), parameter :: np1 = nci + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), parameter :: usual_delta = 0.1d0
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytopeci
    ! ----------------------------------------------------------------------------
    module parameters_estimate
		real(8) :: mu1, sigma1                                          ! z1(omega) log normal mean and variance
        real(8) :: sigma2                                               ! z2(omega) normal, variance. Mean is zero.
        real(8) :: z3                                                   ! parameter of zbar
        real(8) :: f1, f2                                               ! fixed cost parameters
        real(8) :: fm, fx                                               ! fixed costs to import and export
        real(8) :: fmvar, fxvar                                         ! variance of fixed import and export cost
        real(8) :: l1, l2, l3
        real(8) :: qf                                                   ! quality of representative foreign firm 
        real(8) :: qcf                                                  ! quality of representative foreign consumer
        real(8) :: skpremium                                            ! skill premium wageh/wagel, where wagel = 1, normalized above
        real(8) :: yf 
        real(8) :: varpi                                                ! variance of firm-specific classification error
        real(8) :: nu                                                   ! complementarity parameter in production
        real(8) :: pf
        real(8) :: tariff
        real(8) :: nfirms_data
        real(8) :: income
    end module parameters_estimate

	! ----------------------------------------------------------------------------
	! fortran_codes/data/february 2016/new baseline Jan 27.xlsx
    module data_moments
		integer(4), parameter :: sizecoef = 4
		real(8), dimension(4,4), parameter :: qsalesw_data = reshape((/.143, .073, .029, .005, &          ! ln(wage per worker), ln(domestic sales)
                                                                       .071, .090, .069, .020, &
                                                                       .030, .066, .097, .057, &
                                                                       .005, .020, .056, .168/), shape(qsalesw_data))
        ! size data need to be adjusted for absorption from Feenstra ***************************************         
        real(8), dimension(5), parameter:: psize_data  = (/ -12.587, -11.953, -11.073, -9.838, -8.409 /)    ! ln(domestic sales) distribution of plants 10, 25, 50, 75, 90 percentiles, with feenstra adjustment        
        real(8), dimension(5), parameter:: pskill_data = (/ 0.059, 0.125, 0.217, 0.343, 0.500 /)            ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(7), parameter:: pkimp_data = (/ -5.189, -3.558, -1.392, 0.432, 1.973, 3.437,4.390 /)            ! k-measure kimp_ait distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(4), parameter:: sksize_data = (/ .196, .225, .265, .336 /)                       ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(4), parameter :: exint_data = (/ 1.5, 1.0, 1.6, 2.6 /) / 100.                    ! average export intensity by quartile of domestic sales 
        real(8), dimension(4), parameter :: imint_data = (/ 1.9, 3.7, 7.6, 18.8 /) / 100.                   ! average import intensity by quartile of domestic sales
        real(8), dimension(4), parameter :: exshr_data = (/ 2.7, 3.6, 8.8, 27.6 /)/ 100.                    ! % exporters by quartile of domestic sales
        real(8), dimension(4), parameter :: imshr_data = (/ 7.4, 12.4, 24.9, 58.3 /) / 100.                 ! % importers by quartile of domestic sales
        real(8), dimension(4), parameter :: kimp_ai = (/ -0.808, -0.136, 0.418, 1.101 /)                    ! Khandelwal's measure of quality; out of sample; median set to 0 or average to 1?
        real(8), parameter :: skshare_data = 0.290d0                                                        ! aggregate measured skill share
        real(8), parameter :: skpremium_data = 1.586d0                                                      ! skill premium = (average wage of white/average wage of blue-collar workers) 
        real(8), parameter :: exitrate = 0.087d0                                                            ! historical exit rate
        real(8), parameter :: imps_data = 0.162d0, exps_data = 0.045d0                                      ! aggregate import and export shares, using Feenstra numbers, see aggregate folder
        
        ! data on USA skill premium and skill intensity for estimating bounds (Autor, Katz, Krueger, QJE1998) See calculation in myaggregates_colombiagov.xlsx
        ! ----------------------------------------------------------------------------
        real(8), parameter :: USAskint =  .229d0                          ! table 1 appendix page 1209, average between 1990 and 1980 census, college equivalents
        
        ! price regressions
        ! ----------------------------------------------------------------------------
        ! coefficients
        real(8), dimension(2), parameter :: skill_pdata = (/ 0.369d0, 0.187d0 /)
        real(8), dimension(2), parameter :: sales_pdata = (/ 0.0194d0, 0.0176d0 /)
        real(8), dimension(2), parameter :: kimpai_pdata = (/ 0.177d0, 0.0563d0 /)
        
        ! standard errors
        real(8), dimension(2), parameter :: skill_sepdata = (/ 0.037d0, 0.019d0 /)
        real(8), dimension(2), parameter :: sales_sepdata = (/ 0.0055d0, 0.0026d0 /)
        real(8), dimension(2), parameter :: kimpai_sepdata = (/ 0.002d0, 0.0052d0 /)
        
        ! ----------------------------------------------------------------------------
        integer(4), parameter :: nweight = 53
        real(8), dimension(nweight) :: mweight   ! weighting matrix is a diagonal matrix, and it should be the inverse of variance
        integer(4), parameter :: nboot = 47
        
    end module data_moments    

    
    ! ----------------------------------------------------------------------------
	module counterfactual
        use parameters_model
		integer(4), dimension(sizef) :: posqomega0, posqomega1			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted0, discreted1			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	    real(8), dimension(sizef) :: imp_share0, exp_share0, imp_share1, exp_share1 ! share of materials imported
        real(8), dimension(sizef) :: nhsales0, ntsales0, tsales0
        real(8), dimension(sizef) :: skilla0, skillm0
        real(8), dimension(sizef) :: firml1, firml0                                 ! employment
        real(8) :: aggskm0, aggskpm0 !, aggskm1, aggskpm1                           ! aggregate measured skill intensity and premium
        real(8) :: aggska0 !, aggska1                                               ! aggregate skill intensity
        real(8) :: absorption0 !, absorption1
        real(8) :: aggimps0, aggimps1, aggexps0, aggexps1
        integer(4) :: nexit0, nexit1
        real(8), dimension(sizeq) :: chiq0, chiqf0                                  ! used to calculate changes in K-measures
        real(8), dimension(sizeq) :: priceqh0, priceqa0                             ! used to calculate changes in input prices, inform about the mechanism
        real(8), dimension(sizeq, 4) :: nq0
        real(8), parameter :: pf0 = 1.d0
        real(8) :: yf0, yf1, pf1, skpremium0, skpremium1 
        real(8), dimension(sizef) :: kmeasure0, kmeasure01, kmeasureB01              ! k-measureB01 is the counterfactual measure of quality given demand in period 0. k-measure01  is the hypothetical change in sales, given demand in period 0 (see counterEL) 
        real(8) :: sekm0
        real(8), dimension(sizeq) :: zbarq0
        real(8), dimension(sizef) :: outprice0
                
        ! parameters to estimate factors augmenting foreign prices, foreign demand shifter and domestic consumer income; wfactor changes weight of firms for free entry counterfactual
        real(8) :: pfactor, yfactor, ycfactor, wfactor
        real(8) :: wl0, wh0
        real(8) :: wl1, wh1
        real(8) :: Lh0, Ll0     ! aggregate labor demand, initial
        real(8) :: Lh1, Ll1 
        real(8) :: allsales0    ! total sales targetted under calcobjce
        real(8) :: womega0      ! firm weight
        real(8) :: eprofit0     ! expected profit
        
        ! data>aggregates (Feenstra numbers)
        real(8), parameter :: impp_data01 = .119d0             ! change in import/total sales in domestic market (aggreg_all)
        real(8), parameter :: expe_data01 = .03d0              ! change in exports/total sales in domestic market (aggreg_all)
        
        !alternative 2: export expansion
        !real(8), parameter :: expe_data01 = impp_data01       ! change in exports/total sales in domestic market (aggreg_all)
                
        real(8), parameter:: tariff1 = 0.12                    ! tariffs in 1994 are the same as submission, not in new data tables     
        
        ! Updated from "new baseline jan27"
        real(8), dimension(5), parameter:: psize_data01  = (/ -.16, -.10, -.06, 0.04, -0.08  /)               ! ln(domestic sales) distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(5), parameter:: pskill_data01 = (/ 0.032, 0.042, 0.060, 0.092, 0.137/)             ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(7), parameter:: pkmeas_data01 = (/-0.11, 0.32,0.52, 0.79, 1.09, 1.38, 1.59/)             ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), parameter :: nfirms_data1 = 7304.d0                                                          ! number of plants (in March 8 moments)
        real(8), parameter :: aggimps_data0 = 0.162d0, aggimps_data1 = 0.281d0, aggexps_data0 = 0.045, aggexps_data1 = 0.075d0
        real(8) , parameter:: aggskm_data0 = 0.290d0, aggskm_data1 = 0.354d0                                  ! aggregate skill intensity
        
        
    end module counterfactual
    
    ! *******************************************************************
    program MC_counter
    ! *******************************************************************
    INCLUDE 'link_fnl_static.h'
    use numerical_libraries
    use parameters_estimate
    use parameters_quality
    use counterfactual
    
    use parameters_qf
    implicit none
    
    ! Variables
    ! =============================================================================================
    logical :: succ
    integer(4), parameter :: maxguess = 20          ! repeat optimization
    integer(4) :: nguess
    integer(4) :: j
    logical :: myflag
	
    ! results from optimization algorithms
    real(8), dimension(n) :: xmin
    real(8) :: fmin, obj                                 ! function evaluated at minimizer
    logical :: smin                                 ! success

    ! counterfactual
    real(8), dimension(nc) :: xcounter
	real(8), dimension(nc, nc+1) :: vc
	real(8), dimension(nc+1) :: fvc
    
    real(8), dimension(nci) :: xcounteri
	real(8), dimension(nci, nci+1) :: vci
	real(8), dimension(nci+1) :: fvci
    
    real(8) :: pfc, yfc, wlc, whc
    real(8), dimension(sizeq) :: wq0, aux1, phihl
    
    ! slow movement in parameters
    ! -------------------------------------------------------
    integer(4), parameter :: maxiter = 100    ! number of iterations
    integer(4) :: niter                    ! current iteration
    
    ! iterate on finding optimal
    real(8) :: opfactor, oyfactor, pdown, ydown, rd1, rd2
    ! =============================================================================================
    ! preliminaries: get weights, get random variables, construct grid
    ! ----------------------------------------------------------------
    call read_weights
    call read_random
    call gen_qgrid
    wq = 1.d0        ! Initial labor costs are normalized to one. In counterfactual with inelastic wages, it changes
    womega = 1.d0/sizef     
    wl1 = 1.d0
    
    ! get initial guess for parameters and calculate objective function
    ! ----------------------------------------------------------------
    intermediatechi = 1.d0; priceqh = 1.d0
    myflag = .true.
    call read_initial(xmin, myflag)
    call xtoguessall(xmin) !; call print_parameters
    toprint = .false.
    call calc_obj(xmin, obj, succ)
    call outofsample
    
    ! calculate initial counterfactual - elastic labor
    ! ----------------------------------------------------------------
    toprint = .false.
    call save_eq0
    call read_counter(xcounter)
    call xtoguessc(xcounter)      
    pfc = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yfc = yf0*yfactor
        
    print *, ' '
    print *, ' --------------------------------------------- '
    !call calc_objc(xcounter, obj, succ); print *, 'objective counter = ', obj
    print *, ' '
    print *, 'ELASTIC LABOR: initial, counter, counter/initial'
    print *, 'pf     ', pf0, pfc, pfc/pf0
    print *, 'yf     ', yf0, yfc, yfc/yf0
    
    ! ================================================================================
    ! Monte Carlo simulations
    ! ================================================================================
    ! 1. ELASTIC LABOR: slowly move from estimated model to counterfactual and record changes in chi and p to analyze in stata
    ! ----------------------------------------------------------------
    if (.true.) then
    open(11, FILE='mc_counter1.csv', STATUS='REPLACE')
	open(12, FILE='mc_counter2.csv', STATUS='REPLACE')
	open(13, FILE='mc_counter3.csv', STATUS='REPLACE')
    write(11,*), "quality, niter, priceqh"
    write(12,*), "quality, niter, chiq"
    write(13,*), "quality, niter, vare"
    
    do niter = 0, maxiter
        pf = (niter*pfc + (maxiter - niter)*pf0)/real(maxiter)
        yf = (niter*yfc + (maxiter - niter)*yf0)/real(maxiter)
        call gen_xquality
        call equilibrium(succ)
        print *, niter, succ, priceqh(sizeq)
        do j = 1, sizeq
			write(11,*), qgrid(j), ", ", niter, ", ", priceqh(j)
            write(12,*), qgrid(j), ", ", niter, ", ", chiq(j)
            write(13,*), qgrid(j), ", ", niter, ", ", real(niter)/real(maxiter)
        end do
    end do
    close(11); close(12); close(13)
    stop
    end if
        
    ! ===========================================================================
    ! INELASTIC LABOR
    ! ===========================================================================
    ! calculate initial counterfactual - elastic labor
    ! ----------------------------------------------------------------
    toprint = .false.
    call save_eq0
    call read_counteri(xcounteri)
    call xtoguessci(xcounteri)      
    pfc = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yfc = yf0*yfactor
    wlc = wl1
    whc = wh1
        
    print *, ' '
    print *, ' --------------------------------------------- '
    !call calc_objc(xcounter, obj, succ); print *, 'objective counter = ', obj
    print *, ' '
    print *, 'INELASTIC LABOR: initial, counter, counter/initial'
    print *, 'pf     ', pf0, pfc, pfc/pf0
    print *, 'yf     ', yf0, yfc, yfc/yf0
    print *, 'wl     ', wl0, wlc, wlc/wl0
    print *, 'wh     ', wh0, whc, whc/wh0
    
    
    ! 3. INELASTIC LABOR: slowly move from estimated model to counterfactual and record changes in chi and p to analyze in stata
    ! ----------------------------------------------------------------
    if (.true.) then
    open(31, FILE='mci_counter1.csv', STATUS='REPLACE')
	open(32, FILE='mci_counter2.csv', STATUS='REPLACE')
	open(33, FILE='mci_counter3.csv', STATUS='REPLACE')
    write(31,*), "quality, niter, priceqh"
    write(32,*), "quality, niter, chiq"
    write(33,*), "quality, niter, vare"
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
        
    do niter = 0, maxiter
        pf = (niter*pfc + (maxiter - niter)*pf0)/real(maxiter)
        yf = (niter*yfc + (maxiter - niter)*yf0)/real(maxiter)
        wl1 = (niter*wlc + (maxiter - niter)*wl0)/real(maxiter)
        wh1 = (niter*whc + (maxiter - niter)*wh0)/real(maxiter)
        skpremium = wh1/wl1
        
        wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
        wq = wq/wq0
        call gen_xquality
        call equilibrium(succ)
        print *, niter, succ, priceqh(sizeq)
        do j = 1, sizeq
			write(31,*), qgrid(j), ", ", niter, ", ", priceqh(j)
            write(32,*), qgrid(j), ", ", niter, ", ", chiq(j)
            write(33,*), qgrid(j), ", ", niter, ", ", real(niter)/real(maxiter)
        end do
    end do
    close(31); close(32); close(33)
    end if
    
    end program MC_counter

