! *******************************************************************************************
! contains all subroutines generating functions and variables that depend on parameter values but not on equilibrium choices
! *******************************************************************************************

! ----------------------------------------------------------------------------------------
! firm variables
! ----------------------------------------------------------------------------------------
    subroutine gen_xfirms
        !use parameters_quality
        use random_draw
        use parameters_qf
        use parameters_estimate
        use parameters_quality
        implicit none
        
        real(8), dimension(sizef) :: z1, z2
        integer(4) :: j, q
    
        ! productivity
        ! ======================================================================
        z1 = exp( mu1 + sigma1*random_firms(:,1) )       ! z1 is log-normal and random_firms is standard normal
        z2 = sigma2*random_firms(:,2)                    ! z2 is normal
        do q = 1, sizeq
            zqomega(:,q) = max( 0.d0, z1*(1. + z2*qgrid(q)) )
        end do
        
        ! import and export costs
        ! ======================================================================
        fxomega = exp(fx + random_fxcost(:,1)*fxvar)
        fmomega = exp(fm + random_fxcost(:,2)*fmvar)
     
    end subroutine gen_xfirms
    
    
! ----------------------------------------------------------------------------------------
! quality variables
! ----------------------------------------------------------------------------------------
    subroutine gen_xquality
        use parameters_estimate
        use parameters_quality
        implicit none
        
        real(8), dimension(sizeq) :: qvaux, qvaux2
        integer(4) :: j, q
    
        ! fixed production cost
        ! ======================================================================
        fxq = max(0.d0, f1 + f2*qgrid)
        
        ! foreign demand
        ! ======================================================================
        qvaux = qgrid - qcf
        call phiv(sizeq, qvaux, 1.d0, chiqf)        
        chiqf = chiqf*yf
        
        ! Price index of foreign goods
        ! ======================================================================
        ! evaluation according to firm with output qgrid
        qvaux = qf - qgrid
        call phiv(sizeq, qvaux, nu, priceqf)
        priceqf = pf*priceqf**(1./(1.-mysigma))              ! price of representative foreign firm is 1
        
        ! consumer evaluation, pcf
        call phir(qf, 1.d0, pcf)
        pcf = pf*pcf**(1./(1.-mysigma))

    end subroutine gen_xquality