! auxiliary files for equilibrium in autarky
    ! ***********************************************************************************************
! contains auxiliary subroutines to find equilibrium
!
! nqzq : finds representative firm's mass and productivity
! prices: finds price indices
! chiq: demand function chi
! ***********************************************************************************************


! ================================================================================================
subroutine  nqzqAUT
	use parameters_quality
    use parameters_estimate
    use parameters_firm
    use parameters_qf
    use autarky
    implicit none

	integer(4) :: j, jj
	! ----------------------------------------------------

    nqa = 0.d0
    zqa = 0.d0
    do j = 1, sizef
        if (discreted(j)>0) then
            nqa(posqomega(j)) = nqa(posqomega(j)) + womega
	        zqa(posqomega(j)) = zqa(posqomega(j)) + (zqomega(j, posqomega(j))**(mysigma-1.))*womega
 	    end if
    end do
    where(nqa>0.d0) zqa = (zqa/nqa)**(1./(mysigma-1.))
    
    if ( any(isnan(zq)) .or. any(isnan(zq)) .or. any(isnan(zq*0.))  .or. any(isnan(nq*0.)) ) then
        print *, 'zq, nq is Nan in auxequilibrium.f90, calc_nqzqd'
        stop
    end if
        
end subroutine  nqzqAUT


! ================================================================================================
subroutine pricesAUT
    use parameters_quality
    use parameters_estimate
    use autarky
    implicit none
    
    ! ----------------------------------------------------
    real(8), parameter :: tolp = 1.e-15
	integer(4), parameter :: maxiter = 500
	integer(4) :: iter, j, outq
	real(8) :: error
	real(8), dimension(sizeq) :: oldpq, qvdiff, phivo
	! ----------------------------------------------------
    
    iter = 1
    error = 10.*tolp
    
    ! firm prices
    ! =======================================================================================
    price_loop: do
        ! stop commands --------------------------------------
        if (error <= tolp) then
            !print *, 'prices converged', iter
            exit price_loop
	    elseif (iter >= maxiter) then
    		print *, "prices did not converge ********************", error
	    	!stop
        end if
        ! ----------------------------------------------------
        
        ! calculate auxiliary prices from new price guess
        priceqa = priceqh
        rfirmpa = markup*priceqh**(1.-myalpha)*wq**myalpha/zqa(:)
        
        ! calculate new priceqh
        oldpq = priceqh
        outq_loop: do outq = 1, sizeq
            qvdiff = qgrid - nu*qgrid(outq)
            call phiv0(sizeq, qvdiff, phivo)
            priceqh(outq) = ( sum( rfirmpa**(1.-mysigma)*nqa* phivo ) )**(1./(1.-mysigma))
        end do outq_loop
    
        ! get errors
        error = sum(abs(oldpq - priceqh))
        iter = iter + 1
    end do price_loop
    
    ! use final price to recalculate auxiliary prices and consumer prices
    priceqa = priceqh
    rfirmpa = markup*priceqh**(1.-myalpha)*wq**myalpha/zqa
    
    ! consumer prices
    ! =======================================================================================
    call phiv0(sizeq, qgrid, phivo)
    pch = ( sum ( rfirmpa**(1.-mysigma)*nqa * phivo ) )**(1./(1.-mysigma))
    pca = pch
       
end subroutine pricesAUT

! ================================================================================================
subroutine chiqAUT
    use parameters_quality
    use parameters_estimate
    use autarky
    implicit none
    
    ! ----------------------------------------------------
    real(8), parameter :: tolchi = 1.e-10
	integer(4), parameter :: maxiter = 1000
	integer(4) :: iter, j, outq
	real(8) :: error, rd
	real(8), dimension(sizeq) :: oldchi, qvdiff, phivo
    real(8), dimension(sizeq) :: rfirmaux
	
    ! consumption
    ! ----------------------------------------------------
    call phiv0(sizeq, qgrid, phivo)
    consumptionchi = phivo*pch**(mysigma-1.d0)*income
    
    ! intermediates
    ! ================================================================
    iter = 1
    error = 100.*tolchi
    
    chi_loop : do
        !stop commands
    	! --------------------------------
    	if (error <= tolchi) then
            !print *, 'chi converged   ', iter
    		exit chi_loop
    	elseif ( (iter >= maxiter) .or. (isnan(error)) .or. (any(isnan(intermediatechi/intermediatechi) .or. (isnan(intermediatechi)) )) ) then
    		print *, "chi did not converge", iter, error
            stop
        end if
        oldchi = intermediatechi   
    
        ! calculate total chi and rfirmaux = P^(sigma-1) * rt(representative firm) * weightnq * constant
        chiq = consumptionchi + intermediatechi
        rfirmaux = nqa * rfirmpa**(1.-mysigma) * chiq * priceqh**(mysigma-1.)
        rfirmaux = rfirmaux*(1.-myalpha)/markup
        
        ! new guess for chiq
        outq_loop2: do outq = 1, sizeq
            qvdiff = qgrid(outq) - nu*qgrid
            call phiv0(sizeq, qvdiff, phivo)
            intermediatechi(outq) = sum( rfirmaux * phivo )
        end do outq_loop2
    
        if (error>50.) then
            call random_number(rd) 
            intermediatechi = (0.7 + rd*0.3)*intermediatechi + (0.3 - rd*0.3)*oldchi 
        end if
                        
        error = maxval(abs(intermediatechi - oldchi)/intermediatechi) + maxval(abs(intermediatechi - oldchi))
        iter = iter + 1
    end do chi_loop
    
    chiq = consumptionchi + intermediatechi  
    
    
end subroutine chiqAUT

!****************************************************************************
!	get profits for all choices of each firm and update its choice
!****************************************************************************

subroutine  profitsAUT(njump, nlgjump, ndiscrete, check_all)                          
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use autarky
	implicit none

	integer, intent(out) :: njump, nlgjump, ndiscrete
    logical, intent(out) :: check_all
    integer(4), dimension(sizef) :: oldposq, olddiscd
    
    real(8), dimension(sizeq) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8), dimension(sizef) :: fprofit
    
   ! --------------------------------------------------------------------
    njump = 0
    nlgjump = 0
    ndiscrete = 0
    check_all = .true.
    oldposq = posqomega
    olddiscd = discreted
    
    !$OMP PARALLEL shared(posqomega, discreted), private(profit, pos_maxpi)
    !$OMP do
    do j = 1, sizef
        ! calculate profit for firm j
         profit = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)*wq**myalpha/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq
         
         ! update its choice
         fprofit(j) = maxval(profit)
         pos_maxpi = maxloc(profit)
         posqomega(j) = pos_maxpi(1)
         
        
    end do
    !$OMP END do
    !$OMP END PARALLEL
    
    where(fprofit < 0.) 
        posqomega = -1
        discreted = 0
    end where
    
    nlgjump   = count(abs(oldposq - posqomega) > 1) 
    njump     = count(oldposq /= posqomega)
    ndiscrete = count(discreted /= olddiscd)
    
end subroutine profitsAUT


!****************************************************************************
!	get profits for all choices of each firm and update its choice
!****************************************************************************

subroutine  profitsAUTnq(njump, nlgjump, ndiscrete, check_all)                          
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use autarky
    use counterfactual
	implicit none

	integer, intent(out) :: njump, nlgjump, ndiscrete
    logical, intent(out) :: check_all
    integer(4), dimension(sizef) :: oldposq, olddiscd
    
    real(8) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
   ! --------------------------------------------------------------------
    njump = 0
    nlgjump = 0
    ndiscrete = 0
    check_all = .true.
    oldposq = posqomega
    olddiscd = discreted
    
    do j = 1, sizef
        
        if(discreted0(j)>0) then
        ! calculate profit for firm j
         profit = (1./mysigma)*( ( markup*priceqh(posqomega0(j))**(1.-myalpha)*wq(posqomega0(j))**myalpha/zqomega(j,(posqomega0(j))) )**(1.-mysigma) )*chiq(posqomega0(j)) - fxq(posqomega0(j))
         if (profit>0.d0) then
             discreted(j) = 1
             posqomega(j) = posqomega0(j)
         else
             discreted(j) = 0
             posqomega(j) = -1
         end if
         
         end if
         
    end do
    
    nlgjump   = count(abs(oldposq - posqomega) > 1) 
    njump     = count(oldposq /= posqomega)
    ndiscrete = count(discreted /= olddiscd)
    
    where(discreted0 == 0) 
        discreted = 0
        posqomega = 0
    end where
    
end subroutine profitsAUTnq