! ====================================================================
! counterfactual with elastic labor, but no change in quality
! ====================================================================

    subroutine calc_objcnq(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use counterfactual
	implicit none
    ! =============================================
    
	real(8), dimension(nc), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x, same parameters as counterEL
    call xtoguessc(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    discreted = discreted0
    posqomega = posqomega0
    
    ! calculate new price indices and demand for foreign goods /no recalculating the equilibrium
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibriumnq(succ)  ! equilibrium where the only firm choice is exit or enter with pre-counterfactual quality
    
    call firmout1
    call momentc(obj)
    
    ! shows results counterfactual
    if (toprint) then
        print *, 'obj counterEL_noq', obj, succ, '******************************'
        call show_counter
    end if
    
    end subroutine calc_objcnq
    
! ====================================================================
! equilibrium with no change in quality
! ====================================================================

subroutine equilibriumnq(succ)
	use numerical_libraries
	use parameters_qf
	use parameters_estimate
    use parameters_firm
    use parameters_quality
	implicit none

    logical, intent(out) :: succ
    
    ! checking income and labor demand does not make sense without skill variables
    integer(4), parameter :: maxiter = 200
    real(8), parameter :: tolerr = 1.e-4 
    
    ! -----------------------------------------------------
    ! estimation auxiliary parameters
    integer(4) :: j, iter
	integer(4), dimension(sizef) :: oldposq		            ! choice of quality (position in quality grid)
	integer(4), dimension(sizef) :: olddiscd				! discrete choices on exit, importing and exporting
    integer :: njump
    real(8) :: errp, errchi
    real(8) :: rd
    real(8), dimension(sizeq) :: oldchi, oldp
    logical :: check_succ
        
	! ======================================================================================
    ! reset parameters
    succ = .false.
	iter = 0
    njump = 1
    
    ! ================================================================
    outernq_loop: do
        ! stop commands	! calc_obj prints success for each evaluation in polytope
		! --------------------------------
		if ( (njump == 0) .and. (abs(errp) <= tolerr) .and. (abs(errchi) <= tolerr) ) then   
            succ = .true.
            if(toprint) print *, 'equilibriumnq found', iter, succ
            exit outernq_loop
        elseif (iter >= maxiter) then
		    if(toprint) then
                print *, 'equilibriumnq not found', iter
                print *, 'nchanges', njump
                print *, 'err', errp, errchi
                print *, ' '
            end if
            exit outernq_loop
        end if
        
        ! record initial values
        ! --------------------------------------------------------------------------------------------
        oldchi = chiq
        oldp = priceqh
        
        ! find representative firms, aggregate prices and demand. Files in auxequilibrium.f90
        ! --------------------------------------------------------------------------------------------
        where(discreted == 0) posqomega = -1
        where(posqomega ==-1) discreted = 0
        call calc_nqzqd             ! representative firms mass and productivity
        call prices                 ! price indices
        call calc_chiq              ! demand shifter
        
        ! calculate profits and update choices
        ! --------------------------------------------------------------------------------------------
        call profitsnq(njump)
        
        ! calculate errors
        errp = maxval(abs(oldp - priceqh)) + maxval(abs(oldp - priceqh)/priceqh)
        errchi = maxval(abs(oldchi - chiq)) + maxval(abs(oldchi - chiq)/chiq)
                
        iter = iter + 1
        
    end do outernq_loop
    
    call RANDOM_NUMBER(rd)
    if ( (toprint) .or. (rd< 0.01d0) ) then
        if (toprint) print *, ' -------------------------------------------'
        call check_prices
        call check_chi
        if (toprint) print *, ' -------------------------------------------'
        if (toprint) print *, ' '
    end if

end subroutine equilibriumnq

!****************************************************************************
!	get profits for all choices of each firm and update its choice
!****************************************************************************

subroutine  profitsnq(njump)                          
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use counterfactual
	implicit none

	integer, intent(out) :: njump
    integer(4), dimension(sizef) :: oldposq, olddiscd
    real(8), dimension(4) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j
    
    real(8) :: fprofit
    
   ! --------------------------------------------------------------------
    njump = 0
    oldposq = posqomega
    olddiscd = discreted
    
    do j = 1, sizef
        
        if (discreted0(j)>0) then
        
        ! calculate profit for firm j
         profit(1) = (1./mysigma)*( ( markup*priceqh(posqomega0(j))**(1.-myalpha)*wq(posqomega0(j))**myalpha/zqomega(j,posqomega0(j)) )**(1.-mysigma) )*chiq(posqomega0(j)) - fxq(posqomega0(j))
         profit(2) = (1./mysigma)*( ( markup*priceqa(posqomega0(j))**(1.-myalpha)*wq(posqomega0(j))**myalpha/zqomega(j,posqomega0(j)) )**(1.-mysigma) )*chiq(posqomega0(j)) - fxq(posqomega0(j)) - fmomega(j)
         profit(3) = (1./mysigma)*( ( markup*priceqh(posqomega0(j))**(1.-myalpha)*wq(posqomega0(j))**myalpha/zqomega(j,posqomega0(j)) )**(1.-mysigma) )*(chiq(posqomega0(j)) + chiqf(posqomega0(j))) - fxq(posqomega0(j)) - fxomega(j)
         profit(4) = (1./mysigma)*( ( markup*priceqa(posqomega0(j))**(1.-myalpha)*wq(posqomega0(j))**myalpha/zqomega(j,posqomega0(j)) )**(1.-mysigma) )*(chiq(posqomega0(j)) + chiqf(posqomega0(j))) - fxq(posqomega0(j)) - (fmomega(j) + fxomega(j))
         
         ! update its choice
         fprofit = maxval(profit)
         pos_maxpi = maxloc(profit)
         discreted(j) = pos_maxpi(1)
         posqomega(j) = posqomega0(j)
         if (fprofit<0.) then 
             posqomega(j) = -1
             discreted(j) = 0
         end if
             
        end if
        
    end do
       
    njump = count(discreted /= olddiscd)
    
end subroutine profitsnq
