!ALL FILES ON COUNTERFACTUAL WITH GROWTH. ELASTIC AND INELASTIC CASES, WITH AND WITHOUT QUALITY CHANGES
! -------------------------------------------------------
! ELASTIC LABOR: calculate objective function: Match import penetration and export expansion/GDP & no change in sales
! -------------------------------------------------------
    subroutine calc_objce(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
	implicit none

	real(8), dimension(nce), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguessce(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    call firmout1
    
    call momentce(obj)
    
    if (toprint) print *, 'obj counterEL, absorption expansion', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    end subroutine calc_objce

    ! ============================================================
    subroutine momentce(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8) :: firm_imp, cimp, allsales
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    ! change in total sales
    allsales = sum(tsales*womega)
    
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + (expe_data01 - (aggexps1 - aggexps0))**2.d0
    obj = obj + ((allsales - allsales0)/allsales0)**2.d0
    
    if(toprint) then
        print *, 'D trade/absorption, data, model, abs'
        print *, 'imp ', impp_data01, (aggimps1 - aggimps0), abs(impp_data01 - (aggimps1 - aggimps0))
        print *, 'exp ', expe_data01, (aggexps1 - aggexps0), abs(expe_data01 - (aggexps1 - aggexps0))
        print *, 'sale', ((allsales - allsales0)/allsales0)
        print *, ' '   
    end if
    
    end subroutine momentce
    
     
! =========================================================
subroutine  guesstoxce(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nce), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    x(3) = log(ycfactor)
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,nce
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine guesstoxce

! =========================================================
subroutine  xtoguessce(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nce), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    ycfactor = exp(x(3))
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, nce
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine xtoguessce

! =========================================================================
! counterfactual with no quality choice and growth
! =========================================================================

    subroutine calc_objcenq(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use counterfactual
	implicit none
    ! =============================================
    
	real(8), dimension(nce), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x, same parameters as counterEL
    call xtoguessce(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    discreted = discreted0
    posqomega = posqomega0
    
    ! calculate new price indices and demand for foreign goods /no recalculating the equilibrium
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibriumnq(succ)  ! equilibrium where the only firm choice is exit or enter with pre-counterfactual quality
    
    call firmout1
    call momentce(obj)
    
    ! shows results counterfactual
    if (toprint) then
        print *, 'obj counterELg_noq', obj, succ, '******************************'
        call show_counter
    end if

    end subroutine calc_objcenq
    
    
! INELASTIC LABOR: calculate objective function. Match import penetration and export expansion/GDP, no change in sales & no change in labor demand
! -------------------------------------------------------
    subroutine calc_objcei(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(ncei), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguesscei(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    skpremium = wh1/wl1
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    discreted = discreted0
    posqomega = posqomega0
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    call firmout1
    call firmout2       ! moments on skills to get labor market clearing condition/aggregate labor
    
    call momentcei(obj)
    
    if (toprint) print *, 'obj, inelastic, growth', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    end subroutine calc_objcei

    ! ============================================================
    subroutine momentcei(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8), dimension(sizef) :: ntsales, avgwage, employment
    real(8) :: firm_imp, cimp, allsales
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    ! change in total sales
    allsales = sum(tsales*womega)
    
    ! aggregate labor demand
    avgwage = 0.d0
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*wh1 + (1. - skilla)*wl1
        employment = (myalpha/markup)*tsales/avgwage
    end where
    Lh1 = sum(employment*skilla)
    Ll1 = sum(employment*(1.-skilla))
    
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + (expe_data01 - (aggexps1 - aggexps0))**2.d0
    obj = obj + ((allsales - allsales0)/allsales0)**2.d0
    obj = obj + ((Lh1 - Lh0)/Lh0)**2.d0
    obj = obj + ((Ll1 - Ll0)/Ll0)**2.d0
    
    
    if(toprint) then
        print *, 'D trade/absorption, data, model, abs'
        print *, 'imp ', impp_data01, (aggimps1 - aggimps0), abs(impp_data01 - (aggimps1 - aggimps0))
        print *, 'exp ', expe_data01, (aggexps1 - aggexps0), abs(expe_data01 - (aggexps1 - aggexps0))
        print *, ' '   
        print *, 'sale', abs((allsales - allsales0)/allsales0)
        print *, ' '   
        print *, 'labor demand, change, obj'
        print *, 'H', ((Lh1 - Lh0)/Lh0), ((Lh1 - Lh0)/Lh0)**2.d0
        print *, 'L', ((Ll1 - Ll0)/Ll0), ((Ll1 - Ll0)/Ll0)**2.d0
        print *, ' '   
    end if
    
    end subroutine momentcei
    
     
! =========================================================
subroutine  guesstoxcei(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(ncei), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    x(3) = log(ycfactor)
    x(4) = log(wl1)
    x(5) = log(wh1/skpremium0)
    
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,ncei
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine guesstoxcei

! =========================================================
subroutine  xtoguesscei(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(ncei), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    ycfactor = exp(x(3))
    wl1 = exp(x(4))
    wh1 = skpremium0*exp(x(5))    
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, n
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine xtoguesscei

! INELASTIC LABOR: calculate objective function. Match import penetration and export expansion/GDP, no change in sales & no change in labor demand
! -------------------------------------------------------
    subroutine calc_objceinq(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(ncei), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguesscei(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    skpremium = wh1/wl1
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    discreted = discreted0
    posqomega = posqomega0
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibriumnq(succ)
    call firmout1
    call firmout2       ! moments on skills to get labor market clearing condition/aggregate labor
    
    call momentcei(obj)
    
    if (toprint) print *, 'obj, inelastic, growth', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    end subroutine calc_objceinq