! ==================================================================
! counterfactuals with free entry
! ==================================================================
! ELASTIC LABOR: calculate objective function: Match import penetration and export expansion/GDP & no change in profits
! -------------------------------------------------------
    subroutine calc_objcfe(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
	implicit none

	real(8), dimension(nce), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguesscfe(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0
    womega = womega0*wfactor
    nfirms_data = nfirms_data1
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    call firmout1
    
    call momentcfe(obj)
    if (toprint) print *, 'obj counterEL, free entry', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    !if( (succ) .and. (obj <= 2.4e-5) ) then; call print_fvvce(x, obj); print *, 'algorithm stopped in counterFE, calc_objcfe'; stop; end if
    
    end subroutine calc_objcfe
    
    
    ! ============================================================
    subroutine momentcfe(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8) :: firm_imp, cimp, eprofit
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    call calc_eprofit(eprofit)
    
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + (expe_data01 - (aggexps1 - aggexps0))**2.d0
    obj = obj + ((eprofit - eprofit0)/eprofit0)**2.d0
    
    if(toprint) then
        print *, 'D trade/absorption, data, model, abs'
        print *, 'imp   ', impp_data01, (aggimps1 - aggimps0), abs(impp_data01 - (aggimps1 - aggimps0))
        print *, 'exp   ', expe_data01, (aggexps1 - aggexps0), abs(expe_data01 - (aggexps1 - aggexps0))
        print *, 'profit', ((eprofit - eprofit0)/eprofit0)
        print *, ' '   
    end if
    
    end subroutine momentcfe
    
    ! ===================================================================================
    ! ===================================================================================
    subroutine calc_eprofit(epi)
        use parameters_quality
        use parameters_firm
	    use parameters_qf
	    use parameters_estimate
	    implicit none

        real(8), intent(out) :: epi
        integer(4) :: j, jj
        ! ----------------------------------------------------------------
        epi = 0.d0
            do j = 1, sizef
                if (discreted(j) == 1) then
                    epi = epi + (1./mysigma)*( ( markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j,posqomega(j)) )**(1.-mysigma) )*chiq(posqomega(j)) - fxq(posqomega(j))
                elseif (discreted(j) == 2) then
                    epi = epi + (1./mysigma)*( ( markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j,posqomega(j)) )**(1.-mysigma) )*chiq(posqomega(j)) - fxq(posqomega(j)) - fmomega(j)
                elseif (discreted(j) == 3) then
                    epi = epi + (1./mysigma)*( ( markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j,posqomega(j)) )**(1.-mysigma) )*(chiq(posqomega(j)) + chiqf(posqomega(j))) - fxq(posqomega(j)) - fxomega(j)
                elseif (discreted(j) == 4) then
                    epi = epi + (1./mysigma)*( ( markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j,posqomega(j)) )**(1.-mysigma) )*(chiq(posqomega(j)) + chiqf(posqomega(j))) - fxq(posqomega(j)) - fxomega(j) - fmomega(j)
                end if
            end do

    end subroutine calc_eprofit
    
    
! =========================================================
subroutine  guesstoxcfe(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nce), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    x(3) = log(wfactor)
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,nce
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine guesstoxcfe

! =========================================================
subroutine  xtoguesscfe(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nce), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    wfactor = exp(x(3))
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, nce
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine xtoguesscfe

! -------------------------------------------------------
! ELASTIC LABOR, NO QUALITY CHOICE: calculate objective function: Match import penetration and export expansion/GDP & no change in profits
! -------------------------------------------------------
    subroutine calc_objcfenq(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
	implicit none

	real(8), dimension(nce), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguesscfe(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0
    womega = womega0*wfactor
    nfirms_data = nfirms_data1
    
    posqomega = posqomega0
    discreted = discreted0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibriumnq(succ)
    call firmout1
    
    call momentcfe(obj)
    if (toprint) print *, 'obj counterEL, free entry, no quality choice', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    end subroutine calc_objcfenq
    
    
! -------------------------------------------------------
! INELASTIC LABOR, FREE ENTRY
! -------------------------------------------------------
    subroutine calc_objcfei(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(ncei), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguesscfei(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    skpremium = wh1/wl1
    womega = womega0*wfactor
    nfirms_data = nfirms_data1
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    call firmout1
    call firmout2
    
    call momentcfei(obj)
    if (toprint) print *, 'obj counter inelastic, free entry', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    if ( (succ) .and. (obj < 0.0035) ) then
        call print_fvvcei(x, obj)
        print *, 'algorithm printed in calc_objcfei'
     !   stop
    end if
    
    end subroutine calc_objcfei
    
     
! =========================================================
subroutine  guesstoxcfei(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(ncei), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    x(3) = log(wfactor)
    x(4) = log(wl1)
    x(5) = log(wh1/skpremium0)
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,ncei
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine guesstoxcfei

! =========================================================
subroutine  xtoguesscfei(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(ncei), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    wfactor = exp(x(3))
    wl1 = exp(x(4))
    wh1 = skpremium0*exp(x(5))    
         
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, ncei
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine xtoguesscfei

 ! ============================================================
    subroutine momentcfei(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8), dimension(sizef) :: avgwage, employment
    real(8) :: firm_imp, cimp, eprofit
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    ! expected profit 
    call calc_eprofit(eprofit)
    
    ! aggregate labor demand
    avgwage = 0.d0
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*wh1 + (1. - skilla)*wl1
        employment = (myalpha/markup)*tsales/avgwage
    end where
    Lh1 = sum(employment*skilla)
    Ll1 = sum(employment*(1.-skilla))
    
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + 20.*(expe_data01 - (aggexps1 - aggexps0))**2.d0
    obj = obj + ((eprofit - eprofit0)/eprofit0)**2.d0
    obj = obj + 10.*((Lh1*womega - Lh0*womega0)/(Lh0*womega0))**2.d0
    obj = obj + 10.*((Ll1*womega - Ll0*womega0)/(Ll0*womega0))**2.d0
    
    
    if(toprint) then
        print *, 'D trade/absorption, data, model, abs'
        print *, 'imp   ', impp_data01, (aggimps1 - aggimps0), abs(impp_data01 - (aggimps1 - aggimps0))
        print *, 'exp   ', expe_data01, (aggexps1 - aggexps0), abs(expe_data01 - (aggexps1 - aggexps0))
        print *, 'profit', ((eprofit - eprofit0)/eprofit0)
        print *, 'labor demand, change, obj'
        print *, 'H', (Lh1*womega - Lh0*womega0)/(Lh0*womega0), ((Lh1*womega - Lh0*womega0)/(Lh0*womega0))**2.d0
        print *, 'L', (Ll1*womega - Ll0*womega0)/(Ll0*womega0), ((Ll1*womega - Ll0*womega0)/(Ll0*womega0))**2.d0
        print *, ' '   
    end if
    
    end subroutine momentcfei
    
    
    
! -------------------------------------------------------
! INELASTIC LABOR, FREE ENTRY, no quality choice
! -------------------------------------------------------
    subroutine calc_objcfeinq(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(ncei), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguesscfei(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    skpremium = wh1/wl1
    womega = womega0*wfactor
    nfirms_data = nfirms_data1
    discreted = discreted0
    posqomega = posqomega0
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibriumnq(succ)
    call firmout1
    call firmout2
    
    call momentcfei(obj)
    if (toprint) print *, 'obj counter inelastic, free entry, no quality choice', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    end subroutine calc_objcfeinq
    