! counterfactuals to autarky
! ==============================================
    ! -------------------------------------------------------
! INELASTIC LABOR
! -------------------------------------------------------
    subroutine calc_autinq(x, obj, succ)
    use crosssec
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(nc), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    
    real(8), dimension(sizef) :: avgwage, employment
    
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
    
    ! new wages, skill premium
    wl1 = exp(x(1))
    wh1 = wl1*exp(x(2)) 
    skpremium = wh1/wl1
    nfirms_data = nfirms_data1
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilib_autnq(succ)
    call firmAUT       ! moments on skills to get labor market clearing condition/aggregate labor
    
    ! calculate moments
    ! ------------------------------------------
    obj = 0.d0
    ! aggregate labor demand
    avgwage = 0.d0
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*wh1 + (1. - skilla)*wl1
        employment = (myalpha/markup)*tsales/avgwage
    end where
    Lh1 = sum(employment*skilla)
    Ll1 = sum(employment*(1.-skilla))
    
    ! objective function
    obj = obj + ((Lh1 - Lh0)/Lh0)**2.d0
    obj = obj + ((Ll1 - Ll0)/Ll0)**2.d0
    
    if(toprint) then
        print *, 'labor demand, change, obj'
        print *, 'H', ((Lh1 - Lh0)/Lh0), ((Lh1 - Lh0)/Lh0)**2.d0
        print *, 'L', ((Ll1 - Ll0)/Ll0), ((Ll1 - Ll0)/Ll0)**2.d0
        print *, ' '   
    end if
    
    if (toprint) print *, 'obj AUTARKY, inelastic, no quality choice', obj, succ, '******************************'
        
    end subroutine calc_autinq
    
! elastic labor, no quality choice
! -----------------------------------------------
    subroutine equilib_autnq(succ)
    use numerical_libraries
	use parameters_qf
	use parameters_estimate
    use parameters_firm
    use parameters_quality
    use autarky
	implicit none

    logical, intent(out) :: succ
    
    ! checking income and labor demand does not make sense without skill variables
    integer(4), parameter :: maxiter = 200
    real(8), parameter :: tolerr = 1.e-4 
     ! -----------------------------------------------------
    ! estimation auxiliary parameters
    integer(4) :: j, jj, iter
	integer(4), dimension(sizef) :: oldposq		            ! choice of quality (position in quality grid)
	integer(4), dimension(sizef) :: olddiscd				! discrete choices on exit, importing and exporting
    logical :: check_all
    integer :: njump, nlgjump, ndiscrete
    real(8) :: errp, errchi
    real(8) :: rd
    real(8), dimension(sizeq) :: oldchi, oldp
    logical :: check_succ
        
	! ======================================================================================
    ! reset parameters
    succ = .false.
	iter = 0
    check_all = .false.
    where(discreted == 0) posqomega = -1
    where(posqomega ==-1) discreted = 0
    if(all(discreted == 0)) then; discreted = 1; posqomega = 1; end if    
    where(discreted>1) discreted=1
    ! ================================================================
    outer_loop: do
        ! stop commands	! calc_obj prints success for each evaluation in polytope
		! --------------------------------
		!if ( (nlgjump <= sizef*.005) .and. (ndiscrete<=sizef*.005) .and. (njump < sizef/100.) .and. (abs(errp) <= tolerr) .and. (abs(errchi) <= tolerr) .and. (check_all) ) then   
        if ( (nlgjump == 0) .and. (ndiscrete == 0) .and. (njump == 0) .and. (abs(errp) <= tolerr) .and. (abs(errchi) <= tolerr) .and. (check_all) ) then   
            succ = .true.
            if(toprint) print *, 'autarky equilibrium found', iter, succ
            exit outer_loop
        elseif (iter >= maxiter) then
		    if(toprint) then
                print *, 'equilibrium not found', iter, check_all
                print *, 'nchanges', nlgjump, njump, ndiscrete
                print *, 'err', errp, errchi
                print *, ' '
            end if
            exit outer_loop
        end if
        
        ! record initial values
        ! --------------------------------------------------------------------------------------------
        oldposq = posqomega
	    olddiscd = discreted
        oldchi = chiq
        oldp = priceqh
        
        ! find representative firms, aggregate prices and demand. Files in auxequilibrium.f90
        ! --------------------------------------------------------------------------------------------
        call nqzqAUT             ! representative firms mass and productivity
        call pricesAUT            ! price indices
        call chiqAUT              ! demand shifter
        
        ! calculate profits and update choices
        ! --------------------------------------------------------------------------------------------
        call profitsAUTnq(njump, nlgjump, ndiscrete, check_all)
        
        ! calculate errors
        errp = maxval(abs(oldp - priceqh)) + maxval(abs(oldp - priceqh)/priceqh)
        errchi = maxval(abs(oldchi - chiq)) + maxval(abs(oldchi - chiq)/chiq)
                
        iter = iter + 1
    end do outer_loop
    
    nq = 0.d0
    zq = 0.d0
    nq(:,1) = nqa
    zq(:,1) = zqa
    
    end subroutine equilib_autnq
