! *******************************************************************************************
! contains all subroutines generating functions and variables that depend on parameter values but not on equilibrium choices
! *******************************************************************************************

! ----------------------------------------------------------------------------------------
! firm variables
! ----------------------------------------------------------------------------------------
    subroutine gen_xfirms
        !use parameters_quality
        use random_draw
        use parameters_qf
        use parameters_estimate
        use parameters_quality
        implicit none
        
        real(8), dimension(sizef) :: z1, z2
        integer(4) :: j, q
        real(8) :: mean1, se1, mean2, se2
    
        ! productivity
        ! ======================================================================
        z1 = exp( mu1 + sigma1*random_firms(:,1) )       ! z1 is log-normal and random_firms is standard normal
        z2 = sigma2*random_firms(:,2)                    ! z2 is normal
        do q = 1, sizeq
            zqomega(:,q) = max( 0.d0, z1*(1. + z2*qgrid(q)) )
        end do
        
        ! import and export costs
        ! ======================================================================
        fxomega = exp(fx + random_fxcost(:,1)*fxvar)
        fmomega = exp(fm + random_fxcost(:,2)*fmvar)
     
        if(toprint) then
            mean1 = sum(z1)/sizef
            se1 = sum((z1 - mean1)**2.)/sizef
            se1 = se1**(.5)
            mean2 = sum(z2)/sizef
            se2 = sum((z2 - mean2)**2.)/sizef
            se2 = se2**(.5)
            print *, 'statistics on firm productivity, mean, se'
            print *, 'z1', mean1, se1
            print *, 'z2', mean2, se2
        end if
    !stop
    
            
    end subroutine gen_xfirms
    
    
! ----------------------------------------------------------------------------------------
! quality variables
! ----------------------------------------------------------------------------------------
    subroutine gen_xquality
        use parameters_estimate
        use parameters_quality
        implicit none
        
        real(8), dimension(sizeq) :: qvaux, qvaux2
        integer(4) :: j, q
    
        ! fixed production cost
        ! ======================================================================
        fxq = max(0.d0, f1 + f2*qgrid)
        
        ! foreign demand
        ! ======================================================================
        qvaux = qgrid - nu*qcf
        call phiv0(sizeq, qvaux, chiqf)        
        chiqf = chiqf*yf
        
        ! Price index of foreign goods
        ! ======================================================================
        ! evaluation according to firm with output qgrid
        qvaux = qf - nu*qgrid
        call phiv0(sizeq, qvaux, priceqf)
        priceqf = pf*priceqf**(1./(1.-mysigma))              ! price of representative foreign firm is 1
        
        ! consumer evaluation, pcf
        call phir0(qf, pcf)
        pcf = pf*pcf**(1./(1.-mysigma))

    end subroutine gen_xquality