! ****************************************************************
! PURPOSE: Match moments and calculate objective function
! ****************************************************************

subroutine moments1(obj)
	use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
	implicit none
    
    real(8), intent(out) :: obj
    integer(4), dimension(sizef) :: qsize, qwage
    real(8), dimension(sizef) :: var1, var2
    real(8), dimension(4,4) :: wquartiles, qsalesw_model
    integer(4) :: j, jj, pospi(1)
    real(8) :: psize_model(5)
    real(8), dimension(4) :: exint_model, imint_model, exshr_model, imshr_model
    real(8) :: aux1, aux2
    real(8) :: objdata
    
    integer(4) :: mgap, isnq
    real(8), dimension(sizeq) :: tnq
    
    ! ==================================================================================
    obj = 0.d0
    
    ! quartiles of size and wage
    ! ----------------------------------------------------------------------------------
    var1 = log(nhsales)
    var2 = real(posqomega)        ! wage rank coincides with actual skill intensity (and quality)
    wquartiles = reshape(mweight(1:16), shape(qsalesw_data))
    call quartiles(var1, qsize)
    call quartiles(var2, qwage)
    call match_quartile(qwage, qsize, qsalesw_model)
    obj = obj + sum( sum(wquartiles*(qsalesw_model - qsalesw_data)**2,1) )
    
    if (toprint) then
        print *, ' '
        print *, '========================================================='
        print *, 'MOMENTS DATA, MODEL, CONTRIBUTION TO OBJ'
        print *, ' '
        print *, 'joint quartile distribution of wages and size. weight, obj'
        do j = 1, 4
            do jj = 1, 4
            print *, j, jj, qsalesw_data(j,jj), qsalesw_model(j,jj) !wquartiles(j,jj)*(qsalesw_model(j,jj) - qsalesw_data(j,jj))**2
            end do
        end do    
        print *, 'contribution to obj = ', sum( sum(wquartiles*(qsalesw_model - qsalesw_data)**2,1) )
        print *, ' ' 
    end if
    
    ! percentiles of home sales & skill intensity (unconditional distributions)
    ! ----------------------------------------------------------------------------------
    var1 = log(nhsales)       ! normalized domestic sales
    call pdistribution(var1, psize_model)
    obj = obj + sum(mweight(17:21)*( psize_model - psize_data )**2)
    
    if (toprint) then
        print *, 'distribution of normalized domestic sales'
        print *, '10% ', psize_data(1) , psize_model(1), mweight(17)*( psize_model(1) - psize_data(1) )**2
        print *, '25% ', psize_data(2) , psize_model(2), mweight(18)*( psize_model(2) - psize_data(2) )**2
        print *, '50% ', psize_data(3) , psize_model(3), mweight(19)*( psize_model(3) - psize_data(3) )**2
        print *, '75% ', psize_data(4) , psize_model(4), mweight(20)*( psize_model(4) - psize_data(4) )**2
        print *, '90% ', psize_data(5) , psize_model(5), mweight(21)*( psize_model(5) - psize_data(5) )**2
        print *, 'ln(90%/10%) ', psize_data(5) - psize_data(1), psize_model(5) - psize_model(1)
        print *, 'contribution to obj = ', sum(mweight(17:21)*( psize_model - psize_data )**2)
        print *, ' ' 
    end if
         
    ! for each size quartile... qsize
    ! -----------------------------------------------
    var1 = log(nhsales)
    call byqsize1(qsize, exint_model, imint_model, exshr_model, imshr_model)
    obj = obj + sum(mweight(31:34)*(exint_model -  exint_data)**2)
    obj = obj + sum(mweight(35:38)*(imint_model - imint_data)**2)
    obj = obj + sum(mweight(39:42)*(exshr_model - exshr_data)**2)
    obj = obj + sum(mweight(43:46)*(imshr_model - imshr_data)**2)
    
    if (toprint) then    
        
        print *, 'export intensity by quartiles of size'
        print *, '1 ', exint_data(1), exint_model(1), mweight(31)*(exint_data(1) - exint_model(1))**2
        print *, '2 ', exint_data(2), exint_model(2), mweight(32)*(exint_data(2) - exint_model(2))**2
        print *, '3 ', exint_data(3), exint_model(3), mweight(33)*(exint_data(3) - exint_model(3))**2
        print *, '4 ', exint_data(4), exint_model(4), mweight(34)*(exint_data(4) - exint_model(4))**2
        print *, 'total contribution to obj', sum(mweight(31:34)*(exint_model - exint_data)**2)
        print *, " "
    
        print *, 'import intensity by quartiles of size'
        print *, '1 ', imint_data(1), imint_model(1), mweight(35)*(imint_data(1) - imint_model(1))**2
        print *, '2 ', imint_data(2), imint_model(2), mweight(36)*(imint_data(2) - imint_model(2))**2
        print *, '3 ', imint_data(3), imint_model(3), mweight(37)*(imint_data(3) - imint_model(3))**2
        print *, '4 ', imint_data(4), imint_model(4), mweight(38)*(imint_data(4) - imint_model(4))**2
        print *, 'total contribution to obj', sum(mweight(35:38)*(imint_model - imint_data)**2)
        print *, " "
    
        print *, '% exporters by quartiles of size'
        print *, '1 ', exshr_data(1), exshr_model(1), mweight(39)*(exshr_data(1) - exshr_model(1))**2
        print *, '2 ', exshr_data(2), exshr_model(2), mweight(40)*(exshr_data(2) - exshr_model(2))**2
        print *, '3 ', exshr_data(3), exshr_model(3), mweight(41)*(exshr_data(3) - exshr_model(3))**2
        print *, '4 ', exshr_data(4), exshr_model(4), mweight(42)*(exshr_data(4) - exshr_model(4))**2
        print *, 'total contribution to obj', sum(mweight(39:42)*(exshr_model - exshr_data)**2)
        print *, " "
    
        print *, '% importers by quartiles of size'
        print *, '1 ', imshr_data(1), imshr_model(1), mweight(43)*(imshr_data(1) - imshr_model(1))**2
        print *, '2 ', imshr_data(2), imshr_model(2), mweight(44)*(imshr_data(2) - imshr_model(2))**2
        print *, '3 ', imshr_data(3), imshr_model(3), mweight(45)*(imshr_data(3) - imshr_model(3))**2
        print *, '4 ', imshr_data(4), imshr_model(4), mweight(46)*(imshr_data(4) - imshr_model(4))**2
        print *, 'total contribution to obj', sum(mweight(43:46)*(imshr_model - imshr_data)**2)
        print *, " "
    end if
    
    ! exit
    ! -----------------------------------------------
    obj = obj + mweight(51)*(exitrate - real(nexit)/real(sizef))**2.
    if (toprint) print*, 'exit', exitrate, real(nexit)/real(sizef),  mweight(51)*(exitrate - real(nexit)/real(sizef))**2.
    
    ! spread of firm quality choices
    ! -----------------------------------------------
    objdata = obj
    pospi = maxloc(sum(nq,2))
    jj = count( (posqomega == 1) .or. (posqomega == pospi(1)) .or. (posqomega == sizeq) ) + .5*count( ((discreted>0) .and. (posqomega <= 5)) .or. (posqomega >= sizeq - 5) )  + .5*count( (posqomega >= pospi(1)-2) .and. (posqomega <= pospi(1)+2) ) 
    
    !jj = count( (posqomega == 1) .or. (posqomega == pospi(1)) .or. (posqomega == sizeq) ) + .5*count( ((discreted>0) .and. (posqomega <= 5)) .or. (posqomega >= sizeq - 5) )  + .5*count( (posqomega >= pospi(1)-5) .and. (posqomega <= pospi(1)+5) ) 
    !jj = count( (posqomega == 1) .or. (posqomega == sizeq) )
    obj = obj + mweight(52)*( real(jj)/real(sizef-nexit) )**2
    
    ! distribution is smooth, without gaps
    mgap = 1
    isnq = 0
    aux1 = 0.d0
    tnq = sum(nq,2)
    gaploop: do j = 1, sizeq-10
        if ( (all(tnq(j:j+5)<= 0.)) .and. (aux1 > 0.9d0 ) ) then
            mgap = j
            exit gaploop
        end if
        aux1 = aux1 + tnq(j)/(womega*(sizef - nexit)) 
    end do gaploop
    if(mgap>1) then
        aux1 = sum( tnq(mgap:sizeq)*( qgrid(mgap:sizeq) - qgrid(mgap) )**2. ) ! mass of firms (in percentage) * squared gap
    else
        aux1 = 0.d0
    end if
    obj = obj + 10.*aux1*mweight(52)
    
    
    ! distribution is smooth, without gaps, gaps from below
    mgap = 1
    isnq = 0
    aux2 = 0.d0
    tnq = sum(nq,2)
    gaploop2: do j = sizeq-10, 1, -1
        if ( (all(tnq(j:j+10)<= 0.)) .and. (aux2 > 0.9d0 ) ) then
            mgap = j
            exit gaploop2
        end if
        aux2 = aux2 + tnq(j)/(womega*(sizef - nexit))
    end do gaploop2
    aux2 = sum( tnq(1:mgap)*(qgrid(mgap) - qgrid(1:mgap)) )**2. ! mass of firms (in percentage) * squared gap
    obj = obj + aux2*mweight(52)
    
    if (toprint) then
        print *, 'q choice not spread ', count( (posqomega == 1) .or. (posqomega == sizeq) )/real(sizef-nexit), real(j)/real(sizef-nexit), mweight(52)*( real(jj)/real(sizef-nexit) )**2
        print *, 'q choice gap        ', 10.*aux1*mweight(52), aux2*mweight(52)
        print *, 'mweight(52)', mweight(52)
        print *, ' '
        print *, ' ------------------------------------------------------------'
        print *, ' objective in data = ', objdata
        print *, ' objective = ', obj
        print *, ' ------------------------------------------------------------'
    end if
    
end subroutine moments1
