! read initial guesses, random variables, and standard errors from data

! ==========================================================================
! initial guesses
! ==========================================================================
subroutine  read_initial(x, myflag)
	use parameters_firm
    use parameters_quality
    implicit none

	logical, intent(in) :: myflag
	real(8), dimension(n), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq  
    open(1, FILE='may17I.txt', STATUS='OLD')  
    !open(1, FILE='may17_alpha05.txt', STATUS='OLD')   ! autarky inelastic labor, no quality choice
    
    ! other skill specifications in appendix
    !open(1, FILE='may17_sklinear.txt', STATUS='OLD')  
    !open(1, FILE='may17_skunbounded.txt', STATUS='OLD')  
    
    ! robustness
    !open(1, FILE='may17I_sigL11.txt', STATUS='OLD')      
    !open(1, FILE='may17I_sigL18.txt', STATUS='OLD')      
    
    ! for standard errors only, counterfactuals are in mar17_alpha.sln
    !open(1, FILE='mar17_sigma3fxd.txt', STATUS='OLD')      
    !open(1, FILE='mar17_sigma7fxd.txt', STATUS='OLD')      
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, n
        if (myflag) then
            read(1,*), x(j) 
        else
            read(1,*), aux 
        end if
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
    end do
    do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_initial 

! diagonal of weighting matrix 
! =====================================================================    
subroutine  read_weights
    use data_moments
	implicit none
	integer(4) :: j
        
    ! ------------------------------------------
    mweight = 0.001d0      ! some moments, like US data and exit do not have data
    mweight(51) = 0.002d0
    mweight(52) = 0.01d0
    
    ! standard errors from bootstrap
    open(1, FILE='mse_2016.dat', STATUS='OLD')
    do j = 1, nboot
        read(1,*), mweight(j)       ! standard error of moments
    end do
    if (nweight>=53) read(1,*), mweight(53)            ! aggregate measured skill intensity
    close(1)
    
    ! moments from regression results on prices
    mweight(j:j+1) = skill_sepdata
    mweight = 1./(mweight)**2.
    mweight(j:j+1) =  mweight(j:j+1)
    
    mweight = 1.d0; mweight(17:21) = 1./100.**2; mweight(52) = 0.02d0
end subroutine read_weights

subroutine read_random
    use numerical_libraries
    use random_draw
    implicit none    
    
    integer(4) :: j, j1
    real(8) :: rd
    ! -----------------------------------------------
    ! to get truncated distributions, add after read rd: rd = rd*.95; rd = rd*.90 + .05
    
    !real(8), dimension(sizef,2) :: random_firms
	!real(8), dimension(sizef,2) :: random_fxcost
    !real(8), dimension(sizef) :: random_skill
    
    open(1, FILE='runiform_long.dat', STATUS='OLD')
    
    ! firm productivity
    do j1 = 1, 2
        do j = 1, sizef
            read(1,*), rd
		    random_firms(j,j1) = DNORIN(rd)       ! transforms uniform into standard normal
        end do
	end do
        
    ! importing and exporting fixed costs
    do j1 = 1, 2
        do j = 1, sizef
            read(1,*), rd
		    random_fxcost(j,j1) = DNORIN(rd)	 ! transforms uniform into standard normal
        end do
	end do
    
    ! measurement error in skills
    do j = 1, sizef
        read(1,*), rd
		random_skill(j) = rd					  ! truncated logistic
	end do
	close(1)
     
    ! create file with random draws
    ! -----------------------------
    !j = 1000000
    !open(1, FILE='runiform_long.dat', STATUS='REPLACE')
    !do j1 = 1, j
    !    call RANDOM_NUMBER(rd)
    !    write(1,*), rd
    !end do
    !write(1,*), 'contains', j, 'draws from uniform'
    !close(1)
    !stop
    
    
    !print *, '---------------------------------------------------------- '
    !print *, 'summary of random variables: mean and variance'
    !print *, 'z		', sum(random_firms)/(2.*sizef), sum(random_firms**2.)/(2.*sizef)
    !print *, 'fm, fx', sum(random_fxcost)/(2.*sizef), sum(random_fxcost**2.)/(2.*sizef)
    !print *, 'skill ', sum(random_skill)/sizef, sum( (random_skill - 0.5d0)**2.)/sizef
    !print *, ' ' 
    
end subroutine read_random

! =================================================================================================
! =================================================================================================
subroutine  read_counter(x)
	use parameters_firm
    use parameters_quality
    implicit none
 
	real(8), dimension(nc), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	
    ! ELASTIC LABOR, may 2017 100000 observations
    open(1, FILE='may17EL.txt', STATUS='OLD')            ! main benchmark 
    !open(1, FILE='may17ELnq.txt', STATUS='OLD')            ! main benchmark 
    !open(1, FILE='may17ELee.txt', STATUS='OLD')         ! export expansion
    !open(1, FILE='may17ELeenq.txt', STATUS='OLD')        ! export expansion, no quality choice
    !open(1, FILE='may17INEL_autnq.txt', STATUS='OLD')            ! main benchmark 
    !open(1, FILE='may17EL_alpha05.txt', STATUS='OLD')   ! autarky inelastic labor, no quality choice
    !open(1, FILE='may17ELnq_alpha05.txt', STATUS='OLD')   ! autarky inelastic labor, no quality choice
 
    ! optimal weights
    !open(1, FILE='counterELW.txt', STATUS='OLD')           ! main benchmark
    !open(1, FILE='counterELeeW.txt', STATUS='OLD')         ! export expansion
    !open(1, FILE='counterELnqW.txt', STATUS='OLD')        ! no quality choice
    
     
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nc
        read(1,*), x(j)  
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef 
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_counter 

! ===============================================================

subroutine  read_counteri(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(nci), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	! new results, sept 2016
    open(1, FILE='may17INELB.txt', STATUS='OLD')      ! main benchmark
    !open(1, FILE='may17INELnq.txt', STATUS='OLD')       ! main, no change in quality
    !open(1, FILE='may17INELeenq.txt', STATUS='OLD')      ! export expansion, no change in quality
    !open(1, FILE='may17INELee.txt', STATUS='OLD')       ! main, no change in quality  
    !open(1, FILE='may17INEL_alpha05B.txt', STATUS='OLD')   ! autarky inelastic labor, no quality choice
    !open(1, FILE='may17INELnq_alpha05.txt', STATUS='OLD')   ! autarky inelastic labor, no quality choice
    
    ! robustness
    !open(1, FILE='may17INEL_sigmaL11.txt', STATUS='OLD')      ! sigma_L = 1.1
    !open(1, FILE='may17INEL_sigmaL18.txt', STATUS='OLD')      ! sigma_L = 1.8
    !open(1, FILE='may17INEL_fxcost.txt', STATUS='OLD')      ! fixed cost changes with wages
        
    
    ! new results, sept 2016
    !open(1, FILE='counterINEL.txt', STATUS='OLD')      ! main benchmark
    !open(1, FILE='counterINELnq.txt', STATUS='OLD')       ! main, no change in quality
    !open(1, FILE='counterINELee.txt', STATUS='OLD')      ! export expansion
    !open(1, FILE='counterINELeenq.txt', STATUS='OLD')      ! export expansion, no change in quality
    
    ! optimal weights
    !open(1, FILE='counterINELW.txt', STATUS='OLD')      ! main benchmark
    !open(1, FILE='counterINELnqW.txt', STATUS='OLD')       ! main, no change in quality
    !open(1, FILE='counterINELeeW.txt', STATUS='OLD')      ! export expansion
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nci
        read(1,*), x(j) 
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_counteri

! ====================================================================================
! ====================================================================================
subroutine  read_countere(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(nce), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	
    ! ELASTIC LABOR, free entry - may17
    open(1, FILE='may17ELfe.txt', STATUS='OLD')           ! main benchmark
    !open(1, FILE='may17ELfenq.txt', STATUS='OLD')           ! no quality change
    
    ! NEW WEIGHTS, free entry
    !open(1, FILE='counterELfeW.txt', STATUS='OLD')           ! main benchmark
        
    ! NOT UPDATED: ELASTIC LABOR, growth of absorption to maintain sales unchanged
    !open(1, FILE='counterELg.txt', STATUS='OLD')           ! main benchmark
    !open(1, FILE='counterELgnq.txt', STATUS='OLD')           ! no quality change
    
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nce
        read(1,*), x(j)  
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_countere


! ====================================================================================
! ====================================================================================
subroutine  read_counterei(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(ncei), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
    
    ! FREE ENTRY - Mar 2017
	open(1, FILE='may17INELfe.txt', STATUS='OLD')           ! main benchmark
    !open(1, FILE='may17INELfenq.txt', STATUS='OLD')        ! no quality choice
    
    ! OPTIMAL WEIGHTS
   ! open(1, FILE='counterINELfeW.txt', STATUS='OLD')           ! main benchmark
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, ncei
        read(1,*), x(j)  
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_counterei
