! ======================================================================
! ======================================================================
! sanneal.F90      minimizes a function using simulated annealing
! ======================================================================
! ======================================================================

subroutine sanneal1p23(xinitial, xmin, fmin, smin)
	use parameters_sanneal
	use numerical_libraries 
	use parameters_model
	use parameters_quality
    implicit none
    
    real(8), dimension(n1), intent(in) :: xinitial
    real(8), dimension(n1), intent(out) :: xmin                  ! result, final minimizer
    real(8), intent(out) :: fmin                                ! function evaluated at minimizer
    logical, intent(out) :: smin                                ! success of function evaluated at minimizer

    ! simulated annealing variables
    real(8), dimension(n1) :: xcurrent, xnew
    real(8) :: fcurrent, fnew
    logical :: scurrent, snew
    real(8) :: temp                                             ! current temperature
    integer(4) :: itry, itotal, isuccess, iconsec, ifail, j     ! counters
    real(8) :: rd
    
    real(8), dimension(n) :: xout, xmin123
    
    ! equilibrium variables to be saved---specific to the Colombian project
    integer(4), dimension(sizef) :: posqmin, posqcurrent, posqnew
	integer(4), dimension(sizef) :: discmin, disccurrent, discnew			        	
    real(8), dimension(sizeq) :: pqmin, pqcurrent, pqnew
    real(8), dimension(sizeq) :: chiqmin, chiqcurrent, chiqnew

    ! body of sanneal
    ! ===================================================================
    xmin = xinitial
    call calc_obj1p23(xmin, xout, fmin, smin) 
    xmin123 = xout
    call saveequil(posqmin, discmin, pqmin, chiqmin)
    if (.not. smin) then
        print *, 'equilibrium not found for initial guess of parameters'
    end if
    
    ! reset counters, temperature and initial values
    ! --------------------------------------
    temp = 300.d0
    itry = 1
    itotal = 0
    isuccess = 0
    iconsec = 0
    ifail = 0
    xcurrent = xmin
    fcurrent = fmin
    call saveequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
    
    ! evaluate picked values for delta, governing new guesses' deviations from xcurrent
    ! --------------------------------------
    if (pflag) call getdelta1(xcurrent)
    
    ! --------------------------------------
    sanneal_loop1: do
        ! stop commands
        if (ifail > maxfail) then
            print *, "no equilibrium found in a neighborhood of current"
            stop
        end if

        if ( (itry >= maxtry) .or. (isuccess >= maxsucc) ) then             ! change temperature or end program
            if ( (temp < stoptemp) .or. (iconsec >= maxconsec) ) then       ! stop program if temperature reaches a minimum or attempts at improving objective function failed
                itotal = itotal + itry
                print *, 'optimization algorithm finished in # tries', itotal
                call resetequil(posqmin, discmin, pqmin, chiqmin)
                call print_sanneal123(xmin123, fmin)
                exit sanneal_loop1
            else
                call tcool(temp, temp)                                      ! cool temperature
                itotal = itotal + itry
                itry = 1
                isuccess = 0
                print *, 'temperature cool123', itotal, temp, fmin
                call resetequil(posqmin, discmin, pqmin, chiqmin)
                call print_sanneal123(xmin123, fmin)
            end if
         end if

         ! draw new random
         call neighborx1(xcurrent, xnew)
         call resetequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
         
         call calc_obj1p23(xnew, xout, fnew, snew)
         
         if (.not. snew) then       ! cycle if loop fails
            ifail = ifail + 1
            cycle sanneal_loop1
         else
            ifail = 0
            itry = itry + 1
         end if

         ! new minimum found: move current and save minimum
         if (fnew < fmin) then
            print *, 'd', temp, fmin, fnew
            xmin = xnew
            fmin = fnew
            smin = snew
            xmin123 = xout
            call saveequil(posqmin, discmin, pqmin, chiqmin)
            xcurrent = xnew
            fcurrent = fnew
            scurrent = snew
            call saveequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
            iconsec = 0
            call print_sanneal123(xmin123, fmin)
            cycle
          end if

          ! change parameters randomly
          call random_number(rd)
          if ( rd < exp( (fcurrent - fnew)/temp) ) then
            !print *, isuccess, fcurrent, fnew, exp((fcurrent - fnew)/temp)
            xcurrent = xnew
            fcurrent = fnew
            scurrent = snew
            call saveequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
            isuccess = isuccess + 1
          else
            iconsec = iconsec + 1
          end if

          xmin1 = xmin
    end do sanneal_loop1


    ! save results
    call resetequil(posqmin, discmin, pqmin, chiqmin)
    call calc_obj1p23(xmin, xmin123, fmin, smin) 
    call print_sanneal123(xmin123, fmin)
    
end subroutine sanneal1p23



! ======================================================================
! AUXILIARY SUBROUTINES
! ======================================================================

! ======================================================================
subroutine getdelta1p23(xcenter)
    use parameters_sanneal
    implicit none 

    real(8), dimension(n1), intent(in) :: xcenter
    real(8), dimension(n1) :: xneighbor
    real(8), dimension(n) :: xout
    real(8) :: rd, obju, objl
    logical :: succ
    integer(4) :: j, jj
    ! -------------------------------------------------------

    print *, 'j, objl, obju'
    do j = 1,n1
        rd = 1.
        xneighbor = xcenter
        xneighbor(j) = xcenter(j) + (rd - 0.5)*mydelta1(j)*2.
        print *, ' --------------------------- '
        call calc_obj1p23(xneighbor, xout, obju, succ)
        rd = 0.
        xneighbor = xcenter
        xneighbor(j) = xcenter(j) + (rd - 0.5)*mydelta1(j)*2.
        call calc_obj1p23(xneighbor, xout, objl, succ)
        print *, j, objl, obju
    end do
    
    print *, ' --------------------------- '
    print *, ' program stopped at getdelta1 routine in sanneal1.f90'
    stop
end subroutine getdelta1p23
