    subroutine stderror1s(xmin)
    use parameters_model
    use data_moments
    use numerical_libraries    
    use parameters_estimate
    implicit none
    
    real(8), dimension(n), intent(in) :: xmin
    real(8), dimension(n) :: mydelta
    integer(4), parameter :: mm = nweight                           ! number of moments
    real(8), dimension(mm) :: vmoment1, vmoment0                    ! vector with moments
    real(8), dimension(mm,mm) :: wmat                               ! weighting matrix
    real(8), dimension(n, mm) :: dmat, amat                         ! matrix of derivatives and auxiliary
    real(8), dimension(n,n) :: vmat, amatn                          ! variance-covariance matrix of parameters
    
    real(8) :: obj, obj1, obj2, obj3, objm, x1(n1), x2(n2), x3(n3), tpar, tx(n)
    logical :: succ
    integer(4):: j
    
    ! ===================================================
    print *, ' ------------------------------------------ '
    print *, ' standard errors'
    
    ! set weighting matrix
    wmat = 0.d0
    do j = 1, nboot-2 ! exclude aggregates
        wmat(j,j) = mweight(j)
    end do
    j = j+2
    ! price moments
    wmat(j,j) = 1./(skill_sepdata(1))**2.
    wmat(j+1,j+1) = 1./(skill_sepdata(2))**2.
    
    ! set deviations, roughly 2% of parameter values
    mydelta = (/ 0.001, 0.02, 0.02, 0.05, 0.05, 0.02, 0.10, 0.02, 0.02, 0.10, 0.10, 0.10, 0.02, 0.02, 0.02, 0.02, 0.02, 0.02 /)
    
    ! calculate objective at minimum
    call xallto123(xmin, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment0, obj)
    objm = obj1 + obj2 + obj3
    print *, 'check obj (gap not included)', objm, (obj - objm)/objm
    print *, ' '
    
    ! deviations for each parameter -- derivatives cannot be taken on x, but on the parameter itself
    print *, 'parameter % change in..., parameter, obj'
    ! mu1 ----------------------------------------------------------
    j = 1
    tx = xmin
    tpar = mu1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(mu1 - tpar)
    print *, 'mu1    ', (mu1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! sigma1 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = sigma1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(sigma1 - tpar)
    print *, 'sigma1 ', (sigma1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! sigma2 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = sigma2 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(sigma2 - tpar)
    print *, 'sigma2 ', (sigma2 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! z3 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = z3 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(z3 - tpar)
    print *, 'z3     ', (z3 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! f1 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = f1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(f1 - tpar)
    print *, 'f1     ', (f1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! f2 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = f2 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(f2 - tpar)
    print *, 'f2     ', (f2 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fm ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fm 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fm - tpar)
    print *, 'fm     ', (fm - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fmvar ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fmvar 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fmvar - tpar)
    print *, 'fmvar  ', (fmvar - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fx ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fx 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fx - tpar)
    print *, 'fx     ', (fx - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fxvar ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fxvar 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fxvar - tpar)
    print *, 'fxvar  ', (fxvar - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! l1 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = l1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(l1 - tpar)
    print *, 'l1     ', (l1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! l2 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = l2 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(l2 - tpar)
    print *, 'l2     ', (l2 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! skpremium ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = skpremium 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(skpremium - tpar)
    print *, 'skpremium', (skpremium - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! qf ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = qf 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(qf - tpar)
    print *, 'qf     ', (qf - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! yf ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = yf 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(yf - tpar)
    print *, 'yf     ', (yf - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! qcf ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = qcf 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(qcf - tpar)
    print *, 'qcf    ', (qcf - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! varpi ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = varpi 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(varpi - tpar)
    print *, 'varpi ', (varpi - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! nu ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = nu 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(nu - tpar)
    print *, 'nu   ', (nu - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    print *, ' '
    
    
    ! get matrix of variance: (dmat * wmat * dmat')^-1
    ! ------------------------------------------------------------
    call DMRRRR(n, mm, dmat, n, mm, mm, wmat, mm, n, mm, amat, n)    ! amat = dmat * wmat 
    call DMXYTF(n, mm, amat, n,  n, mm, dmat, n , n, n , amatn, n)   ! amat2 = amat * dmat'
    call DLINRG(n, amatn, n, vmat, n)       
    
    open(2, FILE='stderror1s.csv', STATUS='REPLACE')
    call xtoguessall(xmin) 
    j = 1
    write(2,*), 'mu1,      ', mu1,    ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'sigma1,   ', sigma1, ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'sigma2,   ', sigma2, ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'z3,       ', z3,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'f1,       ', f1,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'f2,       ', f2,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fm,       ', fm,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fmvar,    ', fmvar,  ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fx,       ', fx,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fxvar,    ', fxvar,  ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'l1,       ', l1,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'l2,       ', l2,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'skpremium,', skpremium, ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'qf,       ', qf,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'yf,       ', yf,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'qcf,      ', qcf,    ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'varpi,    ', varpi,  ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'nu,       ', nu,     ', ', vmat(j,j)**.5; j = j + 1
    close(2)
    
    print *, 'parameter, value, standard errors'
    j = 1
    print *, 'mu1      ', mu1,    vmat(j,j)**.5; j = j + 1
    print *, 'sigma1   ', sigma1, vmat(j,j)**.5; j = j + 1
    print *, 'sigma2   ', sigma2, vmat(j,j)**.5; j = j + 1
    print *, 'z3       ', z3,     vmat(j,j)**.5; j = j + 1
    print *, 'f1       ', f1,     vmat(j,j)**.5; j = j + 1
    print *, 'f2       ', f2,     vmat(j,j)**.5; j = j + 1
    print *, 'fm       ', fm,     vmat(j,j)**.5; j = j + 1
    print *, 'fmvar    ', fmvar,  vmat(j,j)**.5; j = j + 1
    print *, 'fx       ', fx,     vmat(j,j)**.5; j = j + 1
    print *, 'fxvar    ', fxvar,  vmat(j,j)**.5; j = j + 1
    print *, 'l1       ', l1,     vmat(j,j)**.5; j = j + 1
    print *, 'l2       ', l2,     vmat(j,j)**.5; j = j + 1
    print *, 'skpremium', skpremium, vmat(j,j)**.5; j = j + 1
    print *, 'qf       ', qf,     vmat(j,j)**.5; j = j + 1
    print *, 'yf       ', yf,     vmat(j,j)**.5; j = j + 1
    print *, 'qcf      ', qcf,    vmat(j,j)**.5; j = j + 1
    print *, 'varpi    ', varpi,  vmat(j,j)**.5; j = j + 1
    print *, 'nu       ', nu,     vmat(j,j)**.5; j = j + 1
    print *, ' '
    print *, ' ------------------------------------------ '
    
    end subroutine stderror1s
    