! -------------------------------------------------------
! calculate objective function: Match import penetration and export expansion/GDP
! -------------------------------------------------------
    subroutine calc_objc(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
	implicit none

	real(8), dimension(nc), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguessc(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    discreted = discreted0
    posqomega = posqomega0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    
    call firmout1
    
    call momentc(obj)
    
    if (toprint) print *, 'obj counterEL', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) then
        wl1 = 1.d0
        wh1 = skpremium
        call show_counter
    end if
    !if ((succ) .and. (obj<1.e-6)) then
    !    call print_fvvc(x, obj)
    !    print *, 'algorthm stopped at calc_objc, counterEL'
    !    stop
    !end if
    !
    end subroutine calc_objc

    ! ============================================================
    subroutine momentc(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8) :: firm_imp, cimp
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + (expe_data01 - (aggexps1 - aggexps0))**2.d0
    
    if(toprint) then
        print *, 'D trade/absorption, data, model, abs'
        print *, 'imp ', impp_data01, (aggimps1 - aggimps0), abs(impp_data01 - (aggimps1 - aggimps0))
        print *, 'exp ', expe_data01, (aggexps1 - aggexps0), abs(expe_data01 - (aggexps1 - aggexps0))
        print *, ' '   
    end if
    
    end subroutine momentc
    
    
! -------------------------------------------------------
! save initial conditions
! -------------------------------------------------------
subroutine save_eq0
    use crosssec
    use counterfactual
    use parameters_estimate
    use parameters_qf
    implicit none
    
    real(8), dimension(sizef) :: employment, avgwage
    real(8) :: firm_imp, cimp, avgkm0
    ! -------------------------------------------------------
    posqomega0 = posqomega
    discreted0 = discreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
    imp_share0 = imp_share
    exp_share0 = exp_share
    nhsales0 = nhsales
    ntsales0 = nhsales/(1.-exp_share)
    skilla0 = skilla
    skillm0 = skillm
    outprice0 = outprice
    tsales0 = tsales
    fxq0 = fxq
    fmomega0 = fmomega
    fxomega0 = fxomega
    
    ! aggregates
    skpremium0 = skpremium
    yf0 = yf
    absorption0 = absorption
    aggskpm0 = aggskpm
    aggskm0 = aggskm
    chiq0 = chiq
    chiqf0 = chiqf
    priceqh0 = priceqh
    priceqa0 = priceqa
    nq0 = nq
    zbarq0 = zbarq
    wl0 = 1.d0
    wh0 = skpremium
    allsales0 = sum(tsales*womega)
    womega0 = womega
    ycfactor = 1.d0
                
    ! labor outcomes
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment = (myalpha/markup)*tsales/avgwage
    end where
    aggska0 = sum(employment*skilla)/sum(employment)
    Lh0 = sum(employment*skilla)
    Ll0 = sum(employment*(1.-skilla))
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps0 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps0 = sum(exp_share*tsales)*womega/absorption
    nexit0 = nexit
    
    ! k-measure
    where (discreted == 0)
        kmeasure0 = 0.d0
    elsewhere
        kmeasure0 = log(tsales0*(1.-exp_share)) - (1.-mysigma)*log(outprice)
    end where
    avgkm0 = sum(kmeasure0)/(sizef - nexit)
    kmeasure0 = kmeasure0 - avgkm0     ! demean
    where (discreted == 0) kmeasure0 = 0.d0
    sekm0 = sum(kmeasure0**2.)/(sizef - nexit)
    avgkm0 = sum(kmeasure0)/(sizef - nexit)
    
end subroutine save_eq0

   
! =========================================================
subroutine  guesstoxc(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nc), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    !x(3) = log(ycfactor)
    
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,nc
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine guesstoxc

! =========================================================
subroutine  xtoguessc(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nc), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    ycfactor = 1.d0
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, n
            print*, j, x(j)
        end do
        stop
    end if
    
end subroutine xtoguessc



! ==================================================================================
! ==================================================================================
subroutine show_counter
	use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use counterfactual
	implicit none
    
    real(8), dimension(sizef) :: var1, employment, avgwage
    integer(4) :: j, jj
    real(8), dimension(5) :: psize_model0, psize_model1, psize_model01
    real(8), dimension(5) :: pskill_model0, pskill_model1, pskill_model01
    real(8), dimension(5) :: pskilla_model0, pskilla_model1, pskilla_model01
    real(8), dimension(5) :: pkmeas_model0, pkmeas_model1, pdkmeas
    
    real(8) :: avgsales0, avgsales1, Lh, Ll
    
    real(8), dimension(sizef) :: hyprice, hysales, kmeasure1
    real(8), dimension(sizef) :: test
    
    ! ==================================================================================
    ! get all firm characteristics
    call firmout1
    call firmout2
    call firmout3    
    
    ! aggregate moments
    ! ==================================================================================
    avgwage = 0.d0
    avgsales1 = 0.d0
    where (discreted == 0)
        employment = 0.d0
        kmeasure1 = 0.d0
    elsewhere
        avgwage = skilla*wh1 + (1. - skilla)*wl1
        employment = (myalpha/markup)*tsales/avgwage
        kmeasure1 = log(tsales*(1-exp_share)) - (1.-mysigma)*log(outprice)
    end where
    aggska = sum(employment*skilla)/sum(employment)
    kmeasure1 = kmeasure1 - sum(kmeasure1)/(sizef - nexit)
    where (discreted == 0) kmeasure1 = 0.d0
     
    Lh = sum(employment*skilla)
    Ll = sum(employment*(1.-skilla))
    
    ! average sales
    avgsales0 = sum(tsales0)/absorption0 /(sizef-nexit0)
    avgsales1 = sum(tsales)/absorption /(sizef-nexit)
    
    print *, ' --------------------------------------------------- '
    print *, ' aggregate trade'
    print *, ' '
    print *, 'imports/absorption, initial, final, final - initial'
    print *, 'data  ', aggimps_data0, aggimps_data1, aggimps_data1 - aggimps_data0 
    print *, 'model ', aggimps0, aggimps1, aggimps1 - aggimps0 
    print *, ' '
    print *, 'exports/absorption, initial, final, final - initial'
    print *, 'data  ', aggexps_data0, aggexps_data1, aggexps_data1 - aggexps_data0 
    print *, 'model ', aggexps0, aggexps1, aggexps1 - aggexps0 
    print *, ' '
    print *, 'absorption', absorption0, absorption, (absorption-absorption0)/absorption0
    print *, ' --------------------------------------------------- '
    print *, ' aggregate skills'
    print *, ' '
    print *, ' D skill premium = (1-0)/0, actual, measured'
    print *, 'model', (skpremium - skpremium0)/skpremium0, (aggskpm - aggskpm0)/aggskpm0
    print *, ' '
    print *, 'skill intensity, initial, final, (final - initial)'
    print *, 'data  ', aggskm_data0, aggskm_data1, aggskm_data1 - aggskm_data0
    print *, 'modelm', aggskm0, aggskm, aggskm - aggskm0
    print *, 'modela', aggska0, aggska, aggska - aggska0
    print *, ' '
    
    print *, ' total labor, not relative 0, 1, (1-0)/0'
    print *, 'H', Lh0, Lh, (Lh-Lh0)/Lh0
    print *, 'L', Ll0, Ll, (Ll-Ll0)/Ll0
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' Average sales per firm, (1-0)/0, log1-log0'
    print *, 'data', -0.176d0, -0.194d0
    print *, 'model', avgsales1/avgsales0 - 1., log(avgsales1/avgsales0)
    print *, 'model not normalized (1-0)/0', (sum(tsales/(sizef-nexit))-sum(tsales0/(sizef-nexit0)))/sum(tsales0/(sizef-nexit0))
    print *, 'model adj for number of firms  (1-0)/0', (avgsales1/nfirms_data1)/(avgsales0/nfirms_data0) - 1.d0
    print *, 'model (total sales, not average), log', log(sum(tsales)/absorption/(sum(tsales0)/absorption0))
    print *, ' '
    print *, 'exit, 0, 1, 1-0', real(nexit0)/sizef, real(nexit)/sizef, real(nexit - nexit0)/(sizef-nexit0)
    print *, ' '
    
    ! percentiles of home sales & skill intensity (unconditional distributions)
    ! ==================================================================================
    
    ! distributions for pre-liberalization model
    ! ---------------------------------------------
    discreted1 = discreted
    nexit1 = nexit
    discreted = discreted0
    nexit = nexit0
    var1 = log(tsales0/absorption0)       ! normalized sales
    call pdistribution(var1, psize_model0)
    call pdistribution(skillm0, pskill_model0)
    call pdistribution(skilla0, pskilla_model0)
    call pdistribution(kmeasure0, pkmeas_model0)
    discreted = discreted1
    nexit = nexit1
    
    ! for counterfactuals
    var1 = log(tsales/absorption)       ! normalized sales
    call pdistribution(var1, psize_model1)
    call pdistribution(skillm, pskill_model1)
    call pdistribution(skilla, pskilla_model1)
    call pdistribution(kmeasure1, pkmeas_model1)
    
    print *, ' --------------------------------------------------- '
    print *, 'size distribution, d data, dmodel'
    print *, '10% ', psize_data01(1), psize_model1(1) - psize_model0(1)
    print *, '25% ', psize_data01(2), psize_model1(2) - psize_model0(2)
    print *, '50% ', psize_data01(3), psize_model1(3) - psize_model0(3)
    print *, '75% ', psize_data01(4), psize_model1(4) - psize_model0(4)
    print *, '90% ', psize_data01(5), psize_model1(5) - psize_model0(5)
    print *, ' ' 
    
    
    print *, ' --------------------------------------------------- '
    print *, 'skill distribution, d data, dmodel measured, actual'
    print *, '10% ', pskill_data01(1), pskill_model1(1) - pskill_model0(1), pskilla_model1(1) - pskilla_model0(1) 
    print *, '25% ', pskill_data01(2), pskill_model1(2) - pskill_model0(2), pskilla_model1(2) - pskilla_model0(2) 
    print *, '50% ', pskill_data01(3), pskill_model1(3) - pskill_model0(3), pskilla_model1(3) - pskilla_model0(3) 
    print *, '75% ', pskill_data01(4), pskill_model1(4) - pskill_model0(4), pskilla_model1(4) - pskilla_model0(4) 
    print *, '90% ', pskill_data01(5), pskill_model1(5) - pskill_model0(5), pskilla_model1(5) - pskilla_model0(5) 
    print *, ' ' 
     
    print *, ' --------------------------------------------------- '
    print *, 'kmeasure, d data, dmodel '
    print *, '10% ', pkmeas_data01(1), pkmeas_model1(1) - pkmeas_model0(1)
    print *, '25% ', pkmeas_data01(2), pkmeas_model1(2) - pkmeas_model0(2)
    print *, '50% ', pkmeas_data01(3), pkmeas_model1(3) - pkmeas_model0(3)
    print *, '75% ', pkmeas_data01(4), pkmeas_model1(4) - pkmeas_model0(4)
    print *, '90% ', pkmeas_data01(5), pkmeas_model1(5) - pkmeas_model0(5)
    print *, ' ' 
    
    ! k-measure
    ! ============================================================
    ! build hypothetical sales
    do j = 1, sizef
     if ((discreted0(j) == 0) .or. (discreted1(j) == 0)) then
        hyprice(j) = 0.d0
        hysales(j) = 0.d0
     else if (discreted0(j) <= 2) then
        hyprice(j) = outprice(j)*zbarq0(posqomega(j))
        hysales(j) = hyprice(j)**(1.-mysigma)*chiq0(posqomega(j))
     else        
        hyprice(j) = outprice(j)*zbarq0(posqomega(j))
        hysales(j) = hyprice(j)**(1.-mysigma)*(chiq0(posqomega(j)) + chiqf0(posqomega(j)))
     end if
    end do
    
    ! hypothetical change in sales
    where(discreted == 0) 
        kmeasure01 = log(hysales)   ! -infinity
    elsewhere
        kmeasure01 = log(hysales) - log(tsales0)
    end where
    
    do j = 1, sizef
        if ((discreted0(j) == 0) .or. (discreted1(j) == 0)) then
            kmeasureB01(j) = 0.d0
        else
            kmeasureB01(j) = log(chiq0(posqomega(j))) +  (1.-mysigma)*log(zbarq(posqomega(j)))  - log(chiq0(posqomega0(j))) -  (1.-mysigma)*log(zbarq(posqomega0(j)))  
        end if
    end do
    
    var1 = kmeasure01 
    call pdistribution(var1, pdkmeas)
    
    print *, ' --------------------------------------------------- '
    print *, 'distribution of hypothetical change in sales'
    print *, '10% ', pdkmeas(1) 
    print *, '25% ', pdkmeas(2) 
    print *, '50% ', pdkmeas(3) 
    print *, '75% ', pdkmeas(4) 
    print *, '90% ', pdkmeas(5)
    print *, ' ' 
        
    
    var1 = kmeasureB01 
    call pdistribution(var1, pdkmeas)
    
    print *, ' --------------------------------------------------- '
    print *, 'distribution of hypothetical change in q-measure'
    print *, '10% ', pdkmeas(1) 
    print *, '25% ', pdkmeas(2) 
    print *, '50% ', pdkmeas(3) 
    print *, '75% ', pdkmeas(4) 
    print *, '90% ', pdkmeas(5)
    print *, ' ' 
        
    
    ! ---------------------------------------------
    ! analyze outcomes by firm type: domestic, new exporters, etc.
    call antype01b
    
    call print_csv01
    
end subroutine show_counter


! ===========================================================================================
! analyze firms by type: domestic, new exporters, etc
! ===========================================================================================
subroutine antype01
    use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use counterfactual
	implicit none
    
    real(8), dimension(sizef) :: var1, employment0, employment1, avgwage, dkmeas
    integer(4) :: j, jj
    integer(4), dimension(sizef) :: ftype, auxv                                ! 0 exit, 1 domestic, 2 continuing importers, 3 continuing exporters, 4 new importers, 5 new exporters
    integer(4) :: nfirms01
    real(8), dimension(6) :: tshare, tupq, tdsales, tskm0, tskm1, tska0, tska1, tlabor0, tlabor1, tdkmeas
    
    ! ==================================================================================
    ! calculate initial and final employment to get aggregate skills by type
    avgwage = 0.d0
    where (discreted0 == 0)
        employment0 = 0.d0
    elsewhere
        avgwage = skilla0*skpremium0 + (1. - skilla0)
        employment0 = (myalpha/markup)*tsales0/avgwage
    end where

    avgwage = 0.d0
    where (discreted == 0)
        employment1 = 0.d0
        dkmeas = 0.d0
    elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment1 = (myalpha/markup)*tsales/avgwage
        dkmeas = kmeasure01
    end where
    
    ! get types
    ! --------------------------------------------
    ftype = 0
    where     ( (discreted0 >0 ) .and. (discreted1 == 1) )        ! domestic firms
        ftype = 1
    elsewhere ( (discreted0 > 1 ) .and. (discreted1 == 2) )       ! continuing impoters
        ftype = 2
    elsewhere ( (discreted0 >= 3) .and. (discreted1 == 3) )       ! continuing exporters only
        ftype = 3
    elsewhere ( (discreted0 == 4) .and. (discreted1 == 4) )       ! continuing importers and exporters
        ftype = 3
    elsewhere ( (discreted0 == 1) .and. (discreted1 == 2) )       ! new importers: from domestic to importing 
        ftype = 4
    elsewhere ( (discreted0 == 3) .and. (discreted1 == 4) )       ! new importers: from exporting only to importing 
        ftype = 4
    elsewhere ( (discreted0 == 1) .and. (discreted1 >= 3) )       ! new exporters: domestic to exporting and to exporting and importing
        ftype = 5
    elsewhere ( (discreted0 == 2) .and. (discreted1 == 4) )       ! new exporters: from importing only to importing and exporting
        ftype = 5
    end where
    
    if ( any((discreted > 0) .and. (ftype == 0)) ) then
        print *, 'not all firms are classified in antype01, counterEL.f90'
        print *, count((discreted > 0) .and. (ftype == 0)) 
        !stop
    end if

    
    ! calculate statistics by type
    ! ---------------------------------------------------------
    nfirms01 = count(ftype>0)       ! total number of firms active in 0, 1
    
    do j = 1, 6
        auxv = 0
        where (ftype == j) auxv = 1
        if (j==6) auxv = 1                                                                      ! all firms
        tshare(j) = real(count(ftype==j))/nfirms01                                              ! firm shares
        tupq(j) = count( (ftype==j) .and. (posqomega>posqomega0) )/real(count(ftype==j))        ! share of firms upgrading
        if (j==6) then
            tupq(j) = count(posqomega>posqomega0)/real(nfirms01)                                ! share of firms upgrading
            tshare(j) = 1.
        end if
        tdsales(j) = log( sum(tsales*auxv) ) - log( sum(tsales0*auxv)  )                       ! log change in sales
        tlabor0(j) = sum(employment0*auxv)                                                      ! total employment
        tlabor1(j) = sum(employment1*auxv)
        tskm0(j) = sum(employment0*skillm0*auxv)/tlabor0(j)
        tskm1(j) = sum(employment1*skillm*auxv)/tlabor1(j)
        tska0(j) = sum(employment0*skilla0*auxv)/tlabor0(j)
        tska1(j) = sum(employment1*skilla*auxv)/tlabor1(j)
        tdkmeas(j) = sum(dkmeas*auxv)/sum(auxv)
    end do
       
    
    ! --------------------------------------------------------
    print *, ' --------------------------------------------------- '
    print *, ' results by firm type'
    print *, ' 1 = domestic oriented' 
    print *, ' 2 = cont. importers' 
    print *, ' 3 = cont. exporters' 
    print *, ' 4 = new importers' 
    print *, ' 5 = new exporters' 
    print *, ' 6 = all' 
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' share, share upgrading, log change sales '
    do j = 1, 6
        print "(i5, f10.5, f10.5, f10.5)", j, tshare(j), tupq(j), tdsales(j)
    end do
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' measured skills 0, 1, 1-0 '
    do j = 1, 6
        print "(i5, f10.5, f10.5, f10.5)", j, tskm0(j), tskm1(j), tskm1(j) - tskm0(j)
    end do
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' actual skills 0, 1, 1-0 '
    do j = 1, 6
        print "(i5, f10.5, f10.5, f10.5)", j, tska0(j), tska1(j), tska1(j) - tska0(j)
    end do
    print *, ' '
    
    
    print *, ' --------------------------------------------------- '
    print *, ' k-measure, 1-0 '
    do j = 1, 6
        print "(i5, f10.5, f10.5, f10.5)", j, tdkmeas(j)
    end do
    print *, ' '
end subroutine antype01


! ===========================================================================================
! analyze firms by type: domestic, new exporters, etc
! ===========================================================================================
subroutine antype01b
    use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use counterfactual
	implicit none
    
    real(8), dimension(sizef) :: var1, employment0, employment1, avgwage, dkmeas
    integer(4) :: j, jj
    integer(4), dimension(sizef) :: ftype, auxv                                ! 0 exit, 1 domestic, 2 continuing importers, 3 continuing exporters, 4 new importers, 5 new exporters
    integer(4) :: nfirms01
    real(8), dimension(5) :: tshare, tupq, tdsales, tskm0, tskm1, tska0, tska1, tlabor0, tlabor1, tdkmeas, tdkmeasB
    
    ! ==================================================================================
    ! calculate initial and final employment to get aggregate skills by type
    avgwage = 0.d0
    where (discreted0 == 0)
        employment0 = 0.d0
    elsewhere
        avgwage = skilla0*skpremium0 + (1. - skilla0)
        employment0 = (myalpha/markup)*tsales0/avgwage
    end where

    avgwage = 0.d0
    where (discreted == 0)
        employment1 = 0.d0
        dkmeas = 0.d0
    elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment1 = (myalpha/markup)*tsales/avgwage
        dkmeas = kmeasure01
    end where
    
    ! get types
    ! --------------------------------------------
    ftype = 0
    where     ( (discreted0 >0 ) .and. (discreted1 == 1) )        ! domestic firms
        ftype = 1
    elsewhere ( (discreted0 > 1 ) .and. (discreted1 == 2) )       ! continuing impoters
        ftype = 2
    elsewhere ( (discreted0 >= 3) .and. (discreted1 == 3) )       ! continuing exporters only
        ftype = 3
    elsewhere ( (discreted0 == 4) .and. (discreted1 == 4) )       ! continuing importers and exporters
        ftype = 3
    elsewhere ( (discreted0 == 1) .and. (discreted1 == 2) )       ! new importers: from domestic to importing 
        ftype = 4
    elsewhere ( (discreted0 == 3) .and. (discreted1 == 4) )       ! new importers: from exporting only to importing 
        ftype = 4
    elsewhere ( (discreted0 == 1) .and. (discreted1 >= 3) )       ! new exporters: domestic to exporting and to exporting and importing
        ftype = 4
    elsewhere ( (discreted0 == 2) .and. (discreted1 == 4) )       ! new exporters: from importing only to importing and exporting
        ftype = 4
    end where
    
    if ( any((discreted > 0) .and. (ftype == 0)) ) then
        print *, 'not all firms are classified in antype01, counterEL.f90'
        print *, count((discreted > 0) .and. (ftype == 0)) 
        !stop
    end if

    
    ! calculate statistics by type
    ! ---------------------------------------------------------
    nfirms01 = count(ftype>0)       ! total number of firms active in 0, 1
    
    do j = 1, 5
        auxv = 0
        where (ftype == j) auxv = 1
        if (j==5) auxv = 1                                                                      ! all firms
        tshare(j) = real(count(ftype==j))/nfirms01                                              ! firm shares
        tupq(j) = count( (ftype==j) .and. (posqomega>posqomega0) )/real(count(ftype==j))        ! share of firms upgrading
        if (j==5) then
            tupq(j) = count(posqomega>posqomega0)/real(nfirms01)                                ! share of firms upgrading
            tshare(j) = 1.
        end if
        tdsales(j) = log( sum(tsales*auxv) ) - log( sum(tsales0*auxv) )                       ! log change in sales
        tlabor0(j) = sum(employment0*auxv)                                                      ! total employment
        tlabor1(j) = sum(employment1*auxv)
        tskm0(j) = sum(employment0*skillm0*auxv)/tlabor0(j)
        tskm1(j) = sum(employment1*skillm*auxv)/tlabor1(j)
        tska0(j) = sum(employment0*skilla0*auxv)/tlabor0(j)
        tska1(j) = sum(employment1*skilla*auxv)/tlabor1(j)
        tdkmeas(j) = sum(dkmeas*auxv)/sum(auxv)
        tdkmeasB(j) = sum(kmeasureB01*auxv)/sum(auxv)
    end do
       
    
    ! --------------------------------------------------------
    print *, ' --------------------------------------------------- '
    print *, ' results by firm type'
    print *, ' 1 = domestic oriented' 
    print *, ' 2 = cont. importers' 
    print *, ' 3 = cont. exporters' 
    print *, ' 4 = new importers & exporters' 
    print *, ' 6 = all' 
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' share, share upgrading, log change sales '
    do j = 1, 5
        print "(i5, f10.5, f10.5, f10.5)", j, tshare(j), tupq(j), tdsales(j)
    end do
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' measured skills 0, 1, 1-0 '
    do j = 1, 5
        print "(i5, f10.5, f10.5, f10.5)", j, tskm0(j), tskm1(j), tskm1(j) - tskm0(j)
    end do
    print *, ' '
    
    print *, ' --------------------------------------------------- '
    print *, ' actual skills 0, 1, 1-0 '
    do j = 1, 5
        print "(i5, f10.5, f10.5, f10.5)", j, tska0(j), tska1(j), tska1(j) - tska0(j)
    end do
    print *, ' '
    
    
    print *, ' --------------------------------------------------- '
    print *, ' k-measure, hypothetical sales 1-0 '
    do j = 1, 5
        print "(i5, f10.5, f10.5, f10.5)", j, tdkmeas(j)
    end do
    print *, ' '
    print *, ' --------------------------------------------------- '
    print *, ' k-measure, hypothetical q-measure 1-0, 1-0/sekm0'
    do j = 1, 5
        print "(i5, f10.5, f10.5, f10.5)", j, tdkmeasB(j), tdkmeasB(j)/sekm0
    end do
    print *, ' '
end subroutine antype01b