! -------------------------------------------------------
! calculate objective function: Match import penetration and export expansion/GDP
! -------------------------------------------------------
    subroutine calc_objci(x, obj, succ)
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(nci), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    
    ! -----------------------------------------------------------------------------------
    succ = .false.
       
    ! recover parameters from x
    call xtoguessci(x)      
    
    ! new prices, yfactor
    pf = pf0*(1+tariff1)/(1+tariff0)*pfactor
    yf = yf0*yfactor
    skpremium = wh1/wl1
    income = income0*ycfactor
    nfirms_data = nfirms_data1
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
        
    !fxq = fxq0*wq(1)
    !fxomega = fxomega0*wq(1)
    !fmomega = fmomega0*wq(1)
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilibrium(succ)
    call firmout1
    call firmout2       ! moments on skills to get labor market clearing condition/aggregate labor
    
    call momentci(obj)
    
    if (toprint) print *, 'obj counterINEL', obj, succ, '******************************'
    
    ! shows results counterfactual
    if (toprint) call show_counter
    
    end subroutine calc_objci
! ============================================================
    subroutine momentci(obj)
    use crosssec
    use counterfactual
    use parameters_estimate
    implicit none
    
    real(8), intent(out) :: obj
    real(8), dimension(sizef) :: avgwage, employment
    real(8) :: firm_imp, cimp, employment0, temployment
    ! ------------------------------------------
    
    ! aggregate international trade (changes match with data)
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    aggimps1 = (cimp + firm_imp)/absorption/(1.+tariff)
    aggexps1 = sum(exp_share*tsales*womega)/absorption
    
    ! aggregate labor demand
    avgwage = 0.d0
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*wh1 + (1. - skilla)*wl1
        employment = (myalpha/markup)*tsales/avgwage
    end where
    Lh1 = sum(employment*skilla)
    Ll1 = sum(employment*(1.-skilla))
    employment0 = Lh0+Ll0
    temployment = sum(employment)
    
    
    ! objective function
    obj = (impp_data01 - (aggimps1 - aggimps0))**2.d0
    obj = obj + 100.*(expe_data01 - (aggexps1 - aggexps0))**2.d0
    obj = obj + ((temployment - employment0)/employment0)**2.d0
    obj = obj + 1000.*(aggska - aggska0)**2.d0
    obj = obj + 0.1*(max(0., wl1 - 1.))**2.d0         
    
    ! objective function here gives priority to not increasing or decreasing hte skill intensity
    
    if(toprint) then
        print *, '-----------------------------------------------------------------'
        print *, 'OBJECTIVE FUNCTION DECOMPOSITION'
        print *, 'D trade/absorption, data, model, abs'
        print *, 'imp ', impp_data01, (aggimps1 - aggimps0), abs(impp_data01 - (aggimps1 - aggimps0))
        print *, 'exp ', expe_data01, (aggexps1 - aggexps0), 100.*abs(expe_data01 - (aggexps1 - aggexps0))
        print *, 'labor demand, change, obj'
        print *, 'total', ((temployment - employment0)/employment0), ((temployment - employment0)/employment0)**2.d0
        print *, 'ska  ', (aggska - aggska0), 1000.*(aggska - aggska0)**2.d0
        print *, 'ska check = 0?', (aggska - Lh1/(Lh1+Ll1))
        print *, 'wL>1', wl1, 0.1*(max(0., wl1 - 1.))**2.d0
        print *, ' '   
    end if
    
    end subroutine momentci
    ! =========================================================
subroutine  guesstoxci(x) 
    use parameters_model
	use counterfactual
	implicit none

    real(8), dimension(nci), intent(out) :: x
    integer :: j
    ! -------------------------------------------
    
    x(1) = log(yfactor)
    x(2) = log(pfactor)
    x(3) = log(wl1)
    x(4) = log(wh1/(skpremium0*wl1))
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, guesstoxc:'
        do j = 1,nci
            print*, j, x(j)
        end do
        stop
    end if
        
end subroutine guesstoxci

! =========================================================
subroutine  xtoguessci(x) 
    use parameters_model
    use parameters_estimate
	use counterfactual
	implicit none

    real(8), dimension(nci), intent(in) :: x
    integer :: j
    ! -------------------------------------------
    
    yfactor = exp(x(1))
    pfactor = exp(x(2))
    wl1 = exp(x(3))
    wh1 = skpremium0*wl1*exp(x(4))
        
    if (any(isnan(x))) then
        print *, 'parameters not a number, xtoguessc:'
        do j = 1, nci
            print*, j, x(j)
        end do
        stop
    end if
    
    if (toprint) then
        print *, 'ycfactor', ycfactor
        print *, 'yfactor ', yfactor
        print *, 'pfactor ', pfactor
        print *, 'wl1     ', wl1
        print *, 'wh1     ', wh1
        print *, ' ' 
    end if
end subroutine xtoguessci