! counterfactuals to autarky
! ==============================================
! -------------------------------------------------------
! INELASTIC LABOR
! -------------------------------------------------------
    subroutine calc_auti(x, obj, succ)
    use crosssec
    use parameters_model
    use parameters_estimate
    use counterfactual
    use parameters_firm
    use parameters_quality
	implicit none

	real(8), dimension(nc), intent(in) :: x
    real(8), intent(out) :: obj
    logical, intent(out) :: succ
    integer(4) :: j
    real(8), dimension(sizeq) :: phihl, aux1, wq0
    real(8), dimension(sizef) :: avgwage, employment
        
    ! -----------------------------------------------------------------------------------
    succ = .false.
    
    ! new wages, skill premium
    wl1 = exp(x(1))
    wh1 = wl1*exp(x(2)) 
    skpremium = wh1/wl1
    nfirms_data = nfirms_data1
    
    ! new labor CES cost
    aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
    phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
    wq0 = (skpremium0**(1.-sigmal)*phihl + 1.)**(1./(1.-sigmal))
    wq = (wh1**(1.-sigmal)*phihl + wl1**(1.-sigmal))**(1./(1.-sigmal))
    wq = wq/wq0
    
    ! calculate new price indices and demand for foreign goods
    call gen_xquality
    call equilib_aut(succ)
    call firmAUT       ! moments on skills to get labor market clearing condition/aggregate labor
    
    ! calculate moments
    ! ------------------------------------------
    obj = 0.d0
    ! aggregate labor demand
    avgwage = 0.d0
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*wh1 + (1. - skilla)*wl1
        employment = (myalpha/markup)*tsales/avgwage
    end where
    Lh1 = sum(employment*skilla)
    Ll1 = sum(employment*(1.-skilla))
    
    ! objective function
    obj = obj + ((Lh1 - Lh0)/Lh0)**2.d0
    obj = obj + ((Ll1 - Ll0)/Ll0)**2.d0
    
    if(toprint) then
        print *, 'labor demand, change, obj'
        print *, 'H', ((Lh1 - Lh0)/Lh0), ((Lh1 - Lh0)/Lh0)**2.d0
        print *, 'L', ((Ll1 - Ll0)/Ll0), ((Ll1 - Ll0)/Ll0)**2.d0
        print *, ' '   
    end if
            
    if (toprint) print *, 'obj AUTARKY, inelastic', obj, succ, '******************************'

    !if ((obj<1.e-6) .and. (succ)) then
    !    call print_fvvc(x, obj)
    !    print *, 'algorithm stopped at calc_auti in counterz_aut.f90'
    !    stop
    !end if
		
        
        
    end subroutine calc_auti

! =====================================================================================
! elastic labor, quality choice
! -----------------------------------------------
    subroutine equilib_aut(succ)
    use numerical_libraries
	use parameters_qf
	use parameters_estimate
    use parameters_firm
    use parameters_quality
    use autarky
	implicit none

    logical, intent(out) :: succ
    
    ! checking income and labor demand does not make sense without skill variables
    integer(4), parameter :: maxiter = 200
    real(8), parameter :: tolerr = 1.e-4 
     ! -----------------------------------------------------
    ! estimation auxiliary parameters
    integer(4) :: j, jj, iter
	integer(4), dimension(sizef) :: oldposq		            ! choice of quality (position in quality grid)
	integer(4), dimension(sizef) :: olddiscd				! discrete choices on exit, importing and exporting
    logical :: check_all
    integer :: njump, nlgjump, ndiscrete
    real(8) :: errp, errchi
    real(8) :: rd
    real(8), dimension(sizeq) :: oldchi, oldp
    logical :: check_succ
        
	! ======================================================================================
    ! reset parameters
    succ = .false.
	iter = 0
    check_all = .false.
    where(discreted == 0) posqomega = -1
    where(posqomega ==-1) discreted = 0
    if(all(discreted == 0)) then; discreted = 1; posqomega = 100; end if    
    where(discreted>1) discreted=1
    ! ================================================================
    outer_loop: do
        ! stop commands	! calc_obj prints success for each evaluation in polytope
		! --------------------------------
		if ( (nlgjump == 0) .and. (ndiscrete == 0) .and. (njump == 0) .and. (abs(errp) <= tolerr) .and. (abs(errchi) <= tolerr) .and. (check_all) ) then   
            succ = .true.
            if(toprint) print *, 'autarky equilibrium found', iter, succ
            exit outer_loop
        elseif (iter >= maxiter) then
		    if(toprint) then
                print *, 'equilibrium not found', iter, check_all
                print *, 'nchanges', nlgjump, njump, ndiscrete
                print *, 'err', errp, errchi
                print *, ' '
            end if
            exit outer_loop
        end if
        
        ! record initial values
        ! --------------------------------------------------------------------------------------------
        oldposq = posqomega
	    olddiscd = discreted
        oldchi = chiq
        oldp = priceqh
        
        ! find representative firms, aggregate prices and demand. Files in auxequilibrium.f90
        ! --------------------------------------------------------------------------------------------
        call nqzqAUT             ! representative firms mass and productivity
        call pricesAUT            ! price indices
        call chiqAUT              ! demand shifter
        
        ! calculate profits and update choices
        ! --------------------------------------------------------------------------------------------
        call profitsAUT(njump, nlgjump, ndiscrete, check_all)
        
        ! calculate errors
        errp = maxval(abs(oldp - priceqh)) + maxval(abs(oldp - priceqh)/priceqh)
        errchi = maxval(abs(oldchi - chiq)) + maxval(abs(oldchi - chiq)/chiq)
                
        iter = iter + 1
    end do outer_loop
    
    nq = 0.d0
    zq = 0.d0
    nq(:,1) = nqa
    zq(:,1) = zqa
    
    end subroutine equilib_aut

! ===============================================================
! Given the equilibrium of the economy, derives the characteristics of firms
! skill moments
! ===============================================================

    subroutine firmAUT
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    use counterfactual
    use autarky
    implicit none
    
    integer(4) :: j, firmj, qj
    
    real(8), dimension(sizeq) :: phihl, aux1, aux2
    real(8) :: Fl, Fu, up, pil                          ! measurement error in skill
    real(8) :: twageh, tlh, tl                          ! total wage bill of skilled. Total number of workers skilled, all
    real(8) :: firml
    real(8) :: tlha
    
    ! ===============================================
    ! outcomes by firm: sales, import and export intensity only
    ! ===============================================
    if (any(discreted>1)) then
        print *, 'calculate moments from autarky with discreted>1, firmAUT in counterz_AUT.f90'
        stop
    end if
    
    imp_share = 0.d0
    exp_share = 0.d0
    
    ! skill intensity by quality
    ! ===============================================
     aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
     phihl = aux1/(1.-aux1)                             ! Phi_L(skill)/Phi_L(unskill)
     aux2 = (skpremium)**(-sigmal)*phihl
     skillq = aux2/(1.+aux2)
     Fl = 0.5d0 
   
     wageq = skillq*skpremium + 1.                      ! This wage is only used for proportional demand for skills. Its level does not matter.
   
     twageh = 0.d0; tlh = 0.d0; tl = 0.d0; tlha = 0.d0
     do j = 1, sizef
		select case(discreted(j))
        case(0)
            outprice(j) = 0.d0
            tsales(j) = 0.d0
            skilla(j) = 0.d0
            skillm(j) = 0.d0
        case(1)
            outprice(j) = markup*priceqh**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            tsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            ! calculations for measured skill intensity
            ! truncated logistic  x = pibar + varpi*log(p/(1-p)) where p = (Fu-Fl)*rd + Fl due to truncation.
            skilla(j) = skillq(posqomega(j))
            Fu = 1.d0 - skilla(j)
            Fu = exp(Fu/varpi)/(1. + exp(Fu/varpi))        ! truncation of logistic
            pil = varpi*log( ( (Fu - Fl)*random_skill(j) + Fl ) / ( 1. - (Fu - Fl)*random_skill(j) - Fl) )
            skillm(j) = skillq(posqomega(j)) + pil 
            if (isnan(skillm(j) )) then
                skillm(j) = 1.
                pil = 1./skilla(j) - 1.d0
            end if
            ! accumulate wage bill and number of workers to get aggregate
            firml = womega*tsales(j)*(myalpha/markup)/wageq(posqomega(j))
            twageh = twageh + firml*( skpremium*skillq(posqomega(j)) + pil*(1. - skillq(posqomega(j))) )
            tlh = tlh + firml*skillm(j)
            tl = tl + firml
            tlha = tlha + firml*skilla(j)
        end select
    end do

    ! Normalized sales, domestic
    ! ----------------------------------------------------------------------------
    call count_firms
    absorption = sum(tsales*womega)
    
    ! aggregate skill premium and skill intensity
    ! ----------------------------------------------------------------------------
    aggskpm = twageh/tlh
    aggskm = tlh/tl
    aggska = tlha/tl
       
    if(toprint) then
    print *, ' --------------------------------------------------- '
    print *, ' aggregate skills - AUTARKY'
    print *, ' '
    print *, ' skill premium 0, 1, (1-0)/0'
    print *, skpremium0, skpremium, (skpremium - skpremium0)/skpremium0
    print *, 'skill intensity, initial, final, (final - initial)'
    print *, 'modelm', aggskm0, aggskm, aggskm - aggskm0
    print *, 'modela', aggska0, aggska, aggska - aggska0
    print *, ' '
    end if
    
    end subroutine firmAUT
    
    
!****************************************************************************
!	get profits for all choices of each firm and update its choice
!****************************************************************************

subroutine  profitsAUT(njump, nlgjump, ndiscrete, check_all)                          
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use autarky
	implicit none

	integer, intent(out) :: njump, nlgjump, ndiscrete
    logical, intent(out) :: check_all
    integer(4), dimension(sizef) :: oldposq, olddiscd
    
    real(8), dimension(sizeq) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8), dimension(sizef) :: fprofit
    
   ! --------------------------------------------------------------------
    njump = 0
    nlgjump = 0
    ndiscrete = 0
    check_all = .true.
    oldposq = posqomega
    olddiscd = discreted
    
    !$OMP PARALLEL shared(posqomega, discreted), private(profit, pos_maxpi)
    !$OMP do
    do j = 1, sizef
        ! calculate profit for firm j
         profit = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)*wq**myalpha/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq
         
         ! update its choice
         fprofit(j) = maxval(profit)
         pos_maxpi = maxloc(profit)
         posqomega(j) = pos_maxpi(1)
         
        
    end do
    !$OMP END do
    !$OMP END PARALLEL
    
    where(fprofit < 0.) 
        posqomega = -1
        discreted = 0
    end where
    
    nlgjump   = count(abs(oldposq - posqomega) > 1) 
    njump     = count(oldposq /= posqomega)
    ndiscrete = count(discreted /= olddiscd)
    
end subroutine profitsAUT
    

    
! ***********************************************************************************************
! auxiliary files for equilibrium in autarky
!
! nqzq : finds representative firm's mass and productivity
! prices: finds price indices
! chiq: demand function chi
! ***********************************************************************************************
! ================================================================================================
subroutine  nqzqAUT
	use parameters_quality
    use parameters_estimate
    use parameters_firm
    use parameters_qf
    use autarky
    implicit none

	integer(4) :: j, jj
	! ----------------------------------------------------

    nqa = 0.d0
    zqa = 0.d0
    do j = 1, sizef
        if (discreted(j)>0) then
            nqa(posqomega(j)) = nqa(posqomega(j)) + womega
	        zqa(posqomega(j)) = zqa(posqomega(j)) + (zqomega(j, posqomega(j))**(mysigma-1.))*womega
 	    end if
    end do
    where(nqa>0.d0) zqa = (zqa/nqa)**(1./(mysigma-1.))
    
    if ( any(isnan(zq)) .or. any(isnan(zq)) .or. any(isnan(zq*0.))  .or. any(isnan(nq*0.)) ) then
        print *, 'zq, nq is Nan in auxequilibrium.f90, calc_nqzqd'
        stop
    end if
        
end subroutine  nqzqAUT


! ================================================================================================
subroutine pricesAUT
    use parameters_quality
    use parameters_estimate
    use autarky
    implicit none
    
    ! ----------------------------------------------------
    real(8), parameter :: tolp = 1.e-15
	integer(4), parameter :: maxiter = 500
	integer(4) :: iter, j, outq
	real(8) :: error
	real(8), dimension(sizeq) :: oldpq, qvdiff, phivo
	! ----------------------------------------------------
    
    iter = 1
    error = 10.*tolp
    
    ! firm prices
    ! =======================================================================================
    price_loop: do
        ! stop commands --------------------------------------
        if (error <= tolp) then
            !print *, 'prices converged', iter
            exit price_loop
	    elseif (iter >= maxiter) then
    		print *, "prices did not converge ********************", error
	    	!stop
        end if
        ! ----------------------------------------------------
        
        ! calculate auxiliary prices from new price guess
        priceqa = priceqh
        rfirmpa = markup*priceqh**(1.-myalpha)*wq**myalpha/zqa
        
        ! calculate new priceqh
        oldpq = priceqh
        priceqh = ( sum( rfirmpa**(1.-mysigma)*nqa*0.5d0 ) )**(1./(1.-mysigma))
        
    
        ! get errors
        error = sum(abs(oldpq - priceqh))
        iter = iter + 1
    end do price_loop
    
    ! use final price to recalculate auxiliary prices and consumer prices
    priceqa = priceqh
    rfirmpa = markup*priceqh**(1.-myalpha)*wq**myalpha/zqa
    
    ! consumer prices
    ! =======================================================================================
    call phiv(sizeq, qgrid, 1.d0, phivo)
    pch = ( sum ( rfirmpa**(1.-mysigma)*nqa * phivo ) )**(1./(1.-mysigma))
    pca = pch
       
end subroutine pricesAUT

! ================================================================================================
subroutine chiqAUT
    use parameters_quality
    use parameters_estimate
    use autarky
    implicit none
    
    ! ----------------------------------------------------
    real(8), parameter :: tolchi = 1.e-10
	integer(4), parameter :: maxiter = 1000
	integer(4) :: iter, j, outq
	real(8) :: error, rd
	real(8), dimension(sizeq) :: oldchi, qvdiff, phivo
    real(8), dimension(sizeq) :: rfirmaux
	
    ! consumption
    ! ----------------------------------------------------
    call phiv(sizeq, qgrid, 1.d0, phivo)
    consumptionchi = phivo*pch**(mysigma-1.d0)*income
    
    ! intermediates
    ! ================================================================
    iter = 1
    error = 100.*tolchi
    
    chi_loop : do
        !stop commands
    	! --------------------------------
    	if (error <= tolchi) then
            !print *, 'chi converged   ', iter
    		exit chi_loop
    	elseif ( (iter >= maxiter) .or. (isnan(error)) .or. (any(isnan(intermediatechi/intermediatechi) .or. (isnan(intermediatechi)) )) ) then
    		print *, "chi did not converge", iter, error
            stop
        end if
        oldchi = intermediatechi   
    
        ! calculate total chi and rfirmaux = P^(sigma-1) * rt(representative firm) * weightnq * constant
        chiq = consumptionchi + intermediatechi
        rfirmaux = nqa * rfirmpa**(1.-mysigma) * chiq * priceqh**(mysigma-1.)
        rfirmaux = rfirmaux*(1.-myalpha)/markup
        
        ! new guess for chiq
        outq_loop2: do outq = 1, sizeq
            intermediatechi(outq) = sum( rfirmaux * 0.5d0 )
        end do outq_loop2
    
        if (error>50.) then
            call random_number(rd) 
            intermediatechi = (0.7 + rd*0.3)*intermediatechi + (0.3 - rd*0.3)*oldchi 
        end if
                        
        error = maxval(abs(intermediatechi - oldchi)/intermediatechi) + maxval(abs(intermediatechi - oldchi))
        iter = iter + 1
    end do chi_loop
    
    chiq = consumptionchi + intermediatechi  
end subroutine chiqAUT

!****************************************************************************
!	get profits for all choices of each firm and update its choice
!****************************************************************************

subroutine  profitsAUTnq(njump, nlgjump, ndiscrete, check_all)                          
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use autarky
    use counterfactual
	implicit none

	integer, intent(out) :: njump, nlgjump, ndiscrete
    logical, intent(out) :: check_all
    integer(4), dimension(sizef) :: oldposq, olddiscd
    
    real(8) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
   ! --------------------------------------------------------------------
    njump = 0
    nlgjump = 0
    ndiscrete = 0
    check_all = .true.
    oldposq = posqomega
    olddiscd = discreted
    
    do j = 1, sizef
        
        if(discreted0(j)>0) then
        ! calculate profit for firm j
         profit = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)*wq(posqomega0(j))**myalpha/zqomega(j,(posqomega0(j))) )**(1.-mysigma) )*chiq(posqomega0(j)) - fxq(posqomega0(j))
         if (profit>0.d0) then
             discreted(j) = 1
             posqomega(j) = posqomega0(j)
         else
             discreted(j) = 0
             posqomega(j) = -1
         end if
         
         end if
         
    end do
    
    nlgjump   = count(abs(oldposq - posqomega) > 1) 
    njump     = count(oldposq /= posqomega)
    ndiscrete = count(discreted /= olddiscd)
    
    where(discreted0 == 0) 
        discreted = 0
        posqomega = 0
    end where
    
end subroutine profitsAUTnq