!****************************************************************************
!   estimate equilibrium - firms' quality, import and export behavior, revenue, demand for intermediates and labor
!****************************************************************************

subroutine equilibrium(succ)
	use numerical_libraries
	use parameters_qf
	use parameters_estimate
    use parameters_firm
    use parameters_quality
	implicit none

    logical, intent(out) :: succ
    
    ! checking income and labor demand does not make sense without skill variables
    integer(4), parameter :: maxiter = 200
    real(8), parameter :: tolerr = 1.e-4 
    integer :: startime, endtime, hz
	    
    ! -----------------------------------------------------
    ! estimation auxiliary parameters
    integer(4) :: j, jj, iter
	integer(4), dimension(sizef) :: oldposq		            ! choice of quality (position in quality grid)
	integer(4), dimension(sizef) :: olddiscd				! discrete choices on exit, importing and exporting
    logical :: check_all
    integer :: njump, nlgjump, ndiscrete
    real(8) :: errp, errchi
    real(8) :: rd
    real(8), dimension(sizeq) :: oldchi
    real(8) :: oldp
    logical :: check_succ
        
	! ======================================================================================
    ! reset parameters
    succ = .false.
	iter = 0
    check_all = .false.
    where(discreted == 0) posqomega = -1
    where(posqomega ==-1) discreted = 0
    if(all(discreted == 0)) then; discreted = 4; posqomega = 100; end if    
    
    ! =============================================================================================
    CALL SYSTEM_CLOCK(COUNT_RATE=hz); CALL SYSTEM_CLOCK(COUNT=startime)     ! start clock
    
    ! ================================================================
    outer_loop: do
        ! stop commands	! calc_obj prints success for each evaluation in polytope
		! --------------------------------
		!if ( (nlgjump <= sizef*.0005) .and. (ndiscrete<=sizef*.0005) .and. (njump < sizef/1000.) .and. (abs(errp) <= tolerr) .and. (abs(errchi) <= tolerr) .and. (check_all) ) then   
        if ( (nlgjump == 0) .and. (ndiscrete == 0) .and. (njump == 0) .and. (abs(errp) <= tolerr) .and. (abs(errchi) <= tolerr) .and. (check_all) ) then   
            succ = .true.
            if(toprint) print *, 'equilibrium found', iter, succ
            exit outer_loop
        elseif (iter >= maxiter) then
		    if(toprint) then
                print *, 'equilibrium not found', iter, check_all
                print *, 'nchanges', nlgjump, njump, ndiscrete
                print *, 'err', errp, errchi
                print *, ' '
            end if
            exit outer_loop
        end if
        
        ! record initial values
        ! --------------------------------------------------------------------------------------------
        oldposq = posqomega
	    olddiscd = discreted
        oldchi = chiq
        oldp = priceqh
        
        ! find representative firms, aggregate prices and demand. Files in auxequilibrium.f90
        ! --------------------------------------------------------------------------------------------
        where(discreted == 0) posqomega = -1
        where(posqomega ==-1) discreted = 0
        call calc_nqzqd             ! representative firms mass and productivity
        call prices                 ! price indices
        call calc_chiq              ! demand shifter
        
        ! calculate profits and update choices
        ! --------------------------------------------------------------------------------------------
        call profits(njump, nlgjump, ndiscrete, check_all)
        
        ! calculate errors
        errp = abs(oldp - priceqh) + abs(oldp - priceqh)/priceqh
        errchi = maxval(abs(oldchi - chiq)) + maxval(abs(oldchi - chiq)/chiq)
                
        iter = iter + 1
        
        CALL SYSTEM_CLOCK(COUNT=endtime)
        if (DBLE(endtime - startime)/(DBLE(hz))/60 >10.) then
            print *, 'minutes, iter, jumps', DBLE(endtime - startime)/(DBLE(hz))/60, iter, njump+nlgjump+ndiscrete
            exit outer_loop
        end if
    end do outer_loop
    
    call RANDOM_NUMBER(rd)
    if ( (toprint) .or. (rd< 0.01d0) ) then
        if (toprint) print *, ' -------------------------------------------'
        call check_prices
        !call check_chi
        if (toprint) print *, ' -------------------------------------------'
        if (toprint) print *, ' '
    end if
    CALL SYSTEM_CLOCK(COUNT=endtime)
    if (DBLE(endtime - startime)/(DBLE(hz))/60 >2.) print*, "Time: ", DBLE(endtime - startime)/(DBLE(hz))/60, " minutes"
    
end subroutine equilibrium
