! ===============================================================
! Given the equilibrium of the economy, derives the characteristics of firms
! ===============================================================

    subroutine firmout1
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    implicit none
    
    integer(4) :: j, firmj, qj
    real(8), dimension(sizef) :: hsales, fsales
    real(8) :: cimp, firm_imp                   ! total consumer and firm imports
    
    ! outcomes by firm
    ! ===============================================
    do j = 1, sizef
		select case(discreted(j))
        case(0)
            outprice(j) = 0.d0
            hsales(j) = 0.d0
            fsales(j) = 0.d0
            tsales(j) = 0.d0
            imp_share(j) = 0.d0
            exp_share(j) = 0.d0
        case(1)
            outprice(j) = markup*priceqh**(1.-myalpha)/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = 0.d0
            tsales(j) = hsales(j) + fsales(j)
            imp_share(j) = 0.d0
            exp_share(j) = 0.d0
        case(2)
            outprice(j) = markup*priceqa**(1.-myalpha)/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = 0.
            tsales(j) = hsales(j) + fsales(j)
            exp_share(j) = 0.
            imp_share(j) = ( priceqf/priceqa )**(1.-mysigma)
        case(3)
            outprice(j) = markup*priceqh**(1.-myalpha)/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = outprice(j)**(1.-mysigma)*chiqf(posqomega(j))
            tsales(j) = hsales(j) + fsales(j)
            exp_share(j) = fsales(j)/tsales(j)
            imp_share(j) = 0.
        case(4)
            outprice(j) = markup*priceqa**(1.-myalpha)/zqomega(j, posqomega(j))
            hsales(j) = outprice(j)**(1.-mysigma)*chiq(posqomega(j))
            fsales(j) = outprice(j)**(1.-mysigma)*chiqf(posqomega(j))
            tsales(j) = hsales(j) + fsales(j)
            exp_share(j) = fsales(j)/tsales(j)
            imp_share(j) = ( priceqf/priceqa )**(1.-mysigma)
        end select
        
    end do

    ! Normalized sales, domestic
    ! ----------------------------------------------------------------------------
    call count_firms
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    timport = cimp + firm_imp
    absorption = sum(hsales*womega) + timport/(1.+tariff)
    nhsales = hsales*womega/(sum(hsales*womega)+firm_imp+cimp)*(sizef-nexit)/nfirms_data
        
    end subroutine firmout1