! ===============================================================
! Given the equilibrium of the economy, derives the characteristics of firms
! ===============================================================

    subroutine firmout2
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    implicit none
    
    integer(4) :: j, firmj, qj
    real(8), dimension(sizeq) :: phihl, aux1, aux2
    real(8) :: Fu, Fl, pil                      ! measurement error in skill
    real(8) :: twageh, tlh, tl, tlha                  ! total wage bill of skilled. Total number of workers skilled, all
    real(8) :: firml
    
    ! skill intensity by quality
    ! ===============================================
     aux1 = l3*exp(l1+l2*qgrid)/(1.+exp(l1+l2*qgrid))
     phihl = aux1/(1.-aux1)                         ! Phi_L(skill)/Phi_L(unskill)
     aux2 = (skpremium)**(-sigmal)*phihl
     skillq = aux2/(1.+aux2)
     Fl = 0.5d0 
     wageq = skillq*skpremium + 1.
     
    ! outcomes by firm
    ! ===============================================
    twageh = 0.d0; tlh = 0.d0; tl = 0.d0; tlha = 0.d0
    do j = 1, sizef
		if (discreted(j) == 0) then
            skilla(j) = 0.d0
            skillm(j) = 0.d0
        else
            ! calculations for measured skill intensity
            ! truncated logistic  x = pibar + varpi*log(p/(1-p)) where p = (Fu-Fl)*rd + Fl due to truncation.
            skilla(j) = skillq(posqomega(j))
            Fu = 1.d0 - skilla(j)
            Fu = exp(Fu/varpi)/(1. + exp(Fu/varpi))        ! truncation of logistic
            pil = varpi*log( ( (Fu - Fl)*random_skill(j) + Fl ) / ( 1. - (Fu - Fl)*random_skill(j) - Fl) )
            skillm(j) = skillq(posqomega(j)) + pil 
            if (isnan(skillm(j) )) then
                skillm(j) = 1.
                pil = 1./skilla(j) - 1.d0
            end if
           ! accumulate wage bill and number of workers to get aggregate
            firml = tsales(j)*(myalpha/markup)/wageq(posqomega(j))
            twageh = twageh + firml*( skpremium*skillq(posqomega(j)) + pil*(1. - skillq(posqomega(j))) )
            tlh = tlh + firml*skillm(j)
            tl = tl + firml
        end if
    end do

    ! aggregate skill premium and skill intensity
    ! ----------------------------------------------------------------------------
    aggskpm = twageh/tlh
    aggskm = tlh/tl
    aggska = sum(skilla*tsales/((1.-skilla) + skpremium*skilla))/sum(tsales/((1.-skilla) + skpremium*skilla))
        
    end subroutine firmout2