! ****************************************************************
! PURPOSE: Match moments and calculate objective function
! ****************************************************************

subroutine moments3(obj)
	use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
	implicit none
    
    real(8), intent(out) :: obj
    !integer(4), dimension(sizef) :: qsize, qwage
    !real(8), dimension(sizef) :: var1, var2
    !real(8), dimension(4,4) :: wquartiles, qsalesw_model
    integer(4) :: j, jj !, pospi(1)
    !real(8) :: psize_model(5), pskill_model(5)
    !real(8), dimension(4) :: sksize_model, exint_model, imint_model, exshr_model, imshr_model
    !real(8) :: aux1, skusa_model
    !real(8) :: objdata
    
    ! price regressions
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: outcoeff       ! regression results: coefficients
        
    ! ==================================================================================
    obj = 0.d0
    
    ! price regressions
    ! ----------------------------------------------------------------------------------
    tobs = sizef - nexit
    allocate(yvector(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! first regression: output prices on measured skill intensity
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            xmat(jj,2) = skillm(j)
        end if
    end do
    ! because I get an error that prices are too high, demean yvectors
    yvector = yvector - sum(yvector)/real(tobs)
    
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols_novar(tobs, nvar, yvector, xmat, outcoeff)      ! output prices on skill intensity
    obj = obj + mweight(48)*(outcoeff(2) - skill_pdata(1))**2. 
    deallocate(yvector, xmat)
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on (skill intensity)'
        print*, 'output', skill_pdata(1), outcoeff(2),  mweight(48)*(outcoeff(2) - skill_pdata(1))**2.
        print *, ' '
        print *, ' ------------------------------------------------------------'
        print *, ' objective prices = ', obj
        print *, ' ------------------------------------------------------------'
    end if
    
end subroutine moments3
