! read initial guesses, random variables, and standard errors from data

! ==========================================================================
! initial guesses
! ==========================================================================
subroutine  read_initial(x, myflag)
	use parameters_firm
    use parameters_quality
    implicit none

	logical, intent(in) :: myflag
	real(8), dimension(n), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	open(1, FILE='may17nu0b.txt', STATUS='OLD')   ! weighting is identity matrix
    !open(1, FILE='may17nu0a05.txt', STATUS='OLD') ! weighting is identity matrix
    
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, n
        if (myflag) then
            read(1,*), x(j)
        else
            read(1,*), aux 
        end if
    end do
    do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_initial 

! diagonal of weighting matrix 
! =====================================================================    
subroutine  read_weights
    use data_moments
	implicit none
	integer(4) :: j
    
    ! ------------------------------------------
    mweight = 0.001d0      ! some moments, like US data and exit do not have data
    mweight(51) = 0.001d0
    mweight(52) = 0.01d0
    
    ! standard errors from bootstrap
    open(1, FILE='mse_2016.dat', STATUS='OLD')
    do j = 1, nboot
        read(1,*), mweight(j)       ! standard error of moments
    end do
    close(1)
    
    ! moments from regression results on prices
    mweight(j:j+1) = skill_sepdata*0.1
    mweight = 1./(mweight)**2.
    
    mweight = 1.d0; mweight(17:21) = 1./100.**2
end subroutine read_weights

subroutine read_random
    use numerical_libraries
    use random_draw
    implicit none    
    
    integer(4) :: j, j1
    real(8) :: rd
    ! -----------------------------------------------
    ! to get truncated distributions, add after read rd: rd = rd*.95; rd = rd*.90 + .05
    
    !real(8), dimension(sizef,2) :: random_firms
	!real(8), dimension(sizef,2) :: random_fxcost
    !real(8), dimension(sizef) :: random_skill
    
    open(1, FILE='runiform_long.dat', STATUS='OLD')
    
    do j = 1, 1111
        read(1,*), rd
    end do
    ! firm productivity
    do j1 = 1, 2
        do j = 1, sizef
            read(1,*), rd
		    random_firms(j,j1) = DNORIN(rd)       ! transforms uniform into standard normal
        end do
	end do
        
    ! importing and exporting fixed costs
    do j1 = 1, 2
        do j = 1, sizef
            read(1,*), rd
		    random_fxcost(j,j1) = DNORIN(rd)	 ! transforms uniform into standard normal
        end do
	end do
    
    ! measurement error in skills
    do j = 1, sizef
        read(1,*), rd
		random_skill(j) = rd					  ! truncated logistic
	end do
	close(1)
     
    ! create file with random draws
    ! -----------------------------
    !rdseed = 5
	!call random_seed(PUT = rdseed)
    !j = 100000
    !open(1, FILE='runiform.dat', STATUS='REPLACE')
    !do j1 = 1, j
    !    call RANDOM_NUMBER(rd)
    !    write(1,*), rd
    !end do
    !write(1,*), 'contains', j, 'draws from uniform'
    !close(1)
    !stop
    
    
    !print *, '---------------------------------------------------------- '
    !print *, 'summary of random variables: mean and variance'
    !print *, 'z		', sum(random_firms)/(2.*sizef), sum(random_firms**2.)/(2.*sizef)
    !print *, 'fm, fx', sum(random_fxcost)/(2.*sizef), sum(random_fxcost**2.)/(2.*sizef)
    !print *, 'skill ', sum(random_skill)/sizef, sum( (random_skill - 0.5d0)**2.)/sizef
    !print *, ' ' 
    
end subroutine read_random


! =================================================================================================
! =================================================================================================
subroutine  read_counter(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(nc), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	
    ! ELASTIC LABOR
    open(1, FILE='may17nu0_EL.txt', STATUS='OLD')           ! main benchmark
    !open(1, FILE='may17nu0_ELee.txt', STATUS='OLD')           ! main benchmark
    !open(1, FILE='may17nu0a05_EL.txt', STATUS='OLD')           ! main benchmark
    
    ! autarky, inelastic labor
    !open(1, FILE='may17nu0_AUT.txt', STATUS='OLD')           ! quality choice
    
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nc
        read(1,*), x(j)  
    end do
	!do j = 1, sizef
	!	read(1,*), posqomega(j)
	!end do
	!do j = 1, sizef
	!	read(1,*), discreted(j)
	!end do
	!do j = 1, sizeq
	!	read(1,*), intermediatechi(j)
	!end do
	close(1)
      
end subroutine  read_counter 


! =================================================================================================
! =================================================================================================
subroutine  read_counteri(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(nci), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	open(1, FILE='may17nu0_INEL.txt', STATUS='OLD')         ! main benchmark
    !open(1, FILE='may17nu0_INELee.txt', STATUS='OLD')      ! export expansion
    !open(1, FILE='counterNSI_INELa05B.txt', STATUS='OLD')      ! main benchmark
    !open(1, FILE='tcounteri.txt', STATUS='OLD')         ! main benchmark
     
    
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nci
        read(1,*), x(j) 
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_counteri


! ====================================================================================
! ====================================================================================
subroutine  read_countere(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(nce), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	
    ! ELASTIC LABOR, free entry
    open(1, FILE='may17nu0_ELfe.txt', STATUS='OLD')           
    !       
    read(1,*), aux
    read(1,*), auxl
	do j = 1, nce
        read(1,*), x(j)  
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_countere


! ====================================================================================
! ====================================================================================
subroutine  read_counterei(x)
	use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(ncei), intent(out) :: x
    integer(4) :: j
	real(8) :: aux
    logical :: auxl

	! ----------------------------------------------------
    ! read equilibrium posqomega, bigpq, chiq
	
    open(1, FILE='may17nu0a05_INELfe.txt', STATUS='OLD')           ! free entry
    !open(1, FILE='counterNSI_INELgB.txt', STATUS='OLD')           
     
    read(1,*), aux
    read(1,*), auxl
	do j = 1, ncei
        read(1,*), x(j)  
    end do
	do j = 1, sizef
		read(1,*), posqomega(j)
	end do
	do j = 1, sizef
		read(1,*), discreted(j)
	end do
	do j = 1, sizeq
		read(1,*), intermediatechi(j)
	end do
	close(1)
      
end subroutine  read_counterei
