!****************************************************************************
!
!  PROGRAM: modelsi
!
!  PURPOSE: Runs a simulated annealing or simplex algorithm to match moments of the data.
!
!****************************************************************************

module parameters_model
		real(8), parameter :: mysigma = 5.0d0				                ! elasticity of substitution across sectors
        real(8), parameter :: markup = mysigma/(mysigma-1.d0)
        real(8), parameter :: myalpha = 0.7d0			            	    ! Cobb-Douglas weight on labor
        real(8), parameter :: tariff0 = .32d0	
        real(8), parameter :: income0 = 1.d0
        real(8), parameter :: sigmal = 1.6d0                                ! elasticity of substitution between skilled and unskilled workers
		integer(4), parameter :: sizef = 5000
        real(8) :: womega                                                   ! firm weight (1/sizef), changes only with free entry counterfactual
        integer(4), parameter :: sizeq = 201					            ! size of q grid and number of firm draws
        real(8), parameter :: qlow = 0., qup = 10.d0                        ! lower and upper bound of existing varieties: nq varieties exist for any q\in [qlow,qbar]; low = (1.d0/(sizeq-1.))
        integer(4), parameter :: n = 18                                     ! number of parameters to be estimated. One extra is l3 which is calibrated (no standard errors)
        integer(4), parameter :: n1 = 13, n2 = 4, n3 = 1                    ! number of parameters to be estimated in each stage
        real(8) :: xminall(n), xmin1(n1), xmin2(n2), xmin3(n3)
        real(8) :: omin1, omin2, omin3
        logical :: toprint, pflag, toprint2
        real(8), parameter :: nfirms_data0 = 6710.d0                                                         ! number of plants (in March 8 moments)
        integer(4) :: nexit, nimport, nexport                               ! number of firms exiting, importing and exporting
        integer(4), parameter :: nc = 2                                     ! number of parameters of counterfactual
		integer(4), parameter :: nci = 4                                    ! ... with inelastic labor
        integer(4), parameter :: nce = 3                                    ! ... increase in sales, free entry
        integer(4), parameter :: ncei = 5                                   ! ... increase in sales and inelastic labor

        real(8) :: objtt, xtt(n)
end module parameters_model

	 
       
	! ----------------------------------------------------------------------------
	 module parameters_quality
     use parameters_model
        real(8) :: pca, pch, pcf                                        ! consumer's price indices---total, domestic and foreign only
	    real(8), dimension(sizeq) :: qgrid
	    real(8), dimension(sizeq) :: chiq, chiqf                        ! demand shifter in Home and Foreign
        real(8), dimension(sizeq) :: priceqa, priceqh, priceqf          ! price index all goods, home and foreign goods only
		real(8), dimension(sizeq) :: consumptionchi, intermediatechi
		real(8), dimension(sizeq, 4) :: rfirmp                          ! price component for each firm type; representative firm p
		real(8), dimension(sizeq, 4) :: nq, zq                  		! zq=1/aq is the productivity of a representative firm (in the Melitz sense) of quality q.
		real(8), dimension(sizeq) :: fxq	    	                    ! fixed cost of production for each firm-quality
        real(8), dimension(sizeq) :: wq	    	                    ! labor cost index for firm of quality q
     end module parameters_quality
     
	! ----------------------------------------------------------------------------
	module parameters_firm
        use parameters_model
		integer(4), dimension(sizef) :: posqomega			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	end module parameters_firm

	! ----------------------------------------------------------------------------
	module parameters_qf
		use parameters_model
		real(8), dimension(sizef) :: qomega						        ! choice of quality (level)
		real(8), dimension(sizef, sizeq) :: zqomega		        		! productivity of each firm 
        real(8), dimension(sizef) :: fxomega, fmomega                   ! firm-specific import and export cost
        real(8), dimension(sizef,2) :: skomega                          ! firm-specific measurement error of skilled workers
    end module parameters_qf
    
    ! ----------------------------------------------------------------------------
	module random_draw
	    use parameters_model
        real(8), dimension(sizef,2) :: random_firms
        real(8), dimension(sizef,2) :: random_fxcost
        real(8), dimension(sizef) :: random_skill
    end module random_draw  
    
    ! ----------------------------------------------------------------------------
	module crosssec
    use parameters_quality
    use parameters_firm
        real(8), dimension(sizef) :: imp_share, exp_share              ! share of materials imported
        real(8), dimension(sizef) :: inprice, outprice
        real(8), dimension(sizef) :: nhsales
        real(8), dimension(sizef) :: skilla, skillm                     ! actual and measured skill intensity
        real(8), dimension(sizeq) :: zbarq                              ! quantity equivalence
        real(8), dimension(sizeq) :: inpq                               ! input price
        real(8), dimension(sizeq) :: skillq                             ! skill intensity by quality level
        real(8), dimension(sizeq) :: wageq                              ! average wage of a firm with quality q
        real(8), dimension(sizef) :: kmeasure                           ! q tilde
        real(8) :: aggskm, aggskpm, aggska                                      ! aggregate measured skill intensity and premium
        real(8) :: timport, absorption
        real(8), dimension(sizef) :: tsales
    end module crosssec
    ! ----------------------------------------------------------------------------
    module parameters_polytopeall
        use parameters_model
		integer(4), parameter :: np1 = n + 1	
        integer(4), parameter ::  maxfun = 200*n, maxiter = 200*n       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), dimension(n), parameter :: usual_delta = (/ 0.01, 0.02, 0.1, 0.1, 0.02, 0.04, 0.1, 0.1, 0.1, 0.1, 0.05, 0.1, 0.01, 0.01, 0.1, 0.1, 0.1, 0.1/)
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytopeall
	! ----------------------------------------------------------------------------
    module parameters_polytope1
        use parameters_model
		integer(4), parameter :: np1 = n1 + 1	
        integer(4), parameter ::  maxfun = 200*n1, maxiter = 200*n1       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), dimension(n1), parameter :: usual_delta = (/ 0.01, 0.02, 0.02, 0.1, 0.008, 0.01, 0.04, 0.04, 0.03, 0.03, 0.01, 0.03, 0.01/)
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytope1
    ! ----------------------------------------------------------------------------
    module parameters_polytope2
        use parameters_model
		integer(4), parameter :: np1 = n2 + 1	
        integer(4), parameter ::  maxfun = 200*n2, maxiter = 200*n2       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), dimension(n2), parameter :: usual_delta = (/ 0.1, 0.1, 0.1, 0.1 /)
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytope2
	! ----------------------------------------------------------------------------
    module parameters_polytope3
        use parameters_model
		integer(4), parameter :: np1 = n3 + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), parameter :: usual_delta = 0.1d0
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytope3
    ! ----------------------------------------------------------------------------
    module parameters_polytopec
        use parameters_model
        integer(4), parameter :: np1 = nc + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), parameter :: usual_delta = 0.1d0
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytopec
    ! ----------------------------------------------------------------------------
    module parameters_polytopeci        ! inelastic labor
        use parameters_model
        integer(4), parameter :: np1 = nci + 1	
        integer(4), parameter ::  maxfun = 200*n3, maxiter = 200*n3       ! maxfun = 10, maxiter = 10 ! 
		integer(4), parameter :: two = 2
        real(8), parameter :: usual_delta = 0.1d0
		real(8), parameter :: zero_term_delta = 0.025d0, &
							prho = 1.d0, pchi = 2.d0, ppsi = 0.5d0, psigma = 0.5d0, &
							tol_fun = 5.e-4, tol_par = 1.e-3, &	        !tolerance within polytope
							ptol_fun = 5.e-4, ptol_par = 1.e-3	        !tolerance comparing polytopes
		integer(4), parameter:: maxj = 5, max_xiguess = 5               !runs at most 5 polytopes per initial guess
    end module parameters_polytopeci
        ! ----------------------------------------------------------------------------
    
    module parameters_sanneal
		use parameters_model
		real(8), parameter :: inittemp = 1.d0
        real(8), parameter :: stoptemp = 1.e-8
        integer(4), parameter :: maxconsec = 3000 !1000                      ! maximum number of rejections before program stops
        integer(4), parameter :: maxtry = 1000    !300                       ! maximum evaluations at any temperature
        integer(4), parameter :: maxsucc = 30     !20                        ! maximum number of successes within one temperature
		integer(4), parameter :: maxsanneal = 5                              ! runs at most 5 simulated annealing algorithms for any guess
        integer(4), parameter :: maxfail = 30     !20                        ! maximum number of attempts to find an equilibrium in a neighborhood of current
        real(8), dimension(n), parameter :: mydelta = (/ 0.03, 0.02, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.01, 0.05, 0.05, 0.05, 0.05, 0.05 /)
        real(8), dimension(n1), parameter :: mydelta1 = 2.*(/ 0.03, 0.02, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.01/)
        real(8), dimension(n2), parameter :: mydelta2 = (/ 0.05, 0.05, 0.05, 0.05 /)
        real(8), dimension(n3), parameter :: mydelta3 = 0.2d0
    end module parameters_sanneal
    
    ! ----------------------------------------------------------------------------
    module parameters_estimate
		real(8) :: mu1, sigma1                                          ! z1(omega) log normal mean and variance
        real(8) :: sigma2                                               ! z2(omega) normal, variance. Mean is zero.
        real(8) :: z3                                                   ! parameter of zbar
        real(8) :: f1, f2                                               ! fixed cost parameters
        real(8) :: fm, fx                                               ! fixed costs to import and export
        real(8) :: fmvar, fxvar                                         ! variance of fixed import and export cost
        real(8) :: l1, l2, l3
        real(8) :: qf                                                   ! quality of representative foreign firm 
        real(8) :: qcf                                                  ! quality of representative foreign consumer
        real(8) :: skpremium                                            ! skill premium wageh/wagel, where wagel = 1, normalized above
        real(8) :: yf 
        real(8) :: varpi                                                ! variance of firm-specific classification error
        !real(8), parameter :: nu = 1.d0                                ! complementarity parameter in production. Fixed for now.
        real(8) :: nu                                                   ! complementarity parameter in production
        real(8) :: pf
        real(8) :: tariff
        real(8) :: nfirms_data
        real(8) :: income
    end module parameters_estimate

	! ----------------------------------------------------------------------------
	! fortran_codes/data/february 2016/new baseline Jan 27.xlsx
    module data_moments
		integer(4), parameter :: sizecoef = 4
		!real(8), dimension(4,4), parameter :: qsalesw_data = reshape((/.143, .073, .029, .005, &          ! ln(avg wage), ln(domestic sales)
  !                                                                     .071, .090, .069, .020, &
  !                                                                     .030, .066, .097, .057, &
  !                                                                     .005, .020, .056, .168/), shape(qsalesw_data))
        real(8), dimension(4,4), parameter :: qsalesw_data = reshape((/.107, .070, .049, .024, &          ! ln(kmeasure), ln(domestic sales)
                                                                       .066, .067, .062, .054, &
                                                                       .044, .060, .066, .080, &
                                                                       .032, .053, .074, .091/), shape(qsalesw_data))
        
  !      ! size data need to be adjusted for absorption from Feenstra ***************************************         
        !real(8), dimension(5), parameter:: psize_data  = (/ -12.606, -11.973, -11.092, -9.858, -8.429 /)    ! ln(domestic sales) distribution of plants 10, 25, 50, 75, 90 percentiles, without feenstra adjustment
        real(8), dimension(5), parameter:: psize_data  = (/ -12.587, -11.953, -11.073, -9.838, -8.409 /)    ! ln(domestic sales) distribution of plants 10, 25, 50, 75, 90 percentiles, with feenstra adjustment        
        real(8), dimension(5), parameter:: pskill_data = (/ 0.059, 0.125, 0.217, 0.343, 0.500 /)            ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(7), parameter:: pkimp_data = (/ -3.93, -2.90, -1.46, -0.04, 1.43, 2.98, 4.00 /)            ! k-measure kimp_ait distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(4), parameter:: sksize_data = (/ .196, .225, .265, .336 /)                       ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(4), parameter :: exint_data = (/ 1.5, 1.0, 1.6, 2.6 /) / 100.                    ! average export intensity by quartile of domestic sales 
        real(8), dimension(4), parameter :: imint_data = (/ 1.9, 3.7, 7.6, 18.8 /) / 100.                   ! average import intensity by quartile of domestic sales
        real(8), dimension(4), parameter :: exshr_data = (/ 2.7, 3.6, 8.8, 27.6 /)/ 100.                    ! % exporters by quartile of domestic sales
        real(8), dimension(4), parameter :: imshr_data = (/ 7.4, 12.4, 24.9, 58.3 /) / 100.                 ! % importers by quartile of domestic sales
        real(8), dimension(4), parameter :: kimp_ai = (/ -1.168, -0.336, 0.222, 0.888 /)                    ! Khandelwal's measure of quality; out of sample; median set to 0 or average to 1?
        real(8), parameter :: skshare_data = 0.290d0                                                        ! aggregate measured skill share
        real(8), parameter :: skpremium_data = 1.586d0                                                      ! skill premium = (average wage of white/average wage of blue-collar workers) 
        real(8), parameter :: exitrate = 0.10d0                                                            ! historical exit rate
        real(8), parameter :: imps_data = 0.162d0, exps_data = 0.045d0                                      ! aggregate import and export shares, using Feenstra numbers, see aggregate folder
        
        ! data on USA skill premium and skill intensity for estimating bounds (Autor, Katz, Krueger, QJE1998) See calculation in myaggregates_colombiagov.xlsx
        ! ----------------------------------------------------------------------------
        !real(8), parameter :: USAskint =  (0.393d0+0.310d0)/2.                          ! table 1 appendix page 1209, average between 1990 and 1980 census, college equivalents
        !!!real(8), parameter :: USAskpremium = 1.52d0                                     ! table 1 appendix page 1209, average between 1990 and 1980 census, college equivalents
        real(8), parameter :: USAskint =  .229d0                          ! table 1 appendix page 1209, average between 1990 and 1980 census, college equivalents
        !real(8), parameter :: USAskpremium = (exp(.391d0) + exp(.549d0))/2.d0           ! table 1, average between 1990 and 1980 census, college 
        
        ! price regressions
        ! ----------------------------------------------------------------------------
        ! coefficients
        real(8), dimension(2), parameter :: skill_pdata = (/ 0.364d0, 0.157d0 /)
        real(8), dimension(2), parameter :: sales_pdata = (/ 0.0194d0, 0.0176d0 /)
        real(8), dimension(2), parameter :: kimpai_pdata = (/ 0.196d0, 0.0276d0 /)
        
        ! standard errors
        real(8), dimension(2), parameter :: skill_sepdata = (/ 0.037d0, 0.019d0 /)
        real(8), dimension(2), parameter :: sales_sepdata = (/ 0.0055d0, 0.0026d0 /)
        real(8), dimension(2), parameter :: kimpai_sepdata = (/ 0.0018d0, 0.0014d0 /)
        
        ! ----------------------------------------------------------------------------
        integer(4), parameter :: nweight = 53
        real(8), dimension(nweight) :: mweight   ! weighting matrix is a diagonal matrix, and it should be the inverse of variance
        integer(4), parameter :: nboot = 47
        
    end module data_moments    

    
    ! ----------------------------------------------------------------------------
	module counterfactual
        use parameters_model
		integer(4), dimension(sizef) :: posqomega0, posqomega1			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted0, discreted1			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	    real(8), dimension(sizef) :: imp_share0, exp_share0, imp_share1, exp_share1 ! share of materials imported
        real(8), dimension(sizef) :: nhsales0, ntsales0, tsales0
        real(8), dimension(sizef) :: skilla0, skillm0
        real(8), dimension(sizef) :: firml1, firml0                                 ! employment
        real(8) :: aggskm0, aggskpm0 !, aggskm1, aggskpm1                           ! aggregate measured skill intensity and premium
        real(8) :: aggska0 !, aggska1                                               ! aggregate skill intensity
        real(8) :: absorption0 !, absorption1
        real(8) :: aggimps0, aggimps1, aggexps0, aggexps1
        integer(4) :: nexit0, nexit1
        real(8), dimension(sizeq) :: chiq0, chiqf0                                  ! used to calculate changes in K-measures
        real(8), dimension(sizeq) :: priceqh0, priceqa0                             ! used to calculate changes in input prices, inform about the mechanism
        real(8), dimension(sizeq, 4) :: nq0
        real(8), parameter :: pf0 = 1.d0
        real(8) :: yf0, yf1, pf1, skpremium0, skpremium1 
        real(8), dimension(sizef) :: kmeasure0, kmeasure01, kmeasureB01              ! k-measureB01 is the counterfactual measure of quality given demand in period 0. k-measure01  is the hypothetical change in sales, given demand in period 0 (see counterEL) 
        real(8) :: sekm0
        real(8), dimension(sizeq) :: zbarq0
        real(8), dimension(sizef) :: outprice0
                
        ! parameters to estimate factors augmenting foreign prices, foreign demand shifter and domestic consumer income; wfactor changes weight of firms for free entry counterfactual
        real(8) :: pfactor, yfactor, ycfactor, wfactor
        real(8) :: wl0, wh0
        real(8) :: wl1, wh1
        real(8) :: Lh0, Ll0     ! aggregate labor demand, initial
        real(8) :: Lh1, Ll1 
        real(8) :: allsales0    ! total sales targetted under calcobjce
        real(8) :: womega0      ! firm weight
        real(8) :: eprofit0     ! expected profit
        
        ! data>aggregates (Feenstra numbers)
        real(8), parameter :: impp_data01 = .119d0             ! change in import/total sales in domestic market (aggreg_all)
        real(8), parameter :: expe_data01 = .03d0              ! change in exports/total sales in domestic market (aggreg_all)
        
        !alternative 2: export expansion
        !real(8), parameter :: expe_data01 = impp_data01       ! change in exports/total sales in domestic market (aggreg_all)
                
        real(8), parameter:: tariff1 = 0.12                    ! tariffs in 1994 are the same as submission, not in new data tables     
        
        ! Updated from "new baseline jan27"
        real(8), dimension(5), parameter:: psize_data01  = (/ -.16, -.10, -.06, 0.04, -0.08  /)               ! ln(domestic sales) distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(5), parameter:: pskill_data01 = (/ 0.032, 0.042, 0.060, 0.092, 0.137/)             ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), dimension(7), parameter:: pkmeas_data01 = (/-0.11, 0.32,0.52, 0.79, 1.09, 1.38, 1.59/)             ! skill intensity distribution of plants 10, 25, 50, 75, 90 percentiles
        real(8), parameter :: nfirms_data1 = 7304.d0                                                          ! number of plants (in March 8 moments)
        real(8), parameter :: aggimps_data0 = 0.162d0, aggimps_data1 = 0.281d0, aggexps_data0 = 0.045, aggexps_data1 = 0.075d0
        real(8) , parameter:: aggskm_data0 = 0.290d0, aggskm_data1 = 0.354d0                                  ! aggregate skill intensity
        
        
    end module counterfactual
    ! ----------------------------------------------------------------------------
	module autarky
        use parameters_model
        real(8), dimension(sizeq) :: nqa, zqa                  		! zq=1/aq is the productivity of a representative firm (in the Melitz sense) of quality q.
		real(8), dimension(sizeq) :: rfirmpa                          ! price component for each firm type; representative firm p
		
    end module autarky
    
    ! =============================================================================================
    program modelsi_nufx
    ! =============================================================================================
    INCLUDE 'link_fnl_static.h'
    use numerical_libraries
    use parameters_polytope1
    use parameters_sanneal
    use parameters_estimate
    use parameters_quality
    use counterfactual
    
    use parameters_qf
    implicit none
    
    ! Variables
    ! =============================================================================================
   	real(8), dimension(n) :: xinitial, xout
    real(8) :: xinit1(n1), xinit2(n2), xinit3(n3)
    real(8) :: obj1, obj2, obj3
    logical :: succ
    integer(4), parameter :: maxguess = 10          ! repeat optimization
    integer(4) :: nguess
    integer(4) :: j
    logical :: myflag
    integer :: startime, endtime, hz
	
    ! results from optimization algorithms
    real(8), dimension(n) :: xmin
    real(8), dimension(n1, np1) :: v1
	real(8), dimension(np1) :: fv1
    real(8), dimension(n2, n2+1) :: v2 
	real(8), dimension(n2+1) :: fv2
    real(8), dimension(n3, n3+1) :: v3
	real(8), dimension(n3+1) :: fv3
    real(8), dimension(n, n+1) :: v
	real(8), dimension(n+1) :: fv
    real(8) :: fmin, obj                                 ! function evaluated at minimizer
    logical :: smin                                 ! success

    ! counterfactual
    real(8), dimension(nc) :: xcounter
	real(8), dimension(nc, nc+1) :: vc
	real(8), dimension(nc+1) :: fvc
    
    real(8), dimension(nci) :: xcounteri
	real(8), dimension(nci, nci+1) :: vci
	real(8), dimension(nci+1) :: fvci
    
    real(8), dimension(nce) :: xcountere
	real(8), dimension(nce, nce+1) :: vce
	real(8), dimension(nce+1) :: fvce
    
    real(8), dimension(ncei) :: xcounterei
	real(8), dimension(ncei, ncei+1) :: vcei
	real(8), dimension(ncei+1) :: fvcei
    ! =============================================================================================
    !CALL SYSTEM_CLOCK(COUNT_RATE=hz); CALL SYSTEM_CLOCK(COUNT=startime)     ! start clock

    ! preliminaries: get weights, get random variables, construct grid
    ! ----------------------------------------------------------------
    call read_weights
    call read_random
    call gen_qgrid
    wq = 1.d0        ! Initial labor costs are normalized to one. In counterfactual with inelastic wages, it changes
    womega = 1.d0/sizef     
    wl1 = 1.d0
    
    ! get initial guess for parameters, equilibrium prices, chi and firm choices
    ! ----------------------------------------------------------------
    intermediatechi = 1.d0; priceqh = 1.d0
    myflag = .true.
    call read_initial(xinitial, myflag)
    call xtoguessall(xinitial)
    !mu1 = -0.12d0
    !sigma1 = 0.70d0**2.
    !sigma2 = 0.0049d0
    !z3 = -0.1d0
	!f1 = 0.0025d0
    !f2 = 0.00011d0
    !fm = -3.6d0
    !fmvar = 1.4d0**2.
    !fx = -0.88d0    
    !fxvar = 1.8d0**2.
    !l1 = -15.d0
    !l2 = 2.8d0
    !l3 = 0.61d0
    !skpremium = 2.45d0
   ! qf = 12.d0
    !yf = 0.11d0
    !qcf = 3.68d0
    !varpi = 20.d0
   ! nu = 1.05d0
    !sigma2 = 0.0051d0
    !l1 = -5.d0
    !f1 = 0.0015
   ! sigma2 = 0.007
   !fx = -.5
   !fxvar = 4.
   !qcf = 3.6d0/nu
   !mu1 = -0.07d0
   !nu = 1.05d0
   !f2 = 1.e-5
   !qf = 12.5d0
   !yf = 0.034d0
    call guesstoxall(xinitial) 
    call xallto123(xinitial, xinit1, xinit2, xinit3)
    xmin1 = xinit1; xmin2 = xinit2; xmin3 = xinit3
    
    ! calculate objective function
    ! ----------------------------------------------------------------
    toprint = .true.
    omin1 = 0.d0; omin2 = 0.d0; omin3 = 0.d0
    call calc_obj1(xinit1, omin1, succ)
    call calc_obj2(xinit2, omin2)
    call calc_obj3(xinit3, omin3)
    print *, ' '
    print *, ' ---------------------------------------------------------'
    print *, 'objective all = ', omin1 + omin2 + omin3
    print *, 'objective 1+3 = ', omin1 + omin3
    print *, ' ---------------------------------------------------------'
    
    objtt = omin1 + omin3
    xtt = xinitial
    
    fmin = omin1 + omin2 + omin3
    !call calc_obj1p23A(xinit1, xout, fmin, succ); xinitial = xout
    call print_result(xinitial, fmin, succ, .true.)
    call decomposey
    call print_csv
    call outofsample
    call print_parameters
    !stop
    ! ==========================================================
    ! optimization routines
    ! ==========================================================
          
    if (.false.) then
        !toprint2 = .true.
        !call calc_obj1p23(xinit1, xout, fmin, succ)
        toprint = .false.
        toprint2 = .false.      ! print output in polytopes 2 and 3
        print *, ' -----------------------------------------------------------------------'
        print *, "One-stage estimation, with calc_obj1p23"
        print *, ' -----------------------------------------------------------------------'
        print *, '  '
        pflag = .false.
        call polytope1p23(xinit1, v1, fv1); xmin1 = v1(:, 1); xinit1 = v1(:, 1); xinit1 = xmin1; omin1 = fv1(1)
        call sanneal1p23(xinit1, xmin1, omin1, smin); xinit1 = xmin1
        call polytope1p23(xinit1, v1, fv1); xmin1 = v1(:, 1); xinit1 = v1(:, 1); xinit1 = xmin1; omin1 = fv1(1)
        
        call calc_obj1p23A(xinit1, xout, fmin, succ)
        call print_result(xout, fmin, succ, .false.)
    
        stop
    end if
    
if (.false.) then   ! outer optimization if
    toprint = .false.
    if (.true.) then
        print *, ' -----------------------------------------------------------------------'
        print *, "stage 1"
        print *, ' -----------------------------------------------------------------------'
        print *, '  '
        pflag = .false.
        call polytope1(xinit1, v1, fv1); xmin1 = v1(:, 1); xinit1 = v1(:, 1); xinit1 = xmin1; omin1 = fv1(1)
        call sanneal1(xinit1, xmin1, omin1, smin); xinit1 = xmin1
        call polytope1(xinit1, v1, fv1); xmin1 = v1(:, 1); xinit1 = v1(:, 1); xinit1 = xmin1; omin1 = fv1(1)
    end if
    
    if (.true.) then
        print *, ' -----------------------------------------------------------------------'
        print *, "stage 2"
        print *, ' -----------------------------------------------------------------------'
        print *, '  '
        pflag = .false.
        call polytope2(xinit2, v2, fv2); xmin2 = v2(:, 1); xinit2 = xmin2; omin2 = fv2(1)
        !call sanneal2(xinit2, xmin2, omin2); xinit2 = xmin2
        !call polytope2(xinit2, v2, fv2); xmin2 = v2(:, 1); xinit2 = xmin2; omin2 = fv2(1)
    end if
    
    if (.true.) then
        print *, ' -----------------------------------------------------------------------'
        print *, "stage 3"
        print *, ' -----------------------------------------------------------------------'
        print *, '  '
        pflag = .false.
        call polytope3(xinit3, v3, fv3); xmin3 = v3(:, 1); xinit3 = xmin3; omin3 = fv3(1)
        !call sanneal3(xinit3, xmin3, omin3); xinit3 = xmin3
        !call polytope3(xinit3, v3, fv3); xmin3 = v3(:, 1); xinit3 = xmin3; omin3 = fv3(1)
    end if
    
    call x123toall(xinitial, xmin1, xmin2, xmin3)
    fmin = omin1 + omin2 + omin3
    call print_result(xinitial, fmin, succ, .true.)
    
    toprint = .true.
    call calc_obj1(xmin1, omin1, succ)
    call calc_obj2(xmin2, omin2)
    call calc_obj3(xmin3, omin3)
    print *, ' '
    print *, ' ---------------------------------------------------------'
    print *, 'objective all = ', omin1 + omin2 + omin3
    print *, ' ---------------------------------------------------------'
    call print_csv
    call decomposey
    call outofsample
    stop
end if   ! optimization outer if
    
    ! =======================================================================
    ! standard errors
    ! =======================================================================
    
    if (.false.) then
        toprint = .false.
        !call stderror1s(xinitial)       ! optimal weighting matrix
        call stderror1sw(xinitial)       ! identity matrix
        stop
    end if
    ! =======================================================================
    ! counterfactual -- elastic labor
    ! =======================================================================
   
    if (.true.) then
        toprint = .false.
        call calc_obj1(xmin1, omin1, succ)
        call calc_obj2(xmin2, omin2)
        call calc_obj3(xmin3, omin3)
        
        print *, ' -----------------------------------------------------------------------'
        print *, "COUNTERFACTUAL, ELASTIC LABOR"
        print *, ' -----------------------------------------------------------------------'
        print *, 'original obj', omin1+omin2+omin3, succ
        print *, '  '
        toprint = .true.
        call save_eq0
        ! decompose income into labor income, fixed costs, tariff transfers and deficit
                
        ! initial guess
        ! -----------------------------------------------
        call read_counter(xcounter)
        call xtoguessc(xcounter)
        !yfactor = 0.95
        !pfactor = 0.95
        call guesstoxc(xcounter)
        print *, 'pfactor =', pfactor
        print *, 'yfactor =', yfactor
        print *, ' '
        call calc_objc(xcounter, obj, succ)
        call print_counter(xcounter, obj, succ)
        call decomposey
        stop
        toprint = .false.
        call polytopec(xcounter, vc, fvc); xcounter = vc(:, 1); obj = fvc(1)
        toprint = .true.
        call calc_objc(xcounter, obj, succ)
        call print_counter(xcounter, obj, succ)
        stop
    end if
        
    ! =======================================================================
    ! counterfactual -- elastic labor, no quality choice
    ! =======================================================================
    if (.false.) then
        toprint = .false.
        call calc_obj1(xmin1, omin1, succ)
        call calc_obj2(xmin2, omin2)
        call calc_obj3(xmin3, omin3)
        
        print *, ' -----------------------------------------------------------------------'
        print *, "COUNTERFACTUAL, ELASTIC LABOR, NO QUALITY CHOICE"
        print *, ' -----------------------------------------------------------------------'
        print *, 'original obj', omin1+omin2+omin3, succ
        print *, '  '
        toprint = .true.
        call save_eq0
        
        ! initial guess
        ! -----------------------------------------------
        call read_counter(xcounter)
        call xtoguessc(xcounter)
        !pfactor = .95d0
        !yfactor = 1.0d0
        call guesstoxc(xcounter)
        print *, 'pfactor =', pfactor
        print *, 'yfactor =', yfactor
        print *, ' '
        call calc_objcnq(xcounter, obj, succ)
        call print_counter(xcounter, obj, succ)
        call decomposey
        stop
        toprint = .false.
        call polytopecnq(xcounter, vc, fvc); xcounter = vc(:, 1); obj = fvc(1)
        stop
    end if
    
    ! =======================================================================
    ! counterfactual -- inelastic labor
    ! =======================================================================
    if (.true.) then
        toprint = .false.
        call calc_obj1(xmin1, omin1, succ)
        call calc_obj2(xmin2, omin2)
        call calc_obj3(xmin3, omin3)
        
        print *, ' -----------------------------------------------------------------------'
        print *, "COUNTERFACTUAL, INELASTIC LABOR "
        print *, ' -----------------------------------------------------------------------'
        print *, 'original obj', omin1+omin2+omin3, succ
        print *, '  '
        toprint = .true.
        call save_eq0
        
        ! initial guess
        ! -----------------------------------------------
        call read_counteri(xcounteri)
        call xtoguessci(xcounteri)
        !pfactor = .95d0
        !yfactor = 1.02d0
        !wl1 = 0.98d0
        !wh1
        call guesstoxci(xcounteri)
        print *, 'pfactor =', pfactor
        print *, 'yfactor =', yfactor
        print *, 'wl1 =', wl1
        print *, 'wh1 =', wh1
        print *, 'wh/wl1 = ', wh1/wl1
        print *, ' '
        !fxomega = 0.85*fxomega; fmomega = 0.85*fmomega; fxq = 0.85*fxq; ! fixed costs change approx. with wages
        call calc_objci(xcounteri, obj, succ)
        call decomposey
        stop
        toprint = .false.
        call polytopeci(xcounteri, vci, fvci); xcounteri = vci(:, 1); obj = fvci(1)
        toprint = .true.
        call calc_objci(xcounteri, obj, succ)
        print *, 'skpremium 1/0-1', skpremium/skpremium0-1.
        stop
    end if
    
    ! =======================================================================
    ! counterfactual -- inelastic labor, no quality choice
    ! =======================================================================
    if (.false.) then
        toprint = .false.
        call calc_obj1(xmin1, omin1, succ)
        call calc_obj2(xmin2, omin2)
        call calc_obj3(xmin3, omin3)
        
        print *, ' -----------------------------------------------------------------------'
        print *, "COUNTERFACTUAL, INELASTIC LABOR, NO QUALITY CHOICE "
        print *, ' -----------------------------------------------------------------------'
        print *, 'original obj', omin1+omin2+omin3, succ
        print *, '  '
        toprint = .true.
        call save_eq0
        
        ! initial guess
        ! -----------------------------------------------
        call read_counteri(xcounteri)
        call xtoguessci(xcounteri)
        !pfactor = .95d0
        !yfactor = 1.0d0
        !wl1 = 0.98d0
        !wh1
        call guesstoxci(xcounteri)
        print *, 'pfactor =', pfactor
        print *, 'yfactor =', yfactor
        print *, 'wl1 =', wl1
        print *, 'wh1 =', wh1
        print *, ' '
        call calc_objcinq(xcounteri, obj, succ)
        call decomposey
        stop
        toprint = .false.
        call polytopecinq(xcounteri, vci, fvci); xcounteri = vci(:, 1); obj = fvci(1)
        print *, 'skpremium 1/0-1', skpremium/skpremium0-1.
        stop
    end if
       
    end program modelsi_nufx
    
    