subroutine calc_obj1p23A(x, xout, obj, succ)
    use parameters_model
    use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(n1), intent(in) :: x
    real(8), intent(out) :: obj
    real(8), dimension(n), intent(out) :: xout
    logical, intent(out) :: succ
    real(8), dimension(n) :: xall
    integer(4) :: j
    
    real(8), dimension(n2, n2+1) :: v2 
	real(8), dimension(n2+1) :: fv2
    real(8), dimension(n3, n3+1) :: v3
	real(8), dimension(n3+1) :: fv3
    
    real(8) :: obj2, obj3, rd
    logical :: templ
    ! ======================================================================================
    ! construct primitives from the economy--independent of equilibrium outcomes
    ! -----------------------------------------------------------------------------------
    succ = .false.
    templ = toprint
    
    ! find equilibrium and get moments 1
    ! ====================================================
    
    call xtoguess1(x)                           ! recover parameters from x
    if (toprint) call print_parameters
    call gen_xfirms                             ! get firm productivity, fixed import and export cost
    call gen_xquality                           ! get aggregate quality moments, independent of equilibrium
    call equilibrium(succ)                      
    call firmout1                               ! firm output
    call moments1(obj)                          ! moments
    obj = obj                           ! equilibrium obj is a punishment if eq is not found
    
    ! run optimization for moments 2 and 3
    ! ====================================================
    toprint = .false.
    call polytope3(xmin3, v3, fv3); xmin3 = v3(:, 1); obj3 = fv3(1)
    call polytope2(xmin2, v2, fv2); xmin2 = v2(:, 1); obj2 = fv2(1)
    ! randomly improve optimization of 2
    call RANDOM_NUMBER(rd); if (rd > 0.999) then; call sanneal2(xmin2, xmin2, omin2); call polytope2(xmin2, v2, fv2); xmin2 = v2(:, 1); obj2 = fv2(1); end if
    
    ! aggregate and print results
    ! ====================================================
    toprint = templ
    obj = obj + obj2 + obj3
    call x123toall(xout, x, xmin2, xmin3)
    if(succ) call print_result(xout, obj, succ, .true.)
    
    if(toprint) then
        call calc_obj3(xmin3, obj3)
        call calc_obj2(xmin2, obj2)
        print *, 'obj in calc_obj1p23', obj
    end if
    ! randomly print csv
    call RANDOM_NUMBER(rd)
    if(rd>0.95) then
        call print_csv
        !if (succ) call print_result(xout, obj, succ, .true.)
    end if
   
    ! print output if succ
    ! ----------------------------------------------------
    !if ((succ) .and. (obj<.011)) then
    !    call print_result(xout, obj, succ, .false.)
    !    print *, 'algorithm stopped in calc_obj1p23, myresult.txt'
    !    stop
    !end if
    
    !print *, succ, obj, real(nexit)/sizef
    
    if ((succ) .and. (obj < objtt)) then
        print *, 'current min', objtt
        objtt = obj
        xtt = xout
        call print_result(xout, obj, succ, .false.)
    end if
    
end subroutine calc_obj1p23A


subroutine calc_obj1p23(x, xout, obj, succ)
    use parameters_model
    use parameters_firm
    use parameters_quality
    implicit none

	real(8), dimension(n1), intent(in) :: x
    real(8), intent(out) :: obj
    real(8), dimension(n), intent(out) :: xout
    logical, intent(out) :: succ
    real(8), dimension(n) :: xall
    integer(4) :: j
    
    real(8), dimension(n2, n2+1) :: v2 
	real(8), dimension(n2+1) :: fv2
    real(8), dimension(n3, n3+1) :: v3
	real(8), dimension(n3+1) :: fv3
    
    real(8) :: obj2, obj3, rd
    logical :: templ
    ! ======================================================================================
    ! construct primitives from the economy--independent of equilibrium outcomes
    ! -----------------------------------------------------------------------------------
    succ = .false.
    templ = toprint
    
    ! find equilibrium and get moments 1
    ! ====================================================
    
    call xtoguess1(x)                           ! recover parameters from x
    if (toprint) call print_parameters
    call gen_xfirms                             ! get firm productivity, fixed import and export cost
    call gen_xquality                           ! get aggregate quality moments, independent of equilibrium
    call equilibrium(succ)                      
    call firmout1                               ! firm output
    call moments1(obj)                          ! moments
    obj = obj                           ! equilibrium obj is a punishment if eq is not found
    
    ! run optimization for moments 3
    ! ====================================================
    toprint = .false.
    call polytope3(xmin3, v3, fv3); xmin3 = v3(:, 1); obj3 = fv3(1)
    
    ! aggregate and print results
    ! ====================================================
    toprint = templ
    obj = obj + obj3
    call x123toall(xout, x, xmin2, xmin3)
    if(succ) call print_result(xout, obj, succ, .true.)
    
    if(toprint) then
        !call calc_obj2(xmin2, obj2)
        call calc_obj3(xmin3, obj3)
        print *, 'obj in calc_obj1p23', obj
    end if
    ! randomly print csv
    call RANDOM_NUMBER(rd)
    if(rd>0.98) then
        call print_csv
        if (succ) call print_result(xout, obj, succ, .false.)
    end if
   
    ! print output if succ
    ! ----------------------------------------------------
    !if ((succ) .and. (obj<.011)) then
    !    call print_result(xout, obj, succ, .false.)
    !    print *, 'algorithm stopped in calc_obj1p23, myresult.txt'
    !    stop
    !end if
    
    !print *, succ, obj, real(nexit)/sizef
    
end subroutine calc_obj1p23