! ****************************************************************
! PURPOSE: Match moments and calculate objective function
! ****************************************************************

subroutine moments2(obj)
	use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
	implicit none
    
    real(8), intent(out) :: obj
    integer(4), dimension(sizef) :: qsize
    real(8), dimension(sizef) :: var1
    integer(4) :: j, jj, pospi(1)
    real(8) :: pskill_model(5)
    real(8), dimension(4) :: sksize_model, skasize_model
    real(8) :: aux1, skusa_model
    
    ! for aggregate skill intensity
    real(8), dimension(sizef) :: employment, avgwage
    real(8) :: maxska, usavg
    
    ! ==================================================================================
    obj = 0.d0
       
    ! percentiles of home sales & skill intensity (unconditional distributions)
    ! ----------------------------------------------------------------------------------
    call pdistribution(skillm, pskill_model)
    obj = obj + sum(mweight(22:26)*( pskill_model - pskill_data )**2)
    
    if (toprint) then
        print *, 'distribution of skill intensity'
        print *, '10% ', pskill_data(1) , pskill_model(1), mweight(22)*( pskill_model(1) - pskill_data(1) )**2
        print *, '25% ', pskill_data(2) , pskill_model(2), mweight(23)*( pskill_model(2) - pskill_data(2) )**2
        print *, '50% ', pskill_data(3) , pskill_model(3), mweight(24)*( pskill_model(3) - pskill_data(3) )**2
        print *, '75% ', pskill_data(4) , pskill_model(4), mweight(25)*( pskill_model(4) - pskill_data(4) )**2
        print *, '90% ', pskill_data(5) , pskill_model(5), mweight(26)*( pskill_model(5) - pskill_data(5) )**2
        print *, 'ln(90%/10%) ', pskill_data(5) - pskill_data(1), pskill_model(5) - pskill_model(1)
        print *, 'contribution to obj = ', sum(mweight(22:26)*( pskill_model - pskill_data )**2)
        print *, ' ' 
    end if
         
    ! for each size quartile... qsize
    ! -----------------------------------------------
    var1 = log(nhsales)
    call quartiles(var1, qsize)
    do j = 1, 4
        var1 = 0.d0
        where (qsize == j) var1 = 1.d0
        sksize_model(j) = sum( var1*skillm )/sum(var1)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    end do
    obj = obj + sum(mweight(27:30)*(sksize_model - sksize_data)**2)
    
    if (toprint) then    
        print *, 'skill intensity by quartiles of size'
        print *, '1 ', sksize_data(1), sksize_model(1), mweight(27)*(sksize_data(1) - sksize_model(1))**2
        print *, '2 ', sksize_data(2), sksize_model(2), mweight(28)*(sksize_data(2) - sksize_model(2))**2
        print *, '3 ', sksize_data(3), sksize_model(3), mweight(29)*(sksize_data(3) - sksize_model(3))**2
        print *, '4 ', sksize_data(4), sksize_model(4), mweight(30)*(sksize_data(4) - sksize_model(4))**2
        print *, 'total contribution to obj', sum(mweight(27:30)*(sksize_model - sksize_data)**2)
        print *, " "
        
        ! actual skill intensity
        do j = 1, 4
            var1 = 0.d0
            where (qsize == j) var1 = 1.d0
            skasize_model(j) = sum( var1*skilla )/sum(var1)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
        end do
        print *, 'actual skill intensity by quartiles of size'
        print *, '1 ', skasize_model(1)
        print *, '2 ', skasize_model(2)
        print *, '3 ', skasize_model(3)
        print *, '4 ', skasize_model(4)
        print *, " "
        
    end if
    
    ! measured skill premium
    ! -----------------------------------------------
    obj = obj + mweight(47)*(aggskpm - skpremium_data)**2.
    obj = obj + mweight(53)*(aggskm - skshare_data)**2.
    if (toprint) then
        print *, 'skill premium & intensity'
        print *, 'prem', skpremium_data, aggskpm, mweight(47)*(aggskpm - skpremium_data)**2.
        print *, 'int. ', skshare_data, aggskm, mweight(53)*(aggskm - skshare_data)**2.
        print *, ' ' 
    end if
    
    ! skill intensity in USA at qf
    ! -----------------------------------------------
    aux1 = l3*exp(l1+l2*qf)/(1.+exp(l1+l2*qf))
    aux1 = aux1/(1.-aux1)                         ! Phi_L(skill)/Phi_L(unskill)
    !aux1 = (USAskpremium)**(-sigmal)*aux1
    aux1 = (skpremium)**(-sigmal)*aux1
    skUSA_model = aux1/(1.+aux1)
    obj = obj + mweight(50)*(skUSA_model - USAskint)**2.
    if (toprint) then
        print *, 'USA skill intensity'
        print *, USAskint, skUSA_model, mweight(50)*(skUSA_model - USAskint)**2.
    end if
    
    ! difference between US and maximum Colombian skill intensity - to get spread in skills
    call pdistribution(skilla, pskill_model) 
    maxska = pskill_model(5)    ! 90th percentile firm
    usavg = l3/(1.-l3)*skpremium**(-sigmal)     ! upper bound at least 2% higher
    usavg = usavg/(1.+usavg)
    obj = obj + mweight(50)*min(0.,( usavg - maxska ) - 0.08)**2.d0
    
    if (toprint) then
        print *, 'Colombian (90%) x USA skill intensity (avg)' !, l2, mweight(50)*max(0.,l2 - 4.3)**2./100.
        print *, maxska, usavg, mweight(50)*min(0.,( usavg - maxska ) - 0.08)**2.d0
    end if
    
    if (toprint) then
        print *, ' '
        print *, ' ------------------------------------------------------------'
        print *, ' objective skills = ', obj
        print *, ' ------------------------------------------------------------'
    end if
    
end subroutine moments2
