! ****************************************************************
! PURPOSE: Match moments and calculate objective function
! ****************************************************************

subroutine moments3(obj)
	use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
	implicit none
    
    real(8), intent(out) :: obj
    integer(4) :: j, jj, pospi(1)
    
    ! price regressions
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector, yvector2
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: incoeff, outcoeff       ! regression results: coefficients
    
    real(8) :: pkmeasure_model(7), ksize_model(4)
    integer(4), dimension(sizef) :: auxv, qsize
    
    ! ==================================================================================
    ! price regressions
    ! ----------------------------------------------------------------------------------
    obj = 0.d0
    tobs = sizef - nexit
    allocate(yvector(tobs), yvector2(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! first regression: output prices on measured skill intensity
    ! --------------------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            yvector2(jj) = log(inprice(j))
            xmat(jj,2) = kmeasure(j)
        end if
    end do
    ! because I get an error that prices are too high, demean yvectors
    yvector = yvector - sum(yvector)/real(tobs)
    yvector2 = yvector2 - sum(yvector2)/real(tobs)
    
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols_novar(tobs, nvar, yvector, xmat, outcoeff)      ! output prices on skill intensity
    call ols_novar(tobs, nvar, yvector2, xmat, incoeff)      ! input  prices on skill intensity
    obj = obj + mweight(48)*(outcoeff(2) - kimpai_pdata(1))**2. + mweight(49)*(incoeff(2) - kimpai_pdata(2))**2.
    if (toprint) then
        print*, 'coefficient, ln(unit price) on (kmeasure)'
        print*, 'output', kimpai_pdata(1), outcoeff(2),  mweight(48)*(outcoeff(2) - kimpai_pdata(1))**2.
        print*, 'input ', kimpai_pdata(2), incoeff(2) ,  mweight(49)*(incoeff(2)  - kimpai_pdata(2))**2.
        print *, ' '
    end if    
        
    ! unconditional distribution of k-measures
    ! --------------------------------------------------------------------------------
    call pdistribution2(kmeasure, pkmeasure_model)   
    obj = obj + 0.1*sum( (pkimp_data(2:6) - pkmeasure_model(2:6) )**2 )
    
    if (toprint) then
        print *, 'distribution of kmeasures, data, model, obj'
        print *, ' 5% ', pkimp_data(1), pkmeasure_model(1)
        print *, '10% ', pkimp_data(2), pkmeasure_model(2),  0.1*(pkimp_data(2) - pkmeasure_model(2) )**2
        print *, '25% ', pkimp_data(3), pkmeasure_model(3),  0.1*(pkimp_data(3) - pkmeasure_model(3) )**2
        print *, '50% ', pkimp_data(4), pkmeasure_model(4),  0.1*(pkimp_data(4) - pkmeasure_model(4) )**2
        print *, '75% ', pkimp_data(5), pkmeasure_model(5),  0.1*(pkimp_data(5) - pkmeasure_model(5) )**2
        print *, '90% ', pkimp_data(6), pkmeasure_model(6),  0.1*(pkimp_data(6) - pkmeasure_model(6) )**2
        print *, '95% ', pkimp_data(7), pkmeasure_model(7)
        print *, 'total obj',  0.1*sum( (pkimp_data(2:6) - pkmeasure_model(2:6) )**2 )
        print *, ' '
    end if
    
    ! distribution of k-measures conditional on domestic sales
    ! --------------------------------------------------------------------------------
    ! khandelwal measure by quartile of firm domestic sales
    call quartiles(log(nhsales), qsize)
    do j = 1, 4
        auxv = 0
        where (qsize == j) auxv = 1
        ksize_model(j) = sum( auxv*kmeasure )/sum(auxv)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    end do
    obj = obj + 0.1*sum( (kimp_ai - ksize_model)**2. )
        
    if (toprint) then
    print *, 'khandelwal measure of quality by quartiles of size (model only, data in excell)'
    print *, '1 ', kimp_ai(1), ksize_model(1), 0.1*(kimp_ai(1) - ksize_model(1))**2.
    print *, '2 ', kimp_ai(2), ksize_model(2), 0.1*(kimp_ai(2) - ksize_model(2))**2.
    print *, '3 ', kimp_ai(3), ksize_model(3), 0.1*(kimp_ai(3) - ksize_model(3))**2.
    print *, '4 ', kimp_ai(4), ksize_model(4), 0.1*(kimp_ai(4) - ksize_model(4))**2.
    print *, 'total obj',  0.1*sum( (kimp_ai - ksize_model)**2. )
    print *, ' '
    end if
    
    if (toprint) then
        print *, ' moments 3 = ', obj
        print *, ' ------------------------------------------------------------'
    end if
    deallocate(yvector, yvector2, xmat)
    
end subroutine moments3
