
subroutine  print_result(x, obj, succ, istemp)
    use parameters_quality
    use parameters_firm
    implicit none

    logical, intent(in) :: istemp, succ
	real(8), dimension(n), intent(in) :: x
    real(8), intent(in) :: obj

	! internal variables
	integer(4) :: j
	real(8) :: aux

	! ----------------------------------------------------
    ! record equilibrium posqomega, bigpq, chiq
    if (istemp) then
	    open(1, FILE='tempo.txt', STATUS='REPLACE')
    else
        open(1, FILE='myresult.txt', STATUS='REPLACE')
    end if
    
    write(1,*), obj
    write(1,*), succ
	do j = 1,n
         write(1,*), x(j)
    end do
	do j = 1, sizef
		write(1,*), posqomega(j)
	end do
	do j = 1, sizef
		write(1,*), discreted(j)
	end do
	do j = 1, sizeq
		write(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		write(1,*), intermediatechi(j)
    end do
    
	close(1)

end subroutine  print_result

! ****************************************************************************
!	PURPOSE: print output of program to .csv files to be read in stata
! ****************************************************************************
subroutine  print_csv
    use parameters_estimate
	use parameters_qf
    use crosssec
    use parameters_quality
    use random_draw
	implicit none

	integer(4) :: j, j1

	! economy wide characteristics
	! ----------------------------------------------------------
	open(2, FILE='quality1.csv', STATUS='REPLACE')	
    write(2,*), "quality, nq, zq"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", sum(nq(j,:)), ", ", sum(zq(j,:)**(mysigma-1.))**(1./(mysigma-1.))
		end do
	close(2)
    
    open(2, FILE='quality2.csv', STATUS='REPLACE')
		write(2,*), "quality, zbar, skillq"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", zbarq(j), ", ", skillq(j)
		end do
	close(2)
 	    
    open(2, FILE='quality3.csv', STATUS='REPLACE')	
    write(2,*), "quality, nq1, nq2"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", nq(j,1), ", ", nq(j,2)
		end do
	close(2)
    
    open(2, FILE='quality4.csv', STATUS='REPLACE')	
    write(2,*), "quality, nq3, nq4"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", nq(j,3), ", ", nq(j,4)
		end do
	close(2)
    
    
    open(2, FILE='quality5.csv', STATUS='REPLACE')	
    write(2,*), "quality, priceqa, priceqh"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", priceqa(j), ", ", priceqh(j)
		end do
	close(2)
    
    
    open(2, FILE='quality6.csv', STATUS='REPLACE')	
    write(2,*), "quality, chiq, chiqf"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", chiq(j), ", ", chiqf(j)
		end do
	close(2)
    
	! firm-specific characteristics
	! ----------------------------------------------------------
    ! discrete choices
    open(2, FILE='firm1.csv', STATUS='REPLACE')
		write(2,*), "firm, posqomega, discreted"
	 	do j = 1, sizef
			    write(2,*), j, ", ", posqomega(j), ", ", discreted(j)
		end do
	close(2)
    
	! input and output quality
	open(2, FILE='firm2.csv', STATUS='REPLACE')
		write(2,*), "firm, hsales, quality"
	 	do j = 1, sizef
			if (posqomega(j) < 0) then
				write(2,*), j, ", ,"
			else 
				write(2,*), j, ", ", nhsales(j), ", ", qgrid(posqomega(j))
			end if
		end do
	close(2)
    
	! skill intensity
	open(2, FILE='firm3.csv', STATUS='REPLACE')
		write(2,*), "firm, skilla, skillm"
	 	do j = 1, sizef
			if (posqomega(j) < 0) then
				write(2,*), j, ", , "
			else 
				write(2,*), j, ", ", skilla(j), ", ", skillm(j)
			end if
		end do
	close(2)
    
	! skill intensity
	open(2, FILE='firm4.csv', STATUS='REPLACE')
		write(2,*), "firm, z1, z2"
	 	do j = 1, sizef
			if (posqomega(j) < 0) then
				write(2,*), j, ", , "
			else 
				write(2,*), j, ", ", random_firms(j,1), ", ", random_firms(j,2)
			end if
		end do
	close(2)
end subroutine print_csv

! =====================================================================================
subroutine  print_counter(x, obj, succ)
    use parameters_quality
    use parameters_firm
    implicit none

    logical, intent(in) :: succ
	real(8), dimension(nc), intent(in) :: x
    real(8), intent(in) :: obj

	! internal variables
	integer(4) :: j
	real(8) :: aux

	! ----------------------------------------------------
    ! record equilibrium posqomega, bigpq, chiq
    
    open(1, FILE='tempoc.txt', STATUS='REPLACE')
    
    
    write(1,*), obj
    write(1,*), succ
	do j = 1,nc
         write(1,*), x(j)
    end do
	do j = 1, sizef
		write(1,*), posqomega(j)
	end do
	do j = 1, sizef
		write(1,*), discreted(j)
	end do
	do j = 1, sizeq
		write(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		write(1,*), intermediatechi(j)
    end do
    
	close(1)

end subroutine  print_counter


! ****************************************************************************
!	PURPOSE: print output of program to .csv files to be read in stata
! ****************************************************************************
subroutine  print_csv01
    use counterfactual
    use crosssec
    use counterfactual
	implicit none

	integer(4) :: j, j1
    real(8), dimension(sizef) :: ntsales

	! economy wide characteristics
	! ----------------------------------------------------------
	open(2, FILE='counterq1.csv', STATUS='REPLACE')	
    write(2,*), "quality, nq0, nq1"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", sum(nq0(j,:)), ", ", sum(nq(j,:))
		end do
	close(2)
    
    open(2, FILE='counterq2.csv', STATUS='REPLACE')	
    write(2,*), "quality, priceqh0, priceqh1"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", priceqh0(j), ", ", priceqh(j)
		end do
	close(2)
    
    open(2, FILE='counterq3.csv', STATUS='REPLACE')	
    write(2,*), "quality, priceqa0, priceqa1"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", priceqa0(j), ", ", priceqa(j)
		end do
	close(2)       
    
    open(2, FILE='counterq4.csv', STATUS='REPLACE')
		write(2,*), "quality, skillq"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", skillq(j)
		end do
	close(2)
    
    open(2, FILE='counterq5.csv', STATUS='REPLACE')	
    write(2,*), "quality, chiq0, chiq1"
	 	do j = 1, sizeq
			write(2,*), qgrid(j), ", ", chiq0(j), ", ", chiq(j)
		end do
	close(2)
    
    ! firm-specific characteristics
	! ----------------------------------------------------------
    ! discrete choices
    open(2, FILE='counterf1.csv', STATUS='REPLACE')
		write(2,*), "firm, discreted0, discreted1"
	 	do j = 1, sizef
			    write(2,*), j, ", ", discreted0(j), ", ", discreted(j)
		end do
	close(2)
    
	!total sales
    ntsales = nhsales/(1. - exp_share)
	open(2, FILE='counterf2.csv', STATUS='REPLACE')
		write(2,*), "firm, ntsales0, ntsales1"
	 	do j = 1, sizef
			if ((posqomega0(j) < 0) .and. (posqomega(j) < 0)) then
				write(2,*), j, ", , "
            else if (posqomega0(j) < 0) then
			    write(2,*), j, ", , ", ntsales(j)
            else if (posqomega(j) < 0) then
			    write(2,*), j, ", ", ntsales0(j), ", "
			else 
				write(2,*), j, ", ", ntsales0(j), ", ", ntsales(j)
			end if
		end do
	close(2)
    
	!total sales
    ntsales = nhsales/(1. - exp_share)
	open(2, FILE='counterf3.csv', STATUS='REPLACE')
		write(2,*), "firm, tsales0, tsales1"
	 	do j = 1, sizef
			if ((posqomega0(j) < 0) .and. (posqomega(j) < 0)) then
				write(2,*), j, ", , "
            else if (posqomega0(j) < 0) then
			    write(2,*), j, ", , ", tsales(j)
            else if (posqomega(j) < 0) then
			    write(2,*), j, ", ", tsales0(j), ", "
			else 
				write(2,*), j, ", ", tsales0(j), ", ", tsales(j)
			end if
		end do
	close(2)
	! skill intensity actual
	open(2, FILE='counterf4.csv', STATUS='REPLACE')
		write(2,*), "firm, skilla0, skilla1"
	 	do j = 1, sizef
			if ((posqomega0(j) < 0) .or. (posqomega(j) < 0)) then
				write(2,*), j, ", , "
            else if (posqomega0(j) < 0) then
			    write(2,*), j, ", , ", skilla(j)
            else if (posqomega(j) < 0) then
			    write(2,*), j, ", ", skilla0(j), ", "
			else 
				write(2,*), j, ", ", skilla0(j), ", ", skilla(j)
			end if
		end do
	close(2)
    
	! skill intensity measured
	open(2, FILE='counterf5.csv', STATUS='REPLACE')
		write(2,*), "firm, skillm0, skillm1"
	 	do j = 1, sizef
			if ((posqomega0(j) < 0) .or. (posqomega(j) < 0)) then
				write(2,*), j, ", , "
            else if (posqomega0(j) < 0) then
			    write(2,*), j, ", , ", skillm(j)
            else if (posqomega(j) < 0) then
			    write(2,*), j, ", ", skillm0(j), ", "
			else 
				write(2,*), j, ", ", skillm0(j), ", ", skillm(j)
			end if
		end do
	close(2)
    
    
	! skill intensity measured
	open(2, FILE='counterf6.csv', STATUS='REPLACE')
		write(2,*), "firm, quality0, quality1"
	 	do j = 1, sizef
			if ((posqomega0(j) < 0) .or. (posqomega(j) < 0)) then
				write(2,*), j, ", , "
            else if (posqomega0(j) < 0) then
			    write(2,*), j, ", , ", qgrid(posqomega(j))
            else if (posqomega(j) < 0) then
			    write(2,*), j, ", ", qgrid(posqomega0(j)), ", "
			else 
				write(2,*), j, ", ", qgrid(posqomega0(j)), ", ", qgrid(posqomega(j))
			end if
		end do
	close(2)
    
    ! employment
    open(2, FILE='counterf7.csv', STATUS='REPLACE')
		write(2,*), "firm, firml0, firml1"
	 	do j = 1, sizef
			if ((posqomega0(j) < 0) .and. (posqomega(j) < 0)) then
				write(2,*), j, ", , "
            else if (posqomega0(j) < 0) then
			    write(2,*), j, ", , ", firml1(j)
            else if (posqomega(j) < 0) then
			    write(2,*), j, ", ", firml0(j), ", "
			else 
				write(2,*), j, ", ", firml0(j), ", ", firml1(j)
            end if
		end do
	close(2)
    
end subroutine print_csv01