!****************************************************************************
!	get profits for all choices of each firm and update its choice
!****************************************************************************

subroutine  profits(njump, nlgjump, ndiscrete, check_all)                          
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
	implicit none

	integer, intent(out) :: njump, nlgjump, ndiscrete
    logical, intent(out) :: check_all
    integer(4), dimension(sizef) :: oldposq, olddiscd
    
    real(8), dimension(sizeq, 4) :: profit
    integer, dimension(2) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8), dimension(sizef) :: fprofit
    
   ! --------------------------------------------------------------------
    njump = 0
    nlgjump = 0
    ndiscrete = 0
    check_all = .true.
    oldposq = posqomega
    olddiscd = discreted
    
    !$OMP PARALLEL shared(posqomega, discreted), private(profit, pos_maxpi)
    !$OMP do
    do j = 1, sizef
        ! calculate profit for firm j
         profit(:, 1) = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)*wq**myalpha/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq
         profit(:, 2) = (1./mysigma)*( ( markup*priceqa**(1.-myalpha)*wq**myalpha/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq - fmomega(j)
         profit(:, 3) = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)*wq**myalpha/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - fxomega(j)
         profit(:, 4) = (1./mysigma)*( ( markup*priceqa**(1.-myalpha)*wq**myalpha/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - (fmomega(j) + fxomega(j))
         
         ! update its choice
         fprofit(j) = maxval(profit)
         pos_maxpi = maxloc(profit)
         posqomega(j) = pos_maxpi(1)
         discreted(j) = pos_maxpi(2)
         
         ! decrease size of jumps if discreted did not change
        if ( (discreted(j) == olddiscd(j)) .and. (abs(oldposq(j) - posqomega(j)) > 1) ) then
            check_all = .false.
            posqomega(j) = ( oldposq(j) + posqomega(j) )/2
        end if 
        
    end do
    !$OMP END do
    !$OMP END PARALLEL
    
    where(fprofit < 0.) 
        posqomega = -1
        discreted = 0
    end where
    
    nlgjump   = count(abs(oldposq - posqomega) > 1) 
    njump     = count(oldposq /= posqomega)
    ndiscrete = count(discreted /= olddiscd)
    
end subroutine profits