! ======================================================================
! ======================================================================
! sanneal.F90      minimizes a function using simulated annealing
! ======================================================================
! ======================================================================

subroutine sanneal3(xinitial, xmin, fmin)
	use parameters_sanneal
	use numerical_libraries 
	implicit none
    
    real(8), dimension(n3), intent(in) :: xinitial
    real(8), dimension(n3), intent(out) :: xmin                  ! result, final minimizer
    real(8), intent(out) :: fmin                                ! function evaluated at minimizer
    
    ! simulated annealing variables
    real(8), dimension(n3) :: xcurrent, xnew
    real(8) :: fcurrent, fnew
    real(8) :: temp                                             ! current temperature
    integer(4) :: itry, itotal, isuccess, iconsec, j            ! counters
    real(8) :: rd
    
    ! body of sanneal
    ! ===================================================================
    xmin = xinitial
    call calc_obj3(xmin, fmin) 
    
    ! reset counters, temperature and initial values
    ! --------------------------------------
    temp = 300.d0
    itry = 1
    itotal = 0
    isuccess = 0
    iconsec = 0
    xcurrent = xmin
    fcurrent = fmin
    
    ! evaluate picked values for delta, governing new guesses' deviations from xcurrent
    ! --------------------------------------
    if(pflag) call getdelta3(xcurrent)
    
    ! --------------------------------------
    sanneal_loop3: do
        if ( (itry >= maxtry) .or. (isuccess >= maxsucc) ) then             ! change temperature or end program
            if ( (temp < stoptemp) .or. (iconsec >= maxconsec) ) then       ! stop program if temperature reaches a minimum or attempts at improving objective function failed
                itotal = itotal + itry
                print *, 'optimization algorithm finished in # tries', itotal
                call print_tsanneal3(xmin, fmin)
                exit sanneal_loop3
            else
                call tcool(temp, temp)                                      ! cool temperature
                itotal = itotal + itry
                itry = 1
                isuccess = 0
                print *, 'temperature cool', itotal, temp, fmin
                call print_tsanneal3(xmin, fmin)
            end if
         end if

         ! draw new random
         call neighborx3(xcurrent, xnew)
         call calc_obj3(xnew, fnew)
         itry = itry + 1
         
         ! new minimum found: move current and save minimum
         if (fnew < fmin) then
            print *, 'd', temp, fmin, fnew
            xmin = xnew
            fmin = fnew
            xcurrent = xnew
            fcurrent = fnew
            iconsec = 0
            call print_tsanneal3(xmin, fmin)
            cycle
          end if

          ! change parameters randomly
          call random_number(rd)
          if ( rd < exp( (fcurrent - fnew)/temp) ) then
            !print *, isuccess, fcurrent, fnew, exp((fcurrent - fnew)/temp)
            xcurrent = xnew
            fcurrent = fnew
            isuccess = isuccess + 1
          else
            iconsec = iconsec + 1
          end if

    end do sanneal_loop3

    ! save results
    call print_sanneal3(xmin, fmin)
    
end subroutine sanneal3

! ======================================================================
! AUXILIARY SUBROUTINES
! ======================================================================

subroutine neighborx3(xcenter, xneighbor)
    use parameters_sanneal
    implicit none 

    real(8), dimension(n3), intent(in) :: xcenter
    real(8), dimension(n3), intent(out) :: xneighbor
    real(8) :: rd
    integer(4) :: j
    ! -------------------------------------------------------
    do j = 1, n3
        call random_number(rd)
        xneighbor(j) = xcenter(j) + (rd - 0.5)*mydelta3(j)*2.
    end do
    
end subroutine neighborx3

! ======================================================================
subroutine getdelta3(xcenter)
    use parameters_sanneal
    implicit none 

    real(8), dimension(n3), intent(in) :: xcenter
    real(8), dimension(n3) :: xneighbor
    real(8) :: rd, obju, objl
    integer(4) :: j, jj
    ! -------------------------------------------------------

    print *, 'j, objl, obju'
    do j = 1, n3
        rd = 1.
        xneighbor = xcenter
        xneighbor(j) = xcenter(j) + (rd - 0.5)*mydelta3(j)*2.
        print *, ' --------------------------- '
        call calc_obj3(xneighbor, obju)
        rd = 0.
        xneighbor = xcenter
        xneighbor(j) = xcenter(j) + (rd - 0.5)*mydelta3(j)*2.
        call calc_obj3(xneighbor, objl)
        print *, j, objl, obju
    end do
    
    print *, ' --------------------------- '
    print *, ' program stopped at getdelta3 routine in sanneal3.f90'
    stop
end subroutine getdelta3


! ======================================================================
subroutine  print_sanneal3(x, obj)
    use parameters_quality
    use parameters_firm
    implicit none

	real(8), dimension(n3), intent(in) :: x
    real(8), intent(in) :: obj
    integer(4) :: j
	real(8) :: aux
    logical, parameter :: auxl = .true.
    real(8), dimension(n) :: xall
    
	! ----------------------------------------------------
    ! record equilibrium posqomega, bigpq, chiq
            
    call x123toall(xall, xmin1, xmin2, x) 
	open(1, FILE='result_sa3.txt', STATUS='REPLACE')
    write(1,*), obj
    write(1,*), auxl
	do j = 1,n
         write(1,*), xall(j)
    end do
	do j = 1, sizef
		write(1,*), posqomega(j)
	end do
	do j = 1, sizef
		write(1,*), discreted(j)
	end do
	do j = 1, sizeq
		write(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		write(1,*), intermediatechi(j)
	end do
	close(1)

end subroutine  print_sanneal3


! ======================================================================
subroutine  print_tsanneal3(x, obj)
    use parameters_quality
    use parameters_firm
    implicit none

	real(8), dimension(n3), intent(in) :: x
    real(8), intent(in) :: obj
    integer(4) :: j
	real(8) :: aux
    logical, parameter :: auxl = .true.
    real(8), dimension(n) :: xall
    
	! ----------------------------------------------------
    ! record equilibrium posqomega, bigpq, chiq
            
    call x123toall(xall, xmin1, xmin2, x) 
	open(1, FILE='result_tsa3.txt', STATUS='REPLACE')
    write(1,*), obj
    write(1,*), auxl
	do j = 1,n
         write(1,*), xall(j)
    end do
	do j = 1, sizef
		write(1,*), posqomega(j)
	end do
	do j = 1, sizef
		write(1,*), discreted(j)
	end do
	do j = 1, sizeq
		write(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		write(1,*), intermediatechi(j)
	end do
	close(1)

end subroutine  print_tsanneal3
