    subroutine stderror1sw(xmin)
    use parameters_model
    use data_moments
    use numerical_libraries    
    use parameters_estimate
    implicit none
    
    real(8), dimension(n), intent(in) :: xmin
    real(8), dimension(n) :: mydelta
    integer(4), parameter :: mm = nweight                           ! number of moments
    real(8), dimension(mm) :: vmoment1, vmoment0                    ! vector with moments
    real(8), dimension(mm,mm) :: wmat                               ! weighting matrix
    real(8), dimension(n, mm) :: dmat, amat                         ! matrix of derivatives and auxiliary
    real(8), dimension(n,n) :: vmat, amatn, amatn2                  ! variance-covariance matrix of parameters
    
    real(8) :: obj, obj1, obj2, obj3, objm, x1(n1), x2(n2), x3(n3), tpar, tx(n)
    logical :: succ
    integer(4):: j
    
    ! ===================================================
    print *, ' ------------------------------------------ '
    print *, ' standard errors'
    
    ! set weighting matrix
    wmat = 0.d0
    do j = 1, nboot-2 ! exclude aggregates
        wmat(j,j) = 1./mweight(j)
    end do
    j = j+2
    ! price moments
    wmat(j,j) = (skill_sepdata(1))**2.
    wmat(j+1,j+1) = (skill_sepdata(2))**2.
    
    ! set deviations, roughly 2% of parameter values
    mydelta = (/ 0.001, 0.02, 0.02, 0.05, 0.05, 0.02, 0.10, 0.02, 0.02, 0.10, 0.10, 0.10, 0.02, 0.02, 0.02, 0.02, 0.02, 0.02 /)
    
    ! calculate objective at minimum
    call xallto123(xmin, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment0, obj)
    objm = obj1 + obj2 + obj3
    print *, 'check obj (gap not included)', objm, (obj - objm)/objm
    print *, ' '
    
    ! deviations for each parameter -- derivatives cannot be taken on x, but on the parameter itself
    print *, 'parameter % change in..., parameter, obj'
    ! mu1 ----------------------------------------------------------
    j = 1
    tx = xmin
    tpar = mu1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(mu1 - tpar)
    print *, 'mu1    ', (mu1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! sigma1 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = sigma1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(sigma1 - tpar)
    print *, 'sigma1 ', (sigma1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! sigma2 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = sigma2 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(sigma2 - tpar)
    print *, 'sigma2 ', (sigma2 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! z3 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = z3 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(z3 - tpar)
    print *, 'z3     ', (z3 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! f1 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = f1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(f1 - tpar)
    print *, 'f1     ', (f1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! f2 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = f2 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(f2 - tpar)
    print *, 'f2     ', (f2 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fm ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fm 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fm - tpar)
    print *, 'fm     ', (fm - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fmvar ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fmvar 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fmvar - tpar)
    print *, 'fmvar  ', (fmvar - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fx ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fx 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fx - tpar)
    print *, 'fx     ', (fx - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! fxvar ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = fxvar 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(fxvar - tpar)
    print *, 'fxvar  ', (fxvar - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! l1 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = l1 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(l1 - tpar)
    print *, 'l1     ', (l1 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! l2 ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = l2 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(l2 - tpar)
    print *, 'l2     ', (l2 - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! skpremium ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = skpremium 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(skpremium - tpar)
    print *, 'skpremium', (skpremium - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! qf ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = qf 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(qf - tpar)
    print *, 'qf     ', (qf - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! yf ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = yf 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(yf - tpar)
    print *, 'yf     ', (yf - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! qcf ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = qcf 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(qcf - tpar)
    print *, 'qcf    ', (qcf - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! varpi ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = varpi 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(varpi - tpar)
    print *, 'varpi ', (varpi - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    
    ! nu ----------------------------------------------------------
    j = j + 1
    tx = xmin
    tpar = nu 
    tx(j) = tx(j) + mydelta(j)
    call xallto123(tx, x1, x2, x3)
    call calc_obj1(x1, obj1, succ); call calc_obj2(x2, obj2); call calc_obj3(x3, obj3)
    call calc_vmoments(mm, vmoment1, obj)
    dmat(j,:) = (vmoment1 - vmoment0)/(nu - tpar)
    print *, 'nu   ', (nu - tpar)/tpar, (obj1 + obj2 + obj3 - objm)/objm
    print *, ' '
    
    
    ! get matrix of variance: (dmat * wmat * dmat')^-1 if weight is (stderror1s)
    
    ! get matrix of variance: (dmat * dmat')^(-1) * (dmat * wmat * dmat') * (dmat * dmat')^(-1) if weight is (stderror1s)
    ! ------------------------------------------------------------
    
    ! center matrix
    call DMRRRR(n, mm, dmat, n, mm, mm, wmat, mm, n, mm, amat, n)    ! amat = dmat * wmat 
    call DMXYTF(n, mm, amat, n,  n, mm, dmat, n , n, n , amatn, n)   ! amatn = amat * dmat'
    
    ! side matrix
    call DMXYTF(n, mm, dmat, n,  n, mm, dmat, n , n, n , amatn2, n)   ! amatn2 = dmat * dmat'
    call DLINRG(n, amatn2, n, vmat, n)                                ! vmat = amatn2 inverse is the side matrix
        
    ! variance
    call DMRRRR(n, n, vmat, n, n, n, amatn, n, n, n, amatn2, n)    ! amatn2 = vmat * amatn
    call DMRRRR(n, n, amatn2, n, n, n, vmat, n, n, n, amatn, n)     ! amatn = amatn2 * vmat
    vmat = amatn  
    
    
    open(2, FILE='stderror1s.csv', STATUS='REPLACE')
    call xtoguessall(xmin) 
    j = 1
    write(2,*), 'mu1,      ', mu1,    ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'sigma1,   ', sigma1, ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'sigma2,   ', sigma2, ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'z3,       ', z3,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'f1,       ', f1,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'f2,       ', f2,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fm,       ', fm,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fmvar,    ', fmvar,  ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fx,       ', fx,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'fxvar,    ', fxvar,  ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'l1,       ', l1,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'l2,       ', l2,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'skpremium,', skpremium, ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'qf,       ', qf,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'yf,       ', yf,     ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'qcf,      ', qcf,    ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'varpi,    ', varpi,  ', ', vmat(j,j)**.5; j = j + 1
    write(2,*), 'nu,       ', nu,     ', ', vmat(j,j)**.5; j = j + 1
    close(2)
    
    print *, 'parameter, value, standard errors'
    j = 1
    print *, 'mu1      ', mu1,    vmat(j,j)**.5; j = j + 1
    print *, 'sigma1   ', sigma1, vmat(j,j)**.5; j = j + 1
    print *, 'sigma2   ', sigma2, vmat(j,j)**.5; j = j + 1
    print *, 'z3       ', z3,     vmat(j,j)**.5; j = j + 1
    print *, 'f1       ', f1,     vmat(j,j)**.5; j = j + 1
    print *, 'f2       ', f2,     vmat(j,j)**.5; j = j + 1
    print *, 'fm       ', fm,     vmat(j,j)**.5; j = j + 1
    print *, 'fmvar    ', fmvar,  vmat(j,j)**.5; j = j + 1
    print *, 'fx       ', fx,     vmat(j,j)**.5; j = j + 1
    print *, 'fxvar    ', fxvar,  vmat(j,j)**.5; j = j + 1
    print *, 'l1       ', l1,     vmat(j,j)**.5; j = j + 1
    print *, 'l2       ', l2,     vmat(j,j)**.5; j = j + 1
    print *, 'skpremium', skpremium, vmat(j,j)**.5; j = j + 1
    print *, 'qf       ', qf,     vmat(j,j)**.5; j = j + 1
    print *, 'yf       ', yf,     vmat(j,j)**.5; j = j + 1
    print *, 'qcf      ', qcf,    vmat(j,j)**.5; j = j + 1
    print *, 'varpi    ', varpi,  vmat(j,j)**.5; j = j + 1
    print *, 'nu       ', nu,     vmat(j,j)**.5; j = j + 1
    print *, ' '
    print *, ' ------------------------------------------ '
    
    end subroutine stderror1sw
    
    
    ! =================================================================
    ! calculates the full vector of moments to get standard errors
    subroutine calc_vmoments(mm, vmoments, obj)
    use parameters_estimate
    use parameters_quality
    use parameters_qf
    use crosssec
    use data_moments
    implicit none
    
    integer(4), intent(in) :: mm
    real(8), dimension(mm), intent(out) :: vmoments
    real(8), intent(out) :: obj
    
    ! Body of calc_vmoments
    integer(4), dimension(sizef) :: qsize, qwage
    real(8), dimension(sizef) :: var1, var2
    real(8), dimension(4,4) :: wquartiles, qsalesw_model
    integer(4) :: j, jj, pospi(1)
    real(8) :: psize_model(5)
    real(8), dimension(4) :: exint_model, imint_model, exshr_model, imshr_model
    real(8) :: pskill_model(5)
    real(8), dimension(4) :: sksize_model, skasize_model
    
    ! for prices
    real(8) :: aux1, skusa_model
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector, yvector2
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: incoeff, outcoeff       ! regression results: coefficients
    real(8) :: pkmeasure_model(7), ksize_model(4)
    integer(4) :: auxv(sizef)
    
    ! ==================================================================================
    obj = 0.d0
    
    ! quartiles of size and wage
    ! ----------------------------------------------------------------------------------
    var1 = log(nhsales)
    var2 = real(posqomega)        ! wage rank coincides with actual skill intensity (and quality)
    wquartiles = reshape(mweight(1:16), shape(qsalesw_data))
    call quartiles(var1, qsize)
    call quartiles(var2, qwage)
    call match_quartile(qwage, qsize, qsalesw_model)
    obj = obj + sum( sum(wquartiles*(qsalesw_model - qsalesw_data)**2,1) )
    vmoments(1:16) = reshape( qsalesw_model - qsalesw_data, shape(vmoments(1:16)) )
    
    ! percentiles of home sales & skill intensity (unconditional distributions)
    ! ----------------------------------------------------------------------------------
    var1 = log(nhsales)       ! normalized domestic sales
    call pdistribution(var1, psize_model)
    obj = obj + sum(mweight(17:21)*( psize_model - psize_data )**2)
    vmoments(17:21) =  psize_model - psize_data 
    
    ! for each size quartile... qsize
    ! -----------------------------------------------
    var1 = log(nhsales)
    call byqsize1(qsize, exint_model, imint_model, exshr_model, imshr_model)
    obj = obj + sum(mweight(31:34)*(exint_model -  exint_data)**2)
    obj = obj + sum(mweight(35:38)*(imint_model - imint_data)**2)
    obj = obj + sum(mweight(39:42)*(exshr_model - exshr_data)**2)
    obj = obj + sum(mweight(43:46)*(imshr_model - imshr_data)**2)
    vmoments(31:34) = exint_model -  exint_data
    vmoments(35:38) = imint_model - imint_data
    vmoments(39:42) = exshr_model - exshr_data
    vmoments(43:46) = imshr_model - imshr_data
    
    ! exit
    ! -----------------------------------------------
    obj = obj + mweight(51)*(exitrate - real(nexit)/real(sizef))**2.
    vmoments(51) = exitrate - real(nexit)/real(sizef)
    
    ! spread of firm quality choices
    ! -----------------------------------------------
    pospi = maxloc(sum(nq,2))
    jj = count( (posqomega == 1) .or. (posqomega == pospi(1)) .or. (posqomega == sizeq) ) + .5*count( ((discreted>0) .and. (posqomega <= 5)) .or. (posqomega >= sizeq - 5) )  + .2*count( (posqomega >= pospi(1) - 2) .and. (posqomega <= pospi(1) + 2) ) 
    obj = obj + mweight(52)*( real(jj)/real(sizef-nexit) )**2
    vmoments(52) =  real(jj)/real(sizef-nexit)
    
    ! ==========================================================
    ! skills - these moments are used only to get std error of the skill parameters
    ! deactivate kmeasure parameters below to do it.
    ! ==========================================================
    ! unconditional distribution
    call pdistribution(skillm, pskill_model)
    obj = obj + sum(mweight(22:26)*( pskill_model - pskill_data )**2)
    vmoments(22:26) = pskill_model - pskill_data
    
    ! for each size quartile... qsize
    var1 = log(nhsales)
    call quartiles(var1, qsize)
    do j = 1, 4
        var1 = 0.d0
        where (qsize == j) var1 = 1.d0
        sksize_model(j) = sum( var1*skillm )/sum(var1)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    end do
    obj = obj + sum(mweight(27:30)*(sksize_model - sksize_data)**2)
    vmoments(27:30) = sksize_model - sksize_data

    ! measured skill premium & aggregate intensity
    obj = obj + mweight(47)*(aggskpm - skpremium_data)**2.
    vmoments(47) = aggskpm - skpremium_data
    obj = obj + mweight(53)*(aggskm - skshare_data)**2.
    vmoments(53) = aggskm - skshare_data
    
    ! skill intensity in USA at qf
    !aux1 = l3*exp(l1+l2*qf)/(1.+exp(l1+l2* qf))
    !aux1 = aux1/(1.-aux1)                         ! Phi_L(skill)/Phi_L(unskill)
    !aux1 = (USAskpremium)**(-sigmal)*aux1
    !skUSA_model = aux1/(1.+aux1)
    !obj = obj + mweight(50)*(skUSA_model - USAskint)**2.
    vmoments(50) = 0.d0
    
    ! =========================================================
    ! price regressions
    tobs = sizef - nexit
    allocate(yvector(tobs), yvector2(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! first regression: output prices on measured skill intensity
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            yvector2(jj) = log(inprice(j))
            xmat(jj,2) = kmeasure(j)
        end if
    end do
    ! because I get an error that prices are too high, demean yvectors
    yvector = yvector - sum(yvector)/real(tobs)
    yvector2 = yvector2 - sum(yvector2)/real(tobs)
    
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see vmoments.f90'
    call ols_novar(tobs, nvar, yvector, xmat, outcoeff)      ! output prices on skill intensity
    call ols_novar(tobs, nvar, yvector2, xmat, incoeff)      ! input  prices on skill intensity
    obj = obj + mweight(48)*(outcoeff(2) - kimpai_pdata(1))**2. + mweight(49)*(incoeff(2) - kimpai_pdata(2))**2.
    
    vmoments(48) = outcoeff(2) - kimpai_pdata(1)
    vmoments(49) = incoeff(2) - kimpai_pdata(2)
    
    !
    !! unconditional distribution of k-measures
    !! --------------------------------------------------------------------------------
    !call pdistribution2(kmeasure, pkmeasure_model)   
    !obj = obj + sum( mweight(22:26)*(pkimp_data(2:6) - pkmeasure_model(2:6) )**2 )
    !vmoments(22:26) = pkimp_data(2:6) - pkmeasure_model(2:6)
    !
    !! distribution of k-measures conditional on domestic sales
    !! --------------------------------------------------------------------------------
    !! khandelwal measure by quartile of firm domestic sales
    !call quartiles(log(nhsales), qsize)
    !do j = 1, 4
    !    auxv = 0
    !    where (qsize == j) auxv = 1
    !    ksize_model(j) = sum( auxv*kmeasure )/sum(auxv)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    !end do
    !obj = obj + sum( mweight(27:30)*(kimp_ai - ksize_model)**2. )
    !    
    !vmoments(27:30) = (kimp_ai - ksize_model)
    !
    
    deallocate(yvector, yvector2, xmat)
    
    
    end subroutine calc_vmoments